unit ClientUnit1;

interface

{ This Project is a perfect Shell for writing
  your own Player for RTC Web Applications.

  Simply include all the units implementing Client-side components
  for RTC WAF and (provided components register themselves) you
  will get a Player capable of executing Web Applications which
  are using these components.

  Naturally, you will also need to configure the 2 RtcHttpClient
  and 1 RtcWafClient component to go to use the correct parameters
  for your RTC Web Application Server. }

uses
  Windows, Messages, SysUtils, Variants, Classes,
  Graphics, Controls, Forms,  Dialogs, StdCtrls,

  rtcCache, rtcLog, rtcThrPool, rtcZLib,

  { rtcMenusCli unit HAS TO BE USED in project
    for RTC WAC Menu components to be compiled into the player.
    But this is a RTC SDK example and the rtcMenusCli unit is in RTC WAC. }
  // rtcMenusCli,

  { rtcStdCtrlsCli unit HAS TO BE USED in project
    for RTC WAC Standard components to be compiled into the player.
    But this is a RTC SDK example and the rtcStdCtrlsCli unit is in RTC WAC. }
  // rtcStdCtrlsCli,

  rtcInfo, rtcConn, rtcDataCli, rtcHttpCli, rtcWafCli;

type
  TForm1 = class(TForm)
    eAddr: TEdit;
    ePort: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    CliGet: TRtcHttpClient;
    CliPut: TRtcHttpClient;
    btnRun: TButton;
    btnQuit: TButton;
    xProxy: TCheckBox;
    xCloseOnQuit: TCheckBox;
    xHideOnStart: TCheckBox;
    RtcWafClient: TRtcWafClientModule;
    procedure btnRunClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnQuitClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure RtcWafClientAppStart(Sender: TRtcWafClientModule);
    procedure RtcWafClientAppError(Sender: TRtcWafClientModule;
      const ErrorMessage: String);
    procedure RtcWafClientAppShow(Sender: TRtcWafClientModule);
    procedure RtcWafClientAppStop(Sender: TRtcWafClientModule);
    procedure RtcWafClientAppHide(Sender: TRtcWafClientModule);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
  begin
  LOG_THREAD_EXCEPTIONS:=True;
  LOG_EXCEPTIONS:=True;
  CACHE_FOLDER:='CLIENT_CACHE';

  StartLog;
  end;

procedure TForm1.btnRunClick(Sender: TObject);
  begin
  // Prepare HTTP connection components ...
  CliGet.ServerAddr:=AnsiString(eAddr.Text);
  CliGet.ServerPort:=AnsiString(ePort.Text);
  CliGet.UseProxy:=xProxy.Checked;

  CliPut.ServerAddr:=AnsiString(eAddr.Text);
  CliPut.ServerPort:=AnsiString(ePort.Text);
  CliPut.UseProxy:=xProxy.Checked;

  // Disable the "RUN" button, so it can not be pressed twice ...
  btnRun.Enabled:=False;

  // Run the RTC WAF Client ...
  RtcWafClient.Run;
  end;

procedure TForm1.btnQuitClick(Sender: TObject);
  begin
  RtcWafClient.Quit;
  end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  begin
  if btnQuit.Enabled and xCloseOnQuit.Enabled then
    begin
    xCloseOnQuit.Checked:=True;
    xCloseOnQuit.Enabled:=False;
    btnQuit.Click;
    CanClose:=False;
    end
  else if btnQuit.Enabled then // waiting for a clean shutdown
    CanClose:=False
  else
    CanClose:=True;
  end;

procedure TForm1.RtcWafClientAppError(Sender: TRtcWafClientModule; const ErrorMessage: String);
  begin
  // Error reported from RTC WAF Client - using ShowMessage as an example
  ShowMessage(ErrorMessage);
  end;

procedure TForm1.RtcWafClientAppStart(Sender: TRtcWafClientModule);
  begin
  // RTC WAF App is starting up ...
  btnQuit.Enabled:=True;
  end;

procedure TForm1.RtcWafClientAppShow(Sender: TRtcWafClientModule);
  begin
  // RTC WAF App is about to show its main form ...
  if xHideOnStart.Checked then
    begin
    ShowWindow(Handle, SW_HIDE);
    ShowWindow(Application.Handle, SW_HIDE);
    end;
  end;

procedure TForm1.RtcWafClientAppStop(Sender: TRtcWafClientModule);
  begin
  // RTC WAF App is stopping ...
  btnRun.Enabled:=True;
  btnQuit.Enabled:=False;
  end;

procedure TForm1.RtcWafClientAppHide(Sender: TRtcWafClientModule);
  begin
  // RTC WAF App has closed/hidden its main form ...
  if xCloseOnQuit.Checked then
    Application.Terminate
  else if xHideOnStart.Checked then
    begin
    ShowWindow(Application.Handle, SW_SHOW);
    ShowWindow(Handle, SW_SHOW);
    Application.BringToFront;
    BringToFront;
    BringWindowToTop(Handle);
    end;
  end;

end.
