{ 
  RealThinClient W/A Framework: Client-side components interaction
  Copyright (c) Danijel Tkalcec

  @exclude
}
unit rtcBaseCli;

interface

{$include rtcDefs.inc}

uses
  rtcTrashcan,
  {$ifdef WINDOWS}
  Windows,
  {$endif}
  Classes, Controls, SysUtils, Forms, Menus,
  rtcSyncObjs, rtcThrPool, rtcInfo,

  rtcLink, rtcCache,
  
  rtc_FormX, rtc_DataModuleX;

type
  TRtcBaseApplicationC=class; // forward

  IrtcFormC=interface(IrtcfComponentC)
    ['{0126E703-5AC3-4BCA-BDB9-F2AF8F6828F2}']
    procedure New_Object(id:longint; obj:IrtcfComponentC);
    function Get_Object(id:longint):IrtcfComponentC;
    procedure Old_Object(id:longint);
    end;

  (* Central class used by all client-side RTC Forms components *)
  TRtc_ComC=class(TRtcComC)
  private
    FID:longint;
    FFormID:longint;
    FForm:IrtcFormC;
    FApp:TRtcBaseApplicationC;

  public
    // Constructor used only by TRtcApplicationC, then the object is passed over to the component
    constructor Create(myApp:TRtcBaseApplicationC; myForm:IrtcFormC; myObj:IrtcfComponentC; xFormID, xID:longint);
    // Component has to Free the object in its destructor
    destructor Destroy; override;

    // Set remote components property or call remote components method by name
    procedure Send(const xName: string; xData: TRtcValueObject); overload; override;
    procedure Send(const xName: string; xData: Variant); overload; override;
    // Call remote components event
    procedure Event(const xID:integer; xData:TRtcValueObject); overload; override;
    procedure Event(const xID:integer; xData:Variant); overload; override;

    // Set remote components property
    procedure Prop(const xName: string; xData: TRtcValueObject); overload; override;
    procedure Prop(const xName: string; xData: Variant); overload; override;

    function GetUID:TRtcRecord; override;
    function FindByUID(UID:TRtcRecord):TComponent; override;

    procedure UpdateNow; override;

    // Application is currently Idle? (we are currently not processing received data)
    function Idle:boolean; override;

    // Return this components ID
    property ID:longint read FID;

    // Return this components (owner) Form ID
    property FormID:longint read FFormID;
    end;

  TRtcExePackPtrC=^TRtcExePackC;
  TRtcExePackC=record
    InData:TRtcArray;
    Next:TRtcExePackPtrC;
    end;

  TRtcBaseApplicationCThread=class(TRtcThread)
  public
    App:TRtcBaseApplicationC;

    constructor Create; override;
    destructor Destroy; override;

    function Work(Job:TObject):boolean; override;
    procedure Kill(Job:TObject); override;
    end;

  (* Client-side Application component, 1 instance needed per application *)
  TRtcBaseApplicationC=class(TComponent)
  private
    fCS:TRtcCritSec;
    fThr:TRtcBaseApplicationCThread;

    fIdle:boolean;
    fPackFirst,
    fPackLast:TRtcExePackPtrC;

    LastObjID:longint;
    fObjs:array of IrtcFormC;

    // Data prepared for sending to the Server, where the application is running
    xformid,xcompid:longint;
    OutData:TRtcArray;
    PropData:TRtcRecord;
    CacheData:TRtcArray;

    // Data prepared for local execution (received from Server)
    _loc:longint;
    InData:TRtcArray;
    MissingCache:TRtcRecord;
    MissingCacheCount:integer;


    FRecv: TRtcReceiveEvent;
    FOnQuit: TNotifyEvent;

    function UpdateFromCache(xData:TRtcArray):boolean; overload;
    function UpdateFromCache(xData:TRtcRecord):boolean; overload;
    function UpdateFromCache(xData:TRtcDataSet):boolean; overload;
    function UpdateFromCache(xData:TRtcValue):boolean; overload;

    function GetFromCache(const FileName:string):TRtcValue;

    { Prepare the next package for execution.
      NOTE: This method should ONLY be called from ExecuteAll! }
    function PrepareMore:boolean;

    procedure ClearReceived;

    procedure Load_Cache(const xID: String);

  protected
    // Send data to the assigned Receiver (Server)
    procedure _Send(Data:TRtcValueObject); virtual;

    function Get_Object(FormID,ID:longint):IrtcfComponentC;

    function Find_Link(xData:TRtcValue):TComponent;

    { Execute next line from received package(s).
      Return TRUE if there was something to execute,
      FALSE if all lines were already executed. }
    function Execute:boolean;
    { Execute all waiting packages }
    procedure ExecuteAll;

    // Form created, store Form under ID for later access
    procedure Form_Create(id:integer; frm:IrtcFormC);
    // Form is being destroyed, remove reference
    procedure Form_Destroy(id: Integer);

    { Methods where call order is important }
    // Component "formID, compID": Set property "xName" to value "xData", or call method name "xName" with parameters "xData"
    procedure Form_Data(formID,compID:longint; const xName:string; xData:TRtcValueObject); overload;
    procedure Form_Data(formID,compID:longint; const xName:string; xData:Variant); overload;
    // Component "formID, compID": Call event "eventID" with data "xData"
    procedure Form_Event(formID,compID,eventID:longint; xData:TRtcValueObject); overload;
    procedure Form_Event(formID,compID,eventID:longint; xData:Variant); overload;

    { Methods where call order is NOT important and new calls for the same "xName" will override old calls }
    // Component "formID, compID": Set property "xName" to value "xData"
    procedure Form_Prop(formID,compID:longint; const xName:string; xData:TRtcValueObject); overload;
    procedure Form_Prop(formID,compID:longint; const xName:string; xData:Variant); overload;

  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

    // Send package "Data" to be executed from the application thread
    // This is the ONLY method on Application component which can be called from ANY thread.
    // All other methods may ONLY be called from the Main thread.

    procedure EnterExecute;
    procedure ToExecute(const Data:TRtcValueObject);
    procedure LeaveExecute;

    procedure ExecuteNow(const Data:TRtcValueObject);

    // Send "Quit" message to the application, which will free all forms and destroy the application object
    procedure Quit;

    // Send all prepared changes to the Server NOW!
    procedure UpdateNow;

    // Destroy all Forms assigned to this application
    procedure FreeAllForms;

    // Access to the application Thread (can be used by components)
    property Thread:TRtcBaseApplicationCThread read fThr;

    // Where should the data prepared by our components be sent?
    property OnSend:TRtcReceiveEvent read FRecv write FRecv;

    function Idle:boolean;

    property OnQuit:TNotifyEvent read FOnQuit write FOnQuit;
    end;

  // New created Forms will automatically be assigned to the default Application for the current thread
  TRtcBaseFormC=class(TrtcFormX, IrtcFormC, IrtcfComponentC)
  private
    LastObjID:longint;
    fObjs:array of IrtcfComponentC;

    procedure MyOnCloseQuery(Sender: TObject; var CanClose: Boolean);

  protected
    FCom:TRtc_ComC;

    function _Process(const xName: string; xData: TRtcValue):boolean; override;
    procedure _SetCom(Com:TRtcComC); virtual;
    function _SetEvent(const xName:string; xID:longint):boolean; virtual;

    procedure New_Object(id:longint; obj:IrtcfComponentC);
    function Get_Object(id:longint):IrtcfComponentC;
    procedure Old_Object(id:longint);

  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

    function Idle:boolean;
    function _Com:TRtcComC;
    end;

  // New created DataModules will automatically be assigned to the default Application for the current thread
  TRtcBaseDataModuleC=class(TrtcDataModuleX, IrtcFormC, IrtcfComponentC)
  private
    LastObjID:longint;
    fObjs:array of IrtcfComponentC;

  protected
    FCom:TRtc_ComC;

    procedure _SetCom(Com:TRtcComC); virtual;
    function _SetEvent(const xName:string; xID:longint):boolean; virtual;

    procedure New_Object(id:longint; obj:IrtcfComponentC);
    function Get_Object(id:longint):IrtcfComponentC;
    procedure Old_Object(id:longint);

  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

    function _Com:TRtcComC;
    end;

implementation

type
  TRtcBaseApplicationCJob=class
    end;

var
  Message_AppRun,
  Message_AppQuit:TRtcBaseApplicationCJob;

(*
procedure TRtcAppCQuit.Run(Thr: TRtcThread);
  begin
  App.Receiver:=nil;
  Thr.Sync(App.FreeAllForms);

  App.Free;
  App:=nil;

  Free;
  end; *)

{ TRtcApplication }

constructor TRtcBaseApplicationC.Create(AOwner:TComponent);
  begin
  inherited;
  fCS:=TRtcCritSec.Create;
  fThr:=TRtcBaseApplicationCThread.Create;
  fThr.App:=self;

  fIdle:=True;
  fPackFirst:=nil;
  fPackLast:=nil;

  LastObjID:=0;
  SetLength(fObjs,0);

  xformid:=-1;
  xcompid:=-1;

  OutData:=nil;
  PropData:=nil;
  CacheData:=nil;
  end;

destructor TRtcBaseApplicationC.Destroy;
  var
    t:TRtcBaseApplicationCThread;
  begin
  if assigned(FOnQuit) then
    FOnQuit(self);

  if assigned(fThr) then
    begin
    t:=fThr; fThr:=nil;
    t.App:=nil;
    TRtcThread.Stop(t);
    end;

  FreeAllForms;
  OnSend:=nil;

  ClearReceived;

  if assigned(OutData) then
    begin
    OutData.Free;
    OutData:=nil;
    end;
  if assigned(PropData) then
    begin
    PropData.Free;
    PropData:=nil;
    end;
  if assigned(CacheData) then
    begin
    CacheData.Free;
    CacheData:=nil;
    end;

  FreeAndNil(fCS);
  inherited;
  end;

procedure TRtcBaseApplicationC.Quit;
  begin
  TRtcThread.PostJob(fThr, Message_AppQuit);
  end;

procedure TRtcBaseApplicationC.FreeAllForms;
  var
    a:integer;
    o:TComponent;
  begin
  if assigned(fObjs) then
    for a:=length(fObjs)-1 downto 0 do
      if assigned(fObjs[a]) then
        begin
        o:=fObjs[a]._Self;
        fObjs[a]:=nil;
        if assigned(o) then o.Free;
        end;

  LastObjID:=0;
  SetLength(fObjs,0);
  end;

procedure TRtcBaseApplicationC.ExecuteAll;
  begin
  while Execute do;
  end;

procedure TRtcBaseApplicationC.Form_Create(id:integer; frm:IrtcFormC);
  begin
  if id>LastObjID then
    begin
    LastObjID:=id;
    SetLength(fObjs,LastObjID);
    end;
  fObjs[id-1]:=frm;
  end;

procedure TRtcBaseApplicationC.Form_Destroy(id: Integer);
  begin
  if id<=LastObjID then
    fObjs[id-1]:=nil;
  end;

procedure TRtcBaseApplicationC.Form_Data(formid, compid: Integer; const xName: string; xData: TRtcValueObject);
  begin
  if not assigned(OutData) then
    OutData:=TRtcArray.Create;
  // Create a new Record in our OutData array ...
  with OutData.NewRecord(OutData.Count) do
    begin
    if assigned(CacheData) then
      begin
      asObject['!']:=CacheData;
      CacheData:=nil;
      end;
    if assigned(PropData) then
      begin
      asObject['*']:=PropData;
      PropData:=nil;
      end;
    if (formid<>xformid) or (compid<>xcompid) then
      begin
      asInteger[IntToStr(formid)]:=compid;
      xformid:=formid;
      xcompid:=compid;
      end;
    if assigned(xData) then
      asObject[xName]:=xData
    else
      asBoolean[xName]:=False;
    end;
  end;

procedure TRtcBaseApplicationC.Form_Data(formID, compID: Integer; const xName: string; xData: Variant);
  begin
  if not assigned(OutData) then
    OutData:=TRtcArray.Create;
  // Create a new Record in our OutData array ...
  with OutData.NewRecord(OutData.Count) do
    begin
    if assigned(CacheData) then
      begin
      asObject['!']:=CacheData;
      CacheData:=nil;
      end;
    if assigned(PropData) then
      begin
      asObject['*']:=PropData;
      PropData:=nil;
      end;
    if (formid<>xformid) or (compid<>xcompid) then
      begin
      asInteger[IntToStr(formid)]:=compid;
      xformid:=formid;
      xcompid:=compid;
      end;
    asValue[xName]:=xData;
    end;
  end;

procedure TRtcBaseApplicationC.Form_Prop(formID, compID: Integer; const xName: string; xData: TRtcValueObject);
  var
    ids:string;
  begin
  if not assigned(PropData) then
    PropData:=TRtcRecord.Create;

  ids:=IntToStr(formID)+'.'+IntToStr(compID);
  if PropData.isNull[ids] then
    PropData.NewRecord(ids)
  else
    PropData.asRecord[ids].isNull[xName]:=True;
  if xData=nil then
    PropData.asRecord[ids].asBoolean[xName]:=False
  else
    PropData.asRecord[ids].asObject[xName]:=xData;
  end;

procedure TRtcBaseApplicationC.Form_Prop(formID, compID: Integer; const xName: string; xData: Variant);
  var
    ids:string;
  begin
  if not assigned(PropData) then
    PropData:=TRtcRecord.Create;

  ids:=IntToStr(formID)+'.'+IntToStr(compID);
  if PropData.isNull[ids] then
    PropData.NewRecord(ids)
  else
    PropData.asRecord[ids].isNull[xName]:=True;
  PropData.asRecord[ids].asValue[xName]:=xData;
  end;

procedure TRtcBaseApplicationC.Load_Cache(const xID: string);
  begin
  if not assigned(CacheData) then
    CacheData:=TRtcArray.Create;
  CacheData.asVarName[CacheData.Count]:=xID;
  end;

procedure TRtcBaseApplicationC.Form_Event(formID, compID, eventID: longint; xData: TRtcValueObject);
  begin
  if not assigned(OutData) then
    OutData:=TRtcArray.Create;
  // Create a new Record in our OutData array ...
  with OutData.NewRecord(OutData.Count) do
    begin
    if assigned(CacheData) then
      begin
      asObject['!']:=CacheData;
      CacheData:=nil;
      end;
    if assigned(PropData) then
      begin
      asObject['*']:=PropData;
      PropData:=nil;
      end;
    if (formid<>xformid) or (compid<>xcompid) then
      begin
      asInteger[IntToStr(formid)]:=compid;
      xformid:=formid;
      xcompid:=compid;
      end;
    if assigned(xData) then
      asObject['.'+IntToStr(eventID)]:=xData
    else
      asBoolean['.'+IntToStr(eventID)]:=False;
    end;
  UpdateNow;
  end;

procedure TRtcBaseApplicationC.Form_Event(formID, compID, eventID: longint; xData: Variant);
  begin
  if not assigned(OutData) then
    OutData:=TRtcArray.Create;
  // Create a new Record in our OutData array ...
  with OutData.NewRecord(OutData.Count) do
    begin
    if assigned(CacheData) then
      begin
      asObject['!']:=CacheData;
      CacheData:=nil;
      end;
    if assigned(PropData) then
      begin
      asObject['*']:=PropData;
      PropData:=nil;
      end;
    if (formid<>xformid) or (compid<>xcompid) then
      begin
      asInteger[IntToStr(formid)]:=compid;
      xformid:=formid;
      xcompid:=compid;
      end;
    asValue['.'+IntToStr(eventID)]:=xData;
    end;
  UpdateNow;
  end;

procedure TRtcBaseApplicationC.UpdateNow;
  var
    s:TRtcValueObject;
  begin
  if assigned(PropData) or assigned(CacheData) then
    begin
    if not assigned(OutData) then
      OutData:=TRtcArray.Create;
    // Create a new Record in our OutData array ...
    with OutData.NewRecord(OutData.Count) do
      begin
      if assigned(PropData) then
        begin
        asObject['*']:=PropData;
        PropData:=nil;
        end;
      if assigned(CacheData) then
        begin
        asObject['!']:=CacheData;
        CacheData:=nil;
        end;
      end;
    end;
  if assigned(OutData) then
    begin
    // Send all prepared Data to the Server
    if OutData.Count=1 then
      begin
      s:=OutData.asObject[0];
      OutData.asObject[0]:=nil;
      OutData.Free;
      OutData:=nil;
      _Send(s);
      end
    else
      begin
      _Send(OutData);
      OutData:=nil;
      end;

    OutData:=nil;
    xcompid:=-1;
    xformid:=-1;
    end;
  end;

procedure TRtcBaseApplicationC.ToExecute(const Data: TRtcValueObject);
  var
    fTemp:TRtcExePackPtrC;
    fData:TRtcArray;
    loc:integer;
    fN:String;
    fD:AnsiString;
  begin
  if Data is TRtcArray then
    fData:=TRtcArray(Data)
  else
    begin
    fData:=TRtcArray.Create;
    fData.asObject[0]:=Data;
    end;

  // Check if we have received data for storing inside our CACHE ...
  // ----
  if assigned(MissingCache) and (fData.isType[0]=rtc_Variable) then
    begin
    try
      loc:=0;
      while loc<fData.Count do
        begin
        fN:=fData.asVarName[loc];
        fD:=fData.asString[loc+1];
        Inc(loc,2);
        // We are waiting for this data?
        if MissingCache.asBoolean[fN] then
          begin
          rtcCache_Write(fD,fN);
          MissingCache.isNull[fn]:=True;
          Dec(MissingCacheCount);
          if MissingCacheCount=0 then
            begin
            // All required cache elements received ...
            MissingCache.Free;
            MissingCache:=nil;
            Break;
            end;
          end;
        end;
    finally
      fData.Free;
      end;
    // Prevent the Cache response from ending up inside our Execution list
    Exit;
    end;

  New(fTemp);
  fTemp^.Next:=nil;
  fTemp^.InData:=fData;
  if assigned(fPackLast) then
    fPackLast^.Next:=fTemp
  else
    fPackFirst:=fTemp;
  fPackLast:=fTemp;
  end;

function TRtcBaseApplicationC.GetFromCache(const FileName: string): TRtcValue;
  var
    s:AnsiString;
  begin
  if assigned(MissingCache) and (MissingCache.asBoolean[FileName]) then
    Result:=nil
  else
    begin
    s:=rtcCache_Read(FileName);
    if s<>'' then
      Result:=TRtcValue.FromCode(s)
    else
      begin
      Result:=nil;
      if not assigned(MissingCache) then
        MissingCache:=TRtcRecord.Create;
      if not MissingCache.asBoolean[FileName] then
        begin
        MissingCache.asBoolean[FileName]:=True;
        Inc(MissingCacheCount);
        // Ask for cache data from Server ...
        Load_Cache(FileName);
        end;
      end;
    end;
  end;

procedure TRtcBaseApplicationC.EnterExecute;
  begin
  fCS.Enter;
  end;

procedure TRtcBaseApplicationC.LeaveExecute;
  begin
  try
    if fIdle then // not working, post job
      begin
      fIdle:=False; // first job, leaving IDLE mode.
      TRtcThread.PostJob(fThr,Message_AppRun);
      end;
  finally
    fCS.Leave;
    end;
  end;

procedure TRtcBaseApplicationC.ExecuteNow(const Data: TRtcValueObject);
  begin
  EnterExecute;
  try
    ToExecute(Data);
  finally
    LeaveExecute;
    end;
  end;

function TRtcBaseApplicationC.Idle: boolean;
  begin
  fCS.Enter;
  try
    Result:=fIdle;
  finally
    fCS.Leave;
    end;
  end;

procedure TRtcBaseApplicationC.ClearReceived;
  var
    fTemp:TRtcExePackPtrC;
  begin
  fCS.Enter;
  try
    if assigned(InData) then InData.Free;
    InData:=nil;
    while assigned(fPackFirst) do
      begin
      fTemp:=fPackFirst;
      fTemp.InData.Free;
      fPackFirst:=fTemp^.Next;
      if fPackFirst=nil then fPackLast:=nil;
      Dispose(fTemp);
      end;
    if assigned(MissingCache) then
      begin
      MissingCache.Free;
      MissingCache:=nil;
      MissingCacheCount:=0;
      end;
    fIdle:=True; // nothing more to do, going to IDLE mode.
  finally
    fCS.Leave;
    end;
  end;

function TRtcBaseApplicationC.UpdateFromCache(xData: TRtcValue): boolean;
  begin
  case xData.isType of
    rtc_Array: Result:=UpdateFromCache(xData.asArray);
    rtc_Record: Result:=UpdateFromCache(xData.asRecord);
    rtc_DataSet: Result:=UpdateFromCache(xData.asDataSet);
    else Result:=True;
    end;
  end;

function TRtcBaseApplicationC.UpdateFromCache(xData: TRtcArray): boolean;
  var
    a:integer;
    fn:string;
    x:TRtcValue;
  begin
  Result:=True;
  for a:=0 to xData.Count-1 do
    begin
    case xData.isType[a] of
      rtc_Variable: // Need to retrieve data from cache
        begin
        fn:=xData.asVarName[a];
        x:=GetFromCache(fn);
        if assigned(x) then
          begin
          xData.isNull[a]:=True;
          xData.asObject[a]:=x;
          if not UpdateFromCache(x) then Result:=False;
          end
        else
          Result:=False;
        end;
      rtc_Array:
        if not UpdateFromCache(xData.asArray[a]) then Result:=False;
      rtc_Record:
        if not UpdateFromCache(xData.asRecord[a]) then Result:=False;
      rtc_DataSet:
        if not UpdateFromCache(xData.asDataSet[a]) then Result:=False;
      end;
    end;
  end;

function TRtcBaseApplicationC.UpdateFromCache(xData: TRtcRecord): boolean;
  var
    i:integer;
    a,fn:string;
    x:TRtcValue;
  begin
  Result:=True;
  for i:=0 to xData.FieldCount-1 do
    begin
    a:=xData.FieldName[i];
    case xData.isType[a] of
      rtc_Variable: // Need to retrieve data from cache
        begin
        fn:=xData.asVarName[a];
        x:=GetFromCache(fn);
        if assigned(x) then
          begin
          xData.isNull[a]:=True;
          xData.asObject[a]:=x;
          if not UpdateFromCache(x) then Result:=False;
          end
        else
          Result:=False;
        end;
      rtc_Array:
        if not UpdateFromCache(xData.asArray[a]) then Result:=False;
      rtc_Record:
        if not UpdateFromCache(xData.asRecord[a]) then Result:=False;
      rtc_DataSet:
        if not UpdateFromCache(xData.asDataSet[a]) then Result:=False;
      end;
    end;
  end;

function TRtcBaseApplicationC.UpdateFromCache(xData: TRtcDataSet): boolean;
  var
    i:integer;
    a,fn:string;
    x:TRtcValue;
  begin
  Result:=True;
  xData.First;
  while not xData.EOF do
    begin
    for i:=0 to xData.FieldCount-1 do
      begin
      a:=xData.FieldName[i];
      case xData.isType[a] of
        rtc_Variable: // Need to retrieve data from cache
          begin
          fn:=xData.asVarName[a];
          x:=GetFromCache(fn);
          if assigned(x) then
            begin
            xData.isNull[a]:=True;
            xData.asObject[a]:=x;
            if not UpdateFromCache(x) then Result:=False;
            end
          else
            Result:=False;
          end;
        rtc_Array:
          if not UpdateFromCache(xData.asArray[a]) then Result:=False;
        rtc_Record:
          if not UpdateFromCache(xData.asRecord[a]) then Result:=False;
        rtc_DataSet:
          if not UpdateFromCache(xData.asDataSet[a]) then Result:=False;
        end;
      end;
    xData.Next;
    end;
  end;

function TRtcBaseApplicationC.PrepareMore: boolean;
  var
    fTemp:TRtcExePackPtrC;
  begin
  if assigned(InData) then
    begin
    if _loc=0 then
      begin
      Result:=UpdateFromCache(InData);
      if not Result then
        begin
        fIdle:=True; // Waiting for Cache to get filled, going to IDLE mode ...
        UpdateNow;
        end;
      Exit;
      end
    else if _loc<InData.Count then
      begin
      Result:=True;
      Exit;
      end
    else
      begin
      InData.Free;
      InData:=nil;
      end;
    end;
  fCS.Enter;
  try
    fTemp:=fPackFirst;
    if not assigned(fTemp) then
      begin
      Result:=False;
      fIdle:=True; // nothing more to do, going to IDLE mode.
      end
    else
      begin
      InData:=fTemp.InData;
      _loc:=0;

      fPackFirst:=fTemp^.Next;
      if fPackFirst=nil then fPackLast:=nil;
      Dispose(fTemp);

      Result:=UpdateFromCache(InData);
      if not Result then
        begin
        fIdle:=True; // Waiting for Cache to get filled, going to IDLE mode ...
        UpdateNow;
        end;
      end;
  finally
    fCS.Leave;
    end;
  end;

function TRtcBaseApplicationC.Execute:boolean;
  var
    r:TRtcArray;
    fn:string;
    fid,cid:integer;
  procedure ExecRec(arr:TRtcArray);
    var
      rec:TRtcRecord;
      Val:TRtcValue;
      a:integer;
      fn:string;
    procedure ExecProp(cid:integer; const pname:string; val:TRtcValue);
      var
        c:TRtc_ComC;
        obc:TComponentClass;
        ob:TComponent;
        myfr:IrtcFormC;
        myob:IrtcfComponentC;
      begin
      if pname='quit' then
        Quit
      else if pname='new' then // Creating a new Form, DataModule or Component
        begin
        obc:=TComponentClass(FindClass('TRtc'+Val.asText+'C'));
        if assigned(obc) then
          begin
          if (fid>0) and (cid=0) and Supports(obc,IrtcFormC) then
            begin
            myfr:=(obc.Create(nil) as IrtcFormC);
            c:=TRtc_ComC.Create(self,myfr,nil,fid,cid);
            myfr._SetCom(c);
            myfr._BeginLoad;
            end
          else if (fid>0) and (cid>0) and Supports(obc,IrtcfComponentC) then
            begin
            if LastObjID>=fid then
              begin
              myfr:=fObjs[fid-1];
              if assigned(myfr) then
                begin
                myob:=(obc.Create(myfr._Self) as IrtcfComponentC);
                c:=TRtc_ComC.Create(self,myfr,myob,fid,cid);
                myob._SetCom(c);
                if myfr._Loading then
                  myob._BeginLoad;
                end;
              end;
            end;
          end;
        end
      else if (fid>0) and (LastObjID>=fid) then // Working with a Form, DataModule or Component
        begin
        if assigned(fObjs[fid-1]) then
          begin
          myob:=fObjs[fid-1].Get_Object(cid);
          if assigned(myob) then
            begin
            if pname<>'' then
              begin
              if pname='name' then // Set component Name
                begin
                ob:=myob._Self;
                ob.Name:=Val.asText;
                end
              else if pname='free' then // Free component
                begin
                ob:=myob._Self;
                myob:=nil;
                ob.Free;
                end
              else if pname='ok' then
                myob._EndLoad
              else if pname[1]='/' then
                begin
                if val.asRecord.isNull['f'] then
                  val.asRecord.asInteger['f']:=fid;
                if val.asRecord.isNull['c'] then
                  val.asRecord.asInteger['c']:=0;
                myob._Link(Copy(pname,2,length(pname)-1),Find_Link(Val));
                end
              else // Process property/method data
                myob._Process(pname,Val);
              end;
            end;
          end;
        end;
      end;
    procedure ExecEvent(cid,pid:longint; val:TRtcValue);
      var
        myob:IrtcfComponentC;
      begin
      if (fid>0) and (LastObjID>=fid) then // Working with a Form, DataModule or Component
        if assigned(fObjs[fid-1]) then
          begin
          myob:=fObjs[fid-1].Get_Object(cid);
          if assigned(myob) then
            if pid>=0 then // Set event ID (event accessible remotely)
              myob._SetEvent(Val.asText,pid)
          end;
      end;
    procedure UnwrapProp(rec:TRtcRecord);
      var
        a,b:integer;
        cid,pid:longint;
        fn:string;
        val:TRtcValue;
        rr:TRtcRecord;
      begin
      val:=nil;
      try
        for a:=0 to rec.Count-1 do
          begin
          fn:=rec.FieldName[a];
          if rec.isType[fn]=rtc_Record then
            begin
            cid:=StrToIntDef(fn,0);
            rr:=rec.asRecord[fn];
            for b:=0 to rr.Count-1 do
              begin
              fn:=rr.FieldName[b];
              if (fn[1]='.') and (length(fn)>1) and ((fn[2]>='0') and (fn[2]<='9')) then // Set Event
                begin
                if assigned(Val) then
                  Val.Clear
                else
                  Val:=TRtcValue.Create;
                pid:=StrToIntDef(Copy(fn,2,length(fn)-1),0);
                if not rr.isNull[fn] then
                  begin
                  Val.asObject:=rr.asObject[fn];
                  rr.asObject[fn]:=nil;
                  end;
                ExecEvent(cid,pid,val);
                end
              else
                begin
                if assigned(Val) then
                  Val.Clear
                else
                  Val:=TRtcValue.Create;
                if not rr.isNull[fn] then
                  begin
                  Val.asObject:=rr.asObject[fn];
                  rr.asObject[fn]:=nil;
                  end;
                ExecProp(cid,fn,val);
                end;
              end;
            end;
          end;
      finally
        if assigned(val) then val.Free;
        end;
      end;
    begin
    Val:=nil;
    try
      for a:=0 to arr.Count-1 do
        begin
        if arr.isType[a]=rtc_Record then
          begin
          rec:=arr.asRecord[a];
          fn:=rec.FieldName[0];
          if (fn[1]>='0') and (fn[1]<='9') then
            UnwrapProp(rec)
          else
            begin
            if rec.isType['.']=rtc_Integer then
              cid:=rec.asInteger['.'];
            if assigned(Val) then
              Val.Clear
            else
              Val:=TRtcValue.Create;
            if not rec.isNull[fn] then
              begin
              Val.asObject:=rec.asObject[fn];
              rec.asObject[fn]:=nil;
              end;
            ExecProp(cid,fn,Val);
            end;
          end;
        end;
    finally
      if assigned(Val) then Val.Free;
      end;
    end;
  begin
  if not PrepareMore then
    Result:=False
  else
    begin
    Result:=True;
    if InData.isType[_loc]=rtc_Record then
      begin
      with InData.asRecord[_loc] do
        begin
        Inc(_loc);
        fn:=FieldName[0];
        fid:=StrToIntDef(fn,0);
        cid:=0;
        if isType[fn]=rtc_Array then
          begin
          r:=asArray[fn];
          Extract(fn);
          end
        else
          begin
          r:=TRtcArray.Create;
          r.asObject[0]:=asObject[fn];
          asObject[fn]:=nil;
          end;
        try
          ExecRec(r);
        finally
          r.Free;
          end;
        end;
      end
    else
      Inc(_loc);
    end;
  end;

function TRtcBaseApplicationC.Get_Object(FormID, ID: Integer): IrtcfComponentC;
  begin
  if (FormID>0) and (LastObjID>=FormID) then
    begin
    if assigned(fObjs[FormID-1]) then
      Result:=fObjs[FormID-1].Get_Object(ID)
    else
      Result:=nil;
    end
  else
    Result:=nil;
  end;

procedure TRtcBaseApplicationC._Send(Data: TRtcValueObject);
  begin
  if assigned(FRecv) then
    FRecv(Data)
  else
    Data.Free;
  end;

function TRtcBaseApplicationC.Find_Link(xData: TRtcValue): TComponent;
  var
    ob:IrtcfComponentC;
  begin
  if xData.isType=rtc_Record then
    begin
    ob:=Get_Object(xData.asRecord.asInteger['f'], xData.asRecord.asInteger['c']);
    if assigned(ob) and (ob._Self is TComponent) then
      Result:=ob._Self as TComponent
    else
      Result:=nil;
    end
  else
    Result:=nil;
  end;

{ TRtc_ComC }

constructor TRtc_ComC.Create(myApp:TRtcBaseApplicationC; myForm:IrtcFormC; myObj:IrtcfComponentC; xFormID, xID:longint);
  begin
  inherited Create;
  FID:=xID;
  FFormID:=xFormID;
  FApp:=myApp;
  FForm:=myForm;
  if assigned(FApp) and assigned(FForm) then
    if assigned(myObj) then
      FForm.New_Object(xID,myObj)
    else
      FApp.Form_Create(xFormID,myForm);
  end;

destructor TRtc_ComC.Destroy;
  begin
  if assigned(FForm) then
    begin
    if FID>0 then
      FForm.Old_Object(FID);
    FForm:=nil;
    end;
  if assigned(FApp) then
    begin
    if (FID=0) and (FFormID>0) then
      FApp.Form_Destroy(FFormID);
    FApp:=nil;
    end;
  inherited;
  end;

procedure TRtc_ComC.Send(const xName: string; xData: TRtcValueObject);
  begin
  if assigned(FApp) then
    FApp.Form_Data(FFormID, FID, xName, xData)
  else if assigned(xData) then
    xData.Free;
  end;

procedure TRtc_ComC.Send(const xName: string; xData: Variant);
  begin
  if assigned(FApp) then
    FApp.Form_Data(FFormID, FID, xName, xData);
  end;

procedure TRtc_ComC.Prop(const xName: string; xData: TRtcValueObject);
  begin
  if assigned(FApp) then
    FApp.Form_Prop(FFormID, FID, xName, xData)
  else if assigned(xData) then
    xData.Free;
  end;

procedure TRtc_ComC.Prop(const xName: string; xData: Variant);
  begin
  if assigned(FApp) then
    FApp.Form_Prop(FFormID, FID, xName, xData);
  end;

procedure TRtc_ComC.Event(const xID:longint; xData: TRtcValueObject);
  begin
  if assigned(FApp) then
    FApp.Form_Event(FFormID, FID, xID, xData)
  else if assigned(xData) then
    xData.Free;
  end;

procedure TRtc_ComC.Event(const xID:longint; xData: Variant);
  begin
  if assigned(FApp) then
    FApp.Form_Event(FFormID, FID, xID, xData);
  end;

procedure TRtc_ComC.UpdateNow;
  begin
  if assigned(fApp) then
    FApp.UpdateNow;
  end;

function TRtc_ComC.Idle: boolean;
  begin
  if assigned(fApp) then
    Result:=FApp.Idle
  else
    Result:=True;
  end;

function TRtc_ComC.GetUID: TRtcRecord;
  begin
  Result:=TRtcRecord.Create;
  Result.asInteger['f']:=FormID;
  Result.asInteger['c']:=ID;
  end;

function TRtc_ComC.FindByUID(UID: TRtcRecord): TComponent;
  var
    c:IRtcfComponentC;
  begin
  if assigned(FApp) then
    begin
    c:=FApp.Get_Object(UID.asInteger['f'],UID.asInteger['c']);
    if assigned(c) then
      Result:=c._Self
    else
      Result:=nil;
    end
  else
    Result:=nil;
  end;

{ TRtcBaseForm }

constructor TRtcBaseFormC.Create(AOwner: TComponent);
  begin
  inherited;
  LastObjID:=0;
  SetLength(fObjs,0);
  end;

destructor TRtcBaseFormC.Destroy;
  var
    c:TObject;
  begin
  LastObjID:=0;
  SetLength(fObjs,0);

  c:=FCom;
  inherited;
  if assigned(c) then FreeAndNil(c);
  end;

procedure TRtcBaseFormC.New_Object(id:longint; obj:IrtcfComponentC);
  begin
  if (id>LastObjID) then
    begin
    LastObjID:=id;
    SetLength(fObjs,LastObjID);
    end;
  fObjs[id-1]:=obj;
  end;

procedure TRtcBaseFormC.Old_Object(id: Integer);
  begin
  if (id>0) and (id<=LastObjID) then
    fObjs[id-1]:=nil;
  end;

function TRtcBaseFormC.Get_Object(id: integer):IrtcfComponentC;
  begin
  if id=0 then
    Result:=self
  else if (id>0) and (LastObjID>=id) then
    Result:=fObjs[id-1]
  else
    Result:=nil;
  end;

procedure TRtcBaseFormC._SetCom(Com: TRtcComC);
  begin
  FCom:=TRtc_ComC(Com);
  OnCloseQuery:=MyOnCloseQuery;
  end;

function TRtcBaseFormC.Idle: boolean;
  begin
  Result:=FCom.Idle;
  end;

function TRtcBaseFormC._SetEvent(const xName: string; xID: Integer):boolean;
  begin
  Result:=False;
  end;

function TRtcBaseFormC._Process(const xName: string; xData: TRtcValue): boolean;
  begin
  Result:=inherited _Process(xName,xData);
  end;

procedure TRtcBaseFormC.MyOnCloseQuery(Sender: TObject; var CanClose: Boolean);
  begin
  // This event ALWAYS has to be assigned, to capture user attampts to close the window.
  // Using this event, we are telling the Server that user wants to close the Form by sending him the "x" function.
  // Server will then execute its OnCloseQuery and OnClose events and do whatever needs to be done
  CanClose:=False;
  FCom.Send('x',nil);
  FCom.UpdateNow;
  end;

function TRtcBaseFormC._Com: TRtcComC;
  begin
  Result:=FCom;
  end;

{ TRtcBaseDataModule }

constructor TRtcBaseDataModuleC.Create(AOwner: TComponent);
  begin
  inherited;
  LastObjID:=0;
  SetLength(fObjs,0);
  end;

destructor TRtcBaseDataModuleC.Destroy;
  var
    c:TObject;
  begin
  LastObjID:=0;
  SetLength(fObjs,0);

  c:=FCom;
  inherited;
  if assigned(c) then FreeAndNil(c);
  end;

procedure TRtcBaseDataModuleC.New_Object(id:longint; obj:IrtcfComponentC);
  begin
  if (id>LastObjID) then
    begin
    LastObjID:=id;
    SetLength(fObjs,LastObjID);
    end;
  fObjs[id-1]:=obj;
  end;

procedure TRtcBaseDataModuleC.Old_Object(id: Integer);
  begin
  if (id>0) and (id<=LastObjID) then
    fObjs[id-1]:=nil;
  end;

function TRtcBaseDataModuleC.Get_Object(id: integer):IrtcfComponentC;
  begin
  if id=0 then
    Result:=self
  else if (id>0) and (LastObjID>=id) then
    Result:=fObjs[id-1]
  else
    Result:=nil;
  end;

procedure TRtcBaseDataModuleC._SetCom(Com: TRtcComC);
  begin
  FCom:=TRtc_ComC(Com);
  end;

function TRtcBaseDataModuleC._SetEvent(const xName: string; xID: Integer): boolean;
  begin
  Result:=False;
  end;

function TRtcBaseDataModuleC._Com: TRtcComC;
  begin
  Result:=FCom;
  end;

{ TRtcBaseApplicationCThread }

constructor TRtcBaseApplicationCThread.Create;
  begin
  inherited;
  end;

destructor TRtcBaseApplicationCThread.Destroy;
  var
    ap:TRtcBaseApplicationC;
  begin
  if assigned(App) then
    begin
    ap:=App; App:=nil;
    ap.fThr:=nil;

    ap.OnSend:=nil;
    Sync(ap.FreeAllForms);
    Sync(ap.Free);
    end;
  inherited;
  end;

procedure TRtcBaseApplicationCThread.Kill(Job: TObject);
  begin
  inherited;
  end;

function TRtcBaseApplicationCThread.Work(Job: TObject): boolean;
  var
    ap:TRtcBaseApplicationC;
  begin
  Result:=False;
  if Job=Message_AppRun then
    begin
    if assigned(App) then
      begin
      ap:=App;
      Sync(ap.ExecuteAll);
      end
    else
      begin
      Result:=True;
      // Free;
      end;
    end
  else if Job=Message_AppQuit then
    begin
    if assigned(App) then
      begin
      ap:=App; App:=nil;
      ap.fThr:=nil;

      ap.OnSend:=nil;
      Sync(ap.FreeAllForms);
      Sync(ap.Free);
      end;
    Result:=True;
    // Free;
    end
  else
    Result:= inherited Work(Job);
  end;

initialization
Message_AppRun:=TRtcBaseApplicationCJob.Create;
Message_AppQuit:=TRtcBaseApplicationCJob.Create;
finalization
Garbage(Message_AppRun);
Garbage(Message_AppQuit);
end.
