{
  RealThinClient W/A Framework: General Server-side Controls
  Copyright (c) Danijel Tkalcec

  @exclude
}
unit rtcControls;

interface

uses
  rtcTrashcan, rtcLog,
  Graphics, Windows, Classes, Messages, SysUtils,
  {$IFNDEF NoForms} Controls {$ELSE} _Controls {$ENDIF},
  rtcInfo,

  rtcLink, rtcBase,
  rtcGraphics;

const
  RtcDefault_Anchors = [akLeft, akTop];
  RtcDefault_BevelEdges = [beLeft, beTop, beRight, beBottom];

type
  TRtcMouseWheelEvent = procedure(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean) of object;
  TRtcMouseWheelUpDownEvent = procedure(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean) of object;
  TRtcMouseEvent = procedure(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer) of object;
  TRtcMouseMoveEvent = procedure(Sender: TObject; Shift: TShiftState; X, Y: Integer) of object;

  TRtcKeyEvent = procedure(Sender: TObject; var Key: Word; Shift: TShiftState) of object;
  TRtcKeyPressEvent = procedure(Sender: TObject; var Key: Char) of object;

  TRtcDragDropEvent = procedure(Sender, Source: TObject; X, Y: Integer) of object;
  TRtcStartDragEvent = procedure(Sender: TObject; var DragObject: TDragObject) of object;
  TRtcEndDragEvent = procedure(Sender, Target: TObject; X, Y: Integer) of object;

  TRtcDropFilesEvent = procedure(Sender:TObject; const Files:array of string) of object;

  TRtcScrollBarKind = (sbHorizontal, sbVertical);
  TRtcScrollBarInc = 1..32767;
  TRtcScrollBarStyle = (ssRegular, ssFlat, ssHotTrack);

type
  TRtcSizeConstraints=class(TPersistent)
  private
    FCom:TRtcCom;
    FFullName: string;
    FPar: TSizeConstraints;
    function Get_MaxHeight: TConstraintSize;
    function Get_MaxWidth: TConstraintSize;
    function Get_MinHeight: TConstraintSize;
    function Get_MinWidth: TConstraintSize;
    procedure Set_MaxHeight(const Value: TConstraintSize);
    procedure Set_MaxWidth(const Value: TConstraintSize);
    procedure Set_MinHeight(const Value: TConstraintSize);
    procedure Set_MinWidth(const Value: TConstraintSize);

  public
    procedure Assign(Source: TPersistent); override;

    property FullName:string read FFullName write FFullName;
    property Par:TSizeConstraints read FPar write FPar;
    property _Com:TRtcCom read FCom write FCom;

  published
    property MaxHeight: TConstraintSize read Get_MaxHeight write Set_MaxHeight default 0;
    property MaxWidth: TConstraintSize read Get_MaxWidth write Set_MaxWidth default 0;
    property MinHeight: TConstraintSize read Get_MinHeight write Set_MinHeight default 0;
    property MinWidth: TConstraintSize read Get_MinWidth write Set_MinWidth default 0;
    end;

  TRtcControlScrollBar = class(TPersistent)
  private
    FCom:TRtcCom;
    FFullName: string;
    FAutoScroll: boolean;

    Par_ButtonSize: Integer;
    Par_Color: TColor;
    Par_Increment: TRtcScrollBarInc;
    Par_Margin: Word;
    Par_ParentColor: Boolean;
    Par_Position: Integer;
    Par_Range: Integer;
    Par_Size: Integer;
    Par_Smooth: Boolean;
    Par_Style: TRtcScrollBarStyle;
    Par_ThumbSize: Integer;
    Par_Tracking: Boolean;
    Par_Visible: Boolean;

    procedure Set_ButtonSize(const Value: Integer);
    procedure Set_Color(const Value: TColor);
    procedure Set_Increment(const Value: TRtcScrollBarInc);
    procedure Set_Margin(const Value: Word);
    procedure Set_ParentColor(const Value: Boolean);
    procedure Set_Position(const Value: Integer);
    procedure Set_Range(const Value: Integer);
    procedure Set_Size(const Value: Integer);
    procedure Set_Smooth(const Value: Boolean);
    procedure Set_Style(const Value: TRtcScrollBarStyle);
    procedure Set_ThumbSize(const Value: Integer);
    procedure Set_Tracking(const Value: Boolean);
    procedure Set_Visible(const Value: Boolean);

    function Get_ButtonSize: Integer;
    function Get_Color: TColor;
    function Get_Increment: TRtcScrollBarInc;
    function Get_Margin: Word;
    function Get_ParentColor: Boolean;
    function Get_Position: Integer;
    function Get_Range: Integer;
    function Get_Size: Integer;
    function Get_Smooth: Boolean;
    function Get_Style: TRtcScrollBarStyle;
    function Get_ThumbSize: Integer;
    function Get_Tracking: Boolean;
    function Get_Visible: Boolean;
    function IsRangeStored: Boolean;
    function IsIncrementStored: Boolean;

  public
    constructor Create; virtual;

    procedure Assign(Source: TPersistent); override;

    property FullName:string read FFullName write FFullName;
    property _Com:TRtcCom read FCom write FCom;
    property AutoScroll:boolean read FAutoScroll write FAutoScroll;

  published
    property ButtonSize: Integer read Get_ButtonSize write Set_ButtonSize default 0;
    property Color: TColor read Get_Color write Set_Color default clBtnHighlight;
    property Increment: TRtcScrollBarInc read Get_Increment write Set_Increment stored IsIncrementStored default 8;
    property Margin: Word read Get_Margin write Set_Margin default 0;
    property ParentColor: Boolean read Get_ParentColor write Set_ParentColor default True;
    property Position: Integer read Get_Position write Set_Position default 0;
    property Range: Integer read Get_Range write Set_Range stored IsRangeStored default 0;
    property Smooth: Boolean read Get_Smooth write Set_Smooth default False;
    property Size: Integer read Get_Size write Set_Size default 0;
    property Style: TRtcScrollBarStyle read Get_Style write Set_Style default ssRegular;
    property ThumbSize: Integer read Get_ThumbSize write Set_ThumbSize default 0;
    property Tracking: Boolean read Get_Tracking write Set_Tracking default False;
    property Visible: Boolean read Get_Visible write Set_Visible default True;
  end;

  { This is a base class which can be used to implement
    any TComponent descendant (server-side code) }
  TRtcComponent=class(TRtcBaseComponent)
  private
    FCanvas:TRtcCanvas;

    function Get_Tag: Longint;
    procedure Set_Tag(const Value: Longint);
    function Get_Canvas: TRtcCanvas;

  protected
    property Canvas: TRtcCanvas read Get_Canvas;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

  published
    property Tag: Longint read Get_Tag write Set_Tag default 0;
    end;

  { This is a base class which can be used to implement
    any TControl descendant (server-side code) }
  TRtcControl=class(TRtcBaseControl)
  private
    FCanvas:TRtcCanvas;
    FAction:IRtcBasicAction;
    FPopupMenu:IRtcPopupMenu;

    FOnMouseDown: TRtcMouseEvent;
    FOnMouseUp: TRtcMouseEvent;
    FOnMouseDownMove: TRtcMouseMoveEvent;
    FOnMouseMove: TRtcMouseMoveEvent;
    FOnMouseUpMove: TRtcMouseMoveEvent;
    FOnMouseWheel: TRtcMouseWheelEvent;
    FOnMouseWheelUp: TRtcMouseWheelUpDownEvent;
    FOnMouseWheelDown: TRtcMouseWheelUpDownEvent;
    FOnResize: TNotifyEvent;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FOnMouseEnter: TNotifyEvent;

    FOnDragDrop: TRtcDragDropEvent;
    FOnEndDrag: TRtcEndDragEvent;
    FOnStartDrag: TRtcStartDragEvent;
    FOnDropFiles: TRtcDropFilesEvent;

    function Get_Cursor: TCursor;
    function Get_HelpContext: THelpContext;
    function Get_HelpKeyword: String;
    function Get_HelpType: THelpType;
    function Get_Hint: string;
    function Get_Tag: Longint;

    procedure Set_Cursor(const Value: TCursor);
    procedure Set_Hint(const Value: string);
    procedure Set_HelpKeyword(const Value: String);
    procedure Set_HelpType(const Value: THelpType);
    procedure Set_Tag(const Value: Longint);
    procedure Set_HelpContext(const Value: THelpContext);

    function Get_Action: IRtcBasicAction;
    function Get_Align: TAlign;
    function Get_Anchors: TAnchors;
    function Get_BiDiMode: TBiDiMode;
    function Get_Color: TColor;
    function Get_Constraints: TRtcSizeConstraints;
    function Get_DesktopFont: Boolean;
    function Get_DragCursor: TCursor;
    function Get_DragKind: TDragKind;
    function Get_DragMode: TDragMode;
    function Get_ParentBiDiMode: Boolean;
    function Get_ParentColor: Boolean;
    function Get_ParentFont: Boolean;
    function Get_ParentShowHint: Boolean;
    function Get_PopupMenu: IRtcPopupMenu;
    function Get_ShowHint: Boolean;
    function Get_Visible: Boolean;
    //function Get_WindowText: PChar;
    function Get_Font: TRtcFont;
    function Get_Enabled: Boolean;

    function IsAnchorsStored: Boolean;
    function IsBiDiModeStored: Boolean;
    function IsColorStored: Boolean;
    function IsFontStored: Boolean;
    function IsOnClickStored: Boolean;
    function IsShowHintStored: Boolean;
    function IsVisibleStored: Boolean;
    function IsEnabledStored: Boolean;

    procedure Set_Enabled(const Value: Boolean);
    procedure Set_Action(const Value: IRtcBasicAction);
    procedure Set_Align(const Value: TAlign);
    procedure Set_Anchors(const Value: TAnchors);
    procedure Set_AutoSize(const Value: Boolean);
    procedure Set_BiDiMode(const Value: TBiDiMode);
    procedure Set_Color(const Value: TColor);
    procedure Set_Constraints(const Value: TRtcSizeConstraints);
    procedure Set_DesktopFont(const Value: Boolean);
    procedure Set_DragCursor(const Value: TCursor);
    procedure Set_DragKind(const Value: TDragKind);
    procedure Set_DragMode(const Value: TDragMode);
    procedure Set_ParentBiDiMode(const Value: Boolean);
    procedure Set_ParentColor(const Value: Boolean);
    procedure Set_ParentFont(const Value: Boolean);
    procedure Set_ParentShowHint(const Value: Boolean);
    procedure Set_PopupMenu(const Value: IRtcPopupMenu);
    procedure Set_ShowHint(const Value: Boolean);
    procedure Set_Visible(const Value: Boolean);
    //procedure Set_WindowText(const Value: PChar);
    procedure Set_Font(const Value: TRtcFont);

    procedure Set_OnClick(const Value: TNotifyEvent);
    procedure Set_OnDblClick(const Value: TNotifyEvent);

    procedure Set_OnDragDrop(const Value: TRtcDragDropEvent);
    procedure Set_OnEndDrag(const Value: TRtcEndDragEvent);
    procedure Set_OnStartDrag(const Value: TRtcStartDragEvent);
    procedure Set_OnDropFiles(const Value: TRtcDropFilesEvent);

    procedure Set_OnMouseDown(const Value: TRtcMouseEvent);
    procedure Set_OnMouseMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnMouseUp(const Value: TRtcMouseEvent);
    procedure Set_OnMouseWheel(const Value: TRtcMouseWheelEvent);
    procedure Set_OnMouseWheelDown(const Value: TRtcMouseWheelUpDownEvent);
    procedure Set_OnMouseWheelUp(const Value: TRtcMouseWheelUpDownEvent);
    procedure Set_OnMouseDownMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnMouseUpMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnResize(const Value: TNotifyEvent);
    procedure Set_OnMouseEnter(const Value: TNotifyEvent);
    procedure Set_OnMouseLeave(const Value: TNotifyEvent);

    procedure MyFontChange(Sender:TObject);

    function Get_Canvas: TRtcCanvas;

  protected
    FAutoSize: boolean;

    FFnt: TRtcFont;
    FConstr: TRtcSizeConstraints;

    procedure MyOnClick(xData:TRtcValue);
    procedure MyOnDblClick(xData:TRtcValue);
    procedure MyOnResize(xData:TRtcValue);

    procedure MyOnDragDrop(xData:TRtcValue);
    procedure MyOnStartDrag(xData:TRtcValue);
    procedure MyOnEndDrag(xData:TRtcValue);
    procedure MyOnDropFiles(xData:TRtcValue);

    procedure MyOnMouseDown(xData:TRtcValue);
    procedure MyOnMouseUp(xData:TRtcValue);
    procedure MyOnMouseMove(xData:TRtcValue);
    procedure MyOnMouseDownMove(xData:TRtcValue);
    procedure MyOnMouseUpMove(xData:TRtcValue);

    procedure MyOnMouseWheel(xData:TRtcValue);
    procedure MyOnMouseWheelUp(xData:TRtcValue);
    procedure MyOnMouseWheelDown(xData:TRtcValue);

    procedure MyOnMouseEnter(xData:TRtcValue);
    procedure MyOnMouseLeave(xData:TRtcValue);

    procedure Screen2Client(var x,y:integer);

    property Canvas: TRtcCanvas read Get_Canvas;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  protected

    property Font: TRtcFont read Get_Font write Set_Font stored IsFontStored;

    property Enabled: Boolean read Get_Enabled write Set_Enabled stored IsEnabledStored default True;

    property AutoSize: Boolean read FAutoSize write Set_AutoSize default False;
    property Color: TColor read Get_Color write Set_Color stored IsColorStored default clWindow;
    property DesktopFont: Boolean read Get_DesktopFont write Set_DesktopFont default False;
    property DragKind: TDragKind read Get_DragKind write Set_DragKind default dkDrag;
    property DragCursor: TCursor read Get_DragCursor write Set_DragCursor default crDrag;
    property DragMode: TDragMode read Get_DragMode write Set_DragMode default dmManual;
    property ParentBiDiMode: Boolean read Get_ParentBiDiMode write Set_ParentBiDiMode default True;
    property ParentColor: Boolean read Get_ParentColor write Set_ParentColor default True;
    property ParentFont: Boolean read Get_ParentFont write Set_ParentFont default True;
    property ParentShowHint: Boolean read Get_ParentShowHint write Set_ParentShowHint default True;
    property PopupMenu: IRtcPopupMenu read Get_PopupMenu write Set_PopupMenu;
    //property WindowText: PChar read Get_WindowText write Set_WindowText;

    property OnClick: TNotifyEvent read FOnClick write Set_OnClick stored IsOnClickStored;
    property OnDblClick: TNotifyEvent read FOnDblClick write Set_OnDblClick;

    property OnDragDrop: TRtcDragDropEvent read FOnDragDrop write Set_OnDragDrop;
    property OnStartDrag: TRtcStartDragEvent read FOnStartDrag write Set_OnStartDrag;
    property OnEndDrag: TRtcEndDragEvent read FOnEndDrag write Set_OnEndDrag;
    property OnDropFiles: TRtcDropFilesEvent read FOnDropFiles write Set_OnDropFiles;

    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write Set_OnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write Set_OnMouseLeave;

    property OnMouseDown: TRtcMouseEvent read FOnMouseDown write Set_OnMouseDown;
    property OnMouseMove: TRtcMouseMoveEvent read FOnMouseMove write Set_OnMouseMove;
    property OnMouseDownMove: TRtcMouseMoveEvent read FOnMouseDownMove write Set_OnMouseDownMove;
    property OnMouseUpMove: TRtcMouseMoveEvent read FOnMouseUpMove write Set_OnMouseUpMove;
    property OnMouseUp: TRtcMouseEvent read FOnMouseUp write Set_OnMouseUp;

    property OnMouseWheel: TRtcMouseWheelEvent read FOnMouseWheel write Set_OnMouseWheel;
    property OnMouseWheelDown: TRtcMouseWheelUpDownEvent read FOnMouseWheelDown write Set_OnMouseWheelDown;
    property OnMouseWheelUp: TRtcMouseWheelUpDownEvent read FOnMouseWheelUp write Set_OnMouseWheelUp;

    property OnResize: TNotifyEvent read FOnResize write Set_OnResize;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Hide;
    procedure Show;

    procedure BringToFront;
    procedure SendToBack;

    procedure Refresh;

    procedure BeginDrag(Immediate: Boolean; Threshold: Integer = -1);
    function Dragging: Boolean;
    procedure EndDrag(Drop: Boolean);
    procedure DragDrop(Source: TObject; X, Y: Integer); override;

    function ClientToScreen(const Point: TPoint): TPoint;
    function ScreenToClient(const Point: TPoint): TPoint;

    // Overriding default implementation --->>
    (*
    procedure Dock(NewDockSite: TWinControl; ARect: TRect); override;

    procedure InitiateAction; override;
    function IsRightToLeft: Boolean;

    function ManualDock(NewDockSite: TWinControl; DropControl: TControl = nil; ControlSide: TAlign = alNone): Boolean;
    function ReplaceDockedControl(Control: TControl; NewDockSite: TWinControl; DropControl: TControl; ControlSide: TAlign): Boolean;
    function ManualFloat(ScreenPos: TRect): Boolean;

    procedure Invalidate; override;
    procedure Repaint; override;
    procedure Update; override;
    *)
    // <<---

    property Action: IRtcBasicAction read Get_Action write Set_Action;
    property Align: TAlign read Get_Align write Set_Align default alNone;
    property Anchors: TAnchors read Get_Anchors write Set_Anchors stored IsAnchorsStored default RtcDefault_Anchors;
    property BiDiMode: TBiDiMode read Get_BiDiMode write Set_BiDiMode stored IsBiDiModeStored;
    property Constraints: TRtcSizeConstraints read Get_Constraints write Set_Constraints;
    property ShowHint: Boolean read Get_ShowHint write Set_ShowHint stored IsShowHintStored;
    property Visible: Boolean read Get_Visible write Set_Visible stored IsVisibleStored default True;

  published
    property Tag: Longint read Get_Tag write Set_Tag default 0;
    property Cursor: TCursor read Get_Cursor write Set_Cursor default crDefault;
    property Hint: string read Get_Hint write Set_Hint;
    property HelpType: THelpType read Get_HelpType write Set_HelpType default htContext;
    property HelpKeyword: String read Get_HelpKeyword write Set_HelpKeyword;
    property HelpContext: THelpContext read Get_HelpContext write Set_HelpContext default 0;
    end;

  TRtcWinControl=class(TRtcControl, IRtcWinControl)
  private
    FOnKeyDown: TRtcKeyEvent;
    FOnKeyUp: TRtcKeyEvent;
    FOnKeyPress: TRtcKeyPressEvent;
    FOnEnter: TNotifyEvent;
    FOnExit: TNotifyEvent;
    FOnChange: TNotifyEvent;

    function Get_ParentBackground: Boolean;
    function Get_TabOrder: TTabOrder;
    function IsCtl3DStored: Boolean;

    procedure Set_BevelEdges(const Value: TBevelEdges);
    procedure Set_BevelKind(const Value: TBevelKind);
    procedure Set_BevelWidth(const Value: TBevelWidth);
    procedure Set_BorderWidth(const Value: TBorderWidth);
    procedure Set_Ctl3D(const Value: Boolean);
    procedure Set_ImeMode(const Value: TImeMode);
    procedure Set_ImeName(const Value: TImeName);
    procedure Set_ParentBackground(const Value: Boolean);
    procedure Set_ParentCtl3D(const Value: Boolean);
    procedure Set_TabOrder(const Value: TTabOrder);
    procedure Set_TabStop(const Value: Boolean);

    procedure Set_OnKeyDown(const Value: TRtcKeyEvent);
    procedure Set_OnKeyPress(const Value: TRtcKeyPressEvent);
    procedure Set_OnKeyUp(const Value: TRtcKeyEvent);
    procedure Set_OnEnter(const Value: TNotifyEvent);
    procedure Set_OnExit(const Value: TNotifyEvent);
    procedure Set_BevelInner(const Value: TBevelCut);
    procedure Set_BevelOuter(const Value: TBevelCut);
    procedure Set_OnChange(const Value: TNotifyEvent);

  protected
    FTabOrder: TTabOrder;
    FParentCtl3D: Boolean;
    FCtl3D: Boolean;
    FTabStop: Boolean;
    FBevelOuter: TBevelCut;
    FBevelInner: TBevelCut;
    FBevelEdges: TBevelEdges;
    FBevelKind: TBevelKind;
    FBevelWidth: TBevelWidth;
    FBorderWidth: TBorderWidth;
    FImeMode: TImeMode;
    FImeName: TImeName;

    procedure MyOnKeyDown(xData:TRtcValue);
    procedure MyOnKeyUp(xData:TRtcValue);
    procedure MyOnKeyPress(xData:TRtcValue);
    procedure MyOnEnter(xData:TRtcValue);
    procedure MyOnExit(xData:TRtcValue);
    procedure MyOnChange(xData:TRtcValue);

    property BevelEdges: TBevelEdges read FBevelEdges write Set_BevelEdges default RtcDefault_BevelEdges;
    property BevelInner: TBevelCut read FBevelInner write Set_BevelInner default bvNone;
    property BevelOuter: TBevelCut read FBevelOuter write Set_BevelOuter default bvRaised;
    property BevelKind: TBevelKind read FBevelKind write Set_BevelKind default bkNone;
    property BevelWidth: TBevelWidth read FBevelWidth write Set_BevelWidth default 1;
    property BorderWidth: TBorderWidth read FBorderWidth write Set_BorderWidth default 0;
    property Ctl3D: Boolean read FCtl3D write Set_Ctl3D stored IsCtl3DStored;
    property ImeMode: TImeMode read FImeMode write Set_ImeMode default imDontCare;
    property ImeName: TImeName read FImeName write Set_ImeName;
    property ParentBackground: Boolean read Get_ParentBackground write Set_ParentBackground;
    property ParentCtl3D: Boolean read FParentCtl3D write Set_ParentCtl3D default True;

    property OnKeyDown:TRtcKeyEvent read FOnKeyDown write Set_OnKeyDown;
    property OnKeyUp:TRtcKeyEvent read FOnKeyUp write Set_OnKeyUp;
    property OnKeyPress:TRtcKeyPressEvent read FOnKeyPress write Set_OnKeyPress;

    property OnEnter: TNotifyEvent read FOnEnter write Set_OnEnter;
    property OnExit: TNotifyEvent read FOnExit write Set_OnExit;
    property OnChange: TNotifyEvent read FOnChange write Set_OnChange;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function Showing: Boolean;
    procedure SetFocus;
    function CanFocus: Boolean;

    (* --->>
    function Focused: Boolean; dynamic;

    function ContainsControl(Control: TControl): Boolean;
    function ControlAtPos(const Pos: TPoint; AllowDisabled: Boolean; AllowWinControls: Boolean = False): TControl;

    procedure DisableAlign;
    procedure EnableAlign;

    property DockClientCount: Integer read GetDockClientCount;
    property DockClients[Index: Integer]: TControl read GetDockClients;
    property DoubleBuffered: Boolean read FDoubleBuffered write FDoubleBuffered;

    procedure DockDrop(Source: TDragDockObject; X, Y: Integer); dynamic;
    function FindChildControl(const ControlName: string): TControl;
    procedure FlipChildren(AllLevels: Boolean); dynamic;

    procedure GetTabOrderList(List: TList); dynamic;
    procedure InsertControl(AControl: TControl);
    procedure RemoveControl(AControl: TControl);

    procedure Invalidate; override;
    procedure Realign;
    procedure Repaint; override;
    procedure Update; override;

    procedure ScaleBy(M, D: Integer);
    procedure ScrollBy(DeltaX, DeltaY: Integer);
    procedure UpdateControlState;

    property AlignDisabled: Boolean read GetAlignDisabled;
    property VisibleDockClientCount: Integer read GetVisibleDockClientCount;
    property Brush: TBrush read FBrush;
    property Controls[Index: Integer]: TControl read GetControl;
    property ControlCount: Integer read GetControlCount;
    <<--- *)

    property TabOrder: TTabOrder read Get_TabOrder write Set_TabOrder default -1;
    property TabStop: Boolean read FTabStop write Set_TabStop default False;
    end;

  { This is a base class which can be used to implement
    any TWinControl descendant (server-side code)
    which is used as a parent for other controls (TPanel, for example) }
  TRtcParWinControl=class(TRtcBaseWinControl)
  private
    FCanvas:TRtcCanvas;
    FAction:IRtcBasicAction;
    FPopupMenu:IRtcPopupMenu;

    FOnMouseDown: TRtcMouseEvent;
    FOnMouseUp: TRtcMouseEvent;
    FOnMouseDownMove: TRtcMouseMoveEvent;
    FOnMouseMove: TRtcMouseMoveEvent;
    FOnMouseUpMove: TRtcMouseMoveEvent;
    FOnMouseWheel: TRtcMouseWheelEvent;
    FOnMouseWheelUp: TRtcMouseWheelUpDownEvent;
    FOnMouseWheelDown: TRtcMouseWheelUpDownEvent;
    FOnResize: TNotifyEvent;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FOnMouseEnter: TNotifyEvent;

    FOnKeyDown: TRtcKeyEvent;
    FOnKeyUp: TRtcKeyEvent;
    FOnKeyPress: TRtcKeyPressEvent;
    FOnEnter: TNotifyEvent;
    FOnExit: TNotifyEvent;
    FOnChange: TNotifyEvent;

    FOnDragDrop: TRtcDragDropEvent;
    FOnEndDrag: TRtcEndDragEvent;
    FOnStartDrag: TRtcStartDragEvent;
    FOnDropFiles: TRtcDropFilesEvent;

    function Get_ParentBackground: Boolean;
    function Get_TabOrder: TTabOrder;
    function IsCtl3DStored: Boolean;

    procedure Set_BevelEdges(const Value: TBevelEdges);
    procedure Set_BevelKind(const Value: TBevelKind);
    procedure Set_BevelWidth(const Value: TBevelWidth);
    procedure Set_BorderWidth(const Value: TBorderWidth);
    procedure Set_Ctl3D(const Value: Boolean);
    procedure Set_ImeMode(const Value: TImeMode);
    procedure Set_ImeName(const Value: TImeName);
    procedure Set_ParentBackground(const Value: Boolean);
    procedure Set_ParentCtl3D(const Value: Boolean);
    procedure Set_TabOrder(const Value: TTabOrder);
    procedure Set_TabStop(const Value: Boolean);

    procedure Set_OnKeyDown(const Value: TRtcKeyEvent);
    procedure Set_OnKeyPress(const Value: TRtcKeyPressEvent);
    procedure Set_OnKeyUp(const Value: TRtcKeyEvent);
    procedure Set_OnEnter(const Value: TNotifyEvent);
    procedure Set_OnExit(const Value: TNotifyEvent);
    procedure Set_BevelInner(const Value: TBevelCut);
    procedure Set_BevelOuter(const Value: TBevelCut);
    procedure Set_OnChange(const Value: TNotifyEvent);

    function Get_Cursor: TCursor;
    function Get_HelpContext: THelpContext;
    function Get_HelpKeyword: String;
    function Get_HelpType: THelpType;
    function Get_Hint: string;
    function Get_Tag: Longint;

    procedure Set_Cursor(const Value: TCursor);
    procedure Set_Hint(const Value: string);
    procedure Set_HelpKeyword(const Value: String);
    procedure Set_HelpType(const Value: THelpType);
    procedure Set_Tag(const Value: Longint);
    procedure Set_HelpContext(const Value: THelpContext);

    function Get_Action: IRtcBasicAction;
    function Get_Align: TAlign;
    function Get_Anchors: TAnchors;
    function Get_BiDiMode: TBiDiMode;
    function Get_Color: TColor;
    function Get_Constraints: TRtcSizeConstraints;
    function Get_DesktopFont: Boolean;
    function Get_DragCursor: TCursor;
    function Get_DragKind: TDragKind;
    function Get_DragMode: TDragMode;
    function Get_ParentBiDiMode: Boolean;
    function Get_ParentColor: Boolean;
    function Get_ParentFont: Boolean;
    function Get_ParentShowHint: Boolean;
    function Get_PopupMenu: IRtcPopupMenu;
    function Get_ShowHint: Boolean;
    function Get_Visible: Boolean;
    //function Get_WindowText: PChar;
    function Get_Font: TRtcFont;
    function Get_Enabled: Boolean;

    function IsAnchorsStored: Boolean;
    function IsBiDiModeStored: Boolean;
    function IsColorStored: Boolean;
    function IsFontStored: Boolean;
    function IsOnClickStored: Boolean;
    function IsShowHintStored: Boolean;
    function IsVisibleStored: Boolean;
    function IsEnabledStored: Boolean;

    procedure Set_Enabled(const Value: Boolean);
    procedure Set_Action(const Value: IRtcBasicAction);
    procedure Set_Align(const Value: TAlign);
    procedure Set_Anchors(const Value: TAnchors);
    procedure Set_AutoSize(const Value: Boolean);
    procedure Set_BiDiMode(const Value: TBiDiMode);
    procedure Set_Color(const Value: TColor);
    procedure Set_Constraints(const Value: TRtcSizeConstraints);
    procedure Set_DesktopFont(const Value: Boolean);
    procedure Set_DragCursor(const Value: TCursor);
    procedure Set_DragKind(const Value: TDragKind);
    procedure Set_DragMode(const Value: TDragMode);
    procedure Set_ParentBiDiMode(const Value: Boolean);
    procedure Set_ParentColor(const Value: Boolean);
    procedure Set_ParentFont(const Value: Boolean);
    procedure Set_ParentShowHint(const Value: Boolean);
    procedure Set_PopupMenu(const Value: IRtcPopupMenu);
    procedure Set_ShowHint(const Value: Boolean);
    procedure Set_Visible(const Value: Boolean);
    //procedure Set_WindowText(const Value: PChar);
    procedure Set_Font(const Value: TRtcFont);

    procedure Set_OnClick(const Value: TNotifyEvent);
    procedure Set_OnDblClick(const Value: TNotifyEvent);
    procedure Set_OnMouseDown(const Value: TRtcMouseEvent);
    procedure Set_OnMouseMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnMouseUp(const Value: TRtcMouseEvent);
    procedure Set_OnMouseWheel(const Value: TRtcMouseWheelEvent);
    procedure Set_OnMouseWheelDown(const Value: TRtcMouseWheelUpDownEvent);
    procedure Set_OnMouseWheelUp(const Value: TRtcMouseWheelUpDownEvent);
    procedure Set_OnMouseDownMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnMouseUpMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnResize(const Value: TNotifyEvent);
    procedure Set_OnMouseEnter(const Value: TNotifyEvent);
    procedure Set_OnMouseLeave(const Value: TNotifyEvent);

    procedure Set_OnDragDrop(const Value: TRtcDragDropEvent);
    procedure Set_OnEndDrag(const Value: TRtcEndDragEvent);
    procedure Set_OnStartDrag(const Value: TRtcStartDragEvent);

    function Get_Canvas: TRtcCanvas;
    procedure Set_OnDropFiles(const Value: TRtcDropFilesEvent);

  protected
    FAutoSize: boolean;

    FFnt: TRtcFont;
    FConstr: TRtcSizeConstraints;

    FTabOrder: TTabOrder;
    FParentCtl3D: Boolean;
    FCtl3D: Boolean;
    FTabStop: Boolean;
    FBevelOuter: TBevelCut;
    FBevelInner: TBevelCut;
    FBevelEdges: TBevelEdges;
    FBevelKind: TBevelKind;
    FBevelWidth: TBevelWidth;
    FBorderWidth: TBorderWidth;
    FImeMode: TImeMode;
    FImeName: TImeName;

    procedure MyOnClick(xData:TRtcValue);
    procedure MyOnDblClick(xData:TRtcValue);
    procedure MyOnResize(xData:TRtcValue);

    procedure MyOnDragDrop(xData:TRtcValue);
    procedure MyOnStartDrag(xData:TRtcValue);
    procedure MyOnEndDrag(xData:TRtcValue);
    procedure MyOnDropFiles(xData:TRtcValue);

    procedure MyOnMouseDown(xData:TRtcValue);
    procedure MyOnMouseUp(xData:TRtcValue);
    procedure MyOnMouseMove(xData:TRtcValue);
    procedure MyOnMouseDownMove(xData:TRtcValue);
    procedure MyOnMouseUpMove(xData:TRtcValue);

    procedure MyOnMouseWheel(xData:TRtcValue);
    procedure MyOnMouseWheelUp(xData:TRtcValue);
    procedure MyOnMouseWheelDown(xData:TRtcValue);

    procedure MyOnMouseEnter(xData:TRtcValue);
    procedure MyOnMouseLeave(xData:TRtcValue);

    procedure Screen2Client(var x,y:integer);

    property Canvas: TRtcCanvas read Get_Canvas;

    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

    procedure MyOnKeyDown(xData:TRtcValue);
    procedure MyOnKeyUp(xData:TRtcValue);
    procedure MyOnKeyPress(xData:TRtcValue);
    procedure MyOnEnter(xData:TRtcValue);
    procedure MyOnExit(xData:TRtcValue);
    procedure MyOnChange(xData:TRtcValue);

    property BevelEdges: TBevelEdges read FBevelEdges write Set_BevelEdges default RtcDefault_BevelEdges;
    property BevelInner: TBevelCut read FBevelInner write Set_BevelInner default bvNone;
    property BevelOuter: TBevelCut read FBevelOuter write Set_BevelOuter default bvRaised;
    property BevelKind: TBevelKind read FBevelKind write Set_BevelKind default bkNone;
    property BevelWidth: TBevelWidth read FBevelWidth write Set_BevelWidth default 1;
    property BorderWidth: TBorderWidth read FBorderWidth write Set_BorderWidth default 0;
    property Ctl3D: Boolean read FCtl3D write Set_Ctl3D stored IsCtl3DStored;
    property ImeMode: TImeMode read FImeMode write Set_ImeMode default imDontCare;
    property ImeName: TImeName read FImeName write Set_ImeName;
    property ParentBackground: Boolean read Get_ParentBackground write Set_ParentBackground;
    property ParentCtl3D: Boolean read FParentCtl3D write Set_ParentCtl3D default True;

    property OnKeyDown:TRtcKeyEvent read FOnKeyDown write Set_OnKeyDown;
    property OnKeyUp:TRtcKeyEvent read FOnKeyUp write Set_OnKeyUp;
    property OnKeyPress:TRtcKeyPressEvent read FOnKeyPress write Set_OnKeyPress;

    property OnEnter: TNotifyEvent read FOnEnter write Set_OnEnter;
    property OnExit: TNotifyEvent read FOnExit write Set_OnExit;
    property OnChange: TNotifyEvent read FOnChange write Set_OnChange;

    procedure MyFontChange(Sender:TObject);

  protected

    property Font: TRtcFont read Get_Font write Set_Font stored IsFontStored;

    property Enabled: Boolean read Get_Enabled write Set_Enabled stored IsEnabledStored default True;

    property AutoSize: Boolean read FAutoSize write Set_AutoSize default False;
    property Color: TColor read Get_Color write Set_Color stored IsColorStored default clWindow;
    property DesktopFont: Boolean read Get_DesktopFont write Set_DesktopFont default False;
    property DragKind: TDragKind read Get_DragKind write Set_DragKind default dkDrag;
    property DragCursor: TCursor read Get_DragCursor write Set_DragCursor default crDrag;
    property DragMode: TDragMode read Get_DragMode write Set_DragMode default dmManual;
    property ParentBiDiMode: Boolean read Get_ParentBiDiMode write Set_ParentBiDiMode default True;
    property ParentColor: Boolean read Get_ParentColor write Set_ParentColor default True;
    property ParentFont: Boolean read Get_ParentFont write Set_ParentFont default True;
    property ParentShowHint: Boolean read Get_ParentShowHint write Set_ParentShowHint default True;
    property PopupMenu: IRtcPopupMenu read Get_PopupMenu write Set_PopupMenu;
    //property WindowText: PChar read Get_WindowText write Set_WindowText;

    property OnClick: TNotifyEvent read FOnClick write Set_OnClick stored IsOnClickStored;
    property OnDblClick: TNotifyEvent read FOnDblClick write Set_OnDblClick;

    property OnDragDrop: TRtcDragDropEvent read FOnDragDrop write Set_OnDragDrop;
    property OnStartDrag: TRtcStartDragEvent read FOnStartDrag write Set_OnStartDrag;
    property OnEndDrag: TRtcEndDragEvent read FOnEndDrag write Set_OnEndDrag;
    property OnDropFiles: TRtcDropFilesEvent read FOnDropFiles write Set_OnDropFiles;

    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write Set_OnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write Set_OnMouseLeave;

    property OnMouseDown: TRtcMouseEvent read FOnMouseDown write Set_OnMouseDown;
    property OnMouseMove: TRtcMouseMoveEvent read FOnMouseMove write Set_OnMouseMove;
    property OnMouseDownMove: TRtcMouseMoveEvent read FOnMouseDownMove write Set_OnMouseDownMove;
    property OnMouseUpMove: TRtcMouseMoveEvent read FOnMouseUpMove write Set_OnMouseUpMove;
    property OnMouseUp: TRtcMouseEvent read FOnMouseUp write Set_OnMouseUp;

    property OnMouseWheel: TRtcMouseWheelEvent read FOnMouseWheel write Set_OnMouseWheel;
    property OnMouseWheelDown: TRtcMouseWheelUpDownEvent read FOnMouseWheelDown write Set_OnMouseWheelDown;
    property OnMouseWheelUp: TRtcMouseWheelUpDownEvent read FOnMouseWheelUp write Set_OnMouseWheelUp;

    property OnResize: TNotifyEvent read FOnResize write Set_OnResize;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Hide;
    procedure Show;

    procedure BringToFront;
    procedure SendToBack;

    procedure Refresh;

    procedure BeginDrag(Immediate: Boolean; Threshold: Integer = -1);
    function Dragging: Boolean;
    procedure EndDrag(Drop: Boolean);
    procedure DragDrop(Source: TObject; X, Y: Integer); override;

    function ClientToScreen(const Point: TPoint): TPoint;
    function ScreenToClient(const Point: TPoint): TPoint;

    function Showing: Boolean;
    procedure SetFocus; override;
    function CanFocus: Boolean; override;

    property Action: IRtcBasicAction read Get_Action write Set_Action;
    property Align: TAlign read Get_Align write Set_Align default alNone;
    property Anchors: TAnchors read Get_Anchors write Set_Anchors stored IsAnchorsStored default RtcDefault_Anchors;
    property BiDiMode: TBiDiMode read Get_BiDiMode write Set_BiDiMode stored IsBiDiModeStored;
    property Constraints: TRtcSizeConstraints read Get_Constraints write Set_Constraints;
    property ShowHint: Boolean read Get_ShowHint write Set_ShowHint stored IsShowHintStored;
    property Visible: Boolean read Get_Visible write Set_Visible stored IsVisibleStored default True;

    property TabOrder: TTabOrder read Get_TabOrder write Set_TabOrder default -1;
    property TabStop: Boolean read FTabStop write Set_TabStop default False;

  published
    property Tag: Longint read Get_Tag write Set_Tag default 0;
    property Cursor: TCursor read Get_Cursor write Set_Cursor default crDefault;
    property Hint: string read Get_Hint write Set_Hint;
    property HelpType: THelpType read Get_HelpType write Set_HelpType default htContext;
    property HelpKeyword: String read Get_HelpKeyword write Set_HelpKeyword;
    property HelpContext: THelpContext read Get_HelpContext write Set_HelpContext default 0;
    end;

function RtcToShiftState(rec:TRtcRecord):TShiftState;

implementation

function RtcToShiftState(rec:TRtcRecord):TShiftState;
  begin
  Result:=[];
  if assigned(rec) then
    begin
    if rec.asBoolean['s'] then Include(Result,ssShift);
    if rec.asBoolean['a'] then Include(Result,ssAlt);
    if rec.asBoolean['c'] then Include(Result,ssCtrl);
    if rec.asBoolean['l'] then Include(Result,ssLeft);
    if rec.asBoolean['r'] then Include(Result,ssRight);
    if rec.asBoolean['m'] then Include(Result,ssMiddle);
    if rec.asBoolean['d'] then Include(Result,ssDouble);
    end;
  end;

{ TRtcControlScrollBar }

constructor TRtcControlScrollBar.Create;
  begin
  inherited;
  Par_ButtonSize:=0;
  Par_Color:=clBtnHighlight;
  Par_Increment:=8;
  Par_Margin:=0;
  Par_ParentColor:=True;
  Par_Position:=0;
  Par_Range:=0;
  Par_Smooth:=False;
  Par_Size:=0;
  Par_Style:=ssRegular;
  Par_ThumbSize:=0;
  Par_Tracking:=False;
  Par_Visible:=True;
  end;

procedure TRtcControlScrollBar.Assign(Source: TPersistent);
  var
    src:TRtcControlScrollBar;
  begin
  if Source is TRtcControlScrollBar then
    begin
    src:=TRtcControlScrollBar(Source);
    ButtonSize:=src.ButtonSize;
    Color:=src.Color;
    Margin:=src.Margin;
    ParentColor:=src.ParentColor;
    Position:=src.Position;
    Smooth:=src.Smooth;
    Size:=src.Size;
    Style:=src.Style;
    ThumbSize:=src.ThumbSize;
    Tracking:=src.Tracking;
    Visible:=src.Visible;
    if not IsRangeStored then
      Range:=src.Range;
    if not IsIncrementStored then
      Increment:=src.Increment;
    end;
  end;

function TRtcControlScrollBar.Get_ButtonSize: Integer;
  begin
  Result:=Par_ButtonSize;
  end;

function TRtcControlScrollBar.Get_Color: TColor;
  begin
  Result:=Par_Color;
  end;

function TRtcControlScrollBar.Get_Increment: TRtcScrollBarInc;
  begin
  Result:=Par_Increment;
  end;

function TRtcControlScrollBar.Get_Margin: Word;
  begin
  Result:=Par_Margin;
  end;

function TRtcControlScrollBar.Get_ParentColor: Boolean;
  begin
  Result:=Par_ParentColor;
  end;

function TRtcControlScrollBar.Get_Position: Integer;
  begin
  Result:=Par_Position;
  end;

function TRtcControlScrollBar.Get_Range: Integer;
  begin
  Result:=Par_Range;
  end;

function TRtcControlScrollBar.Get_Size: Integer;
  begin
  Result:=Par_Size;
  end;

function TRtcControlScrollBar.Get_Smooth: Boolean;
  begin
  Result:=Par_Smooth;
  end;

function TRtcControlScrollBar.Get_Style: TRtcScrollBarStyle;
  begin
  Result:=Par_Style;
  end;

function TRtcControlScrollBar.Get_ThumbSize: Integer;
  begin
  Result:=Par_ThumbSize;
  end;

function TRtcControlScrollBar.Get_Tracking: Boolean;
  begin
  Result:=Par_Tracking;
  end;

function TRtcControlScrollBar.Get_Visible: Boolean;
  begin
  Result:=Par_Visible;
  end;

function TRtcControlScrollBar.IsIncrementStored: Boolean;
  begin
  Result:=not Smooth;
  end;

function TRtcControlScrollBar.IsRangeStored: Boolean;
  begin
  Result:=not FAutoScroll;
  end;

procedure TRtcControlScrollBar.Set_ButtonSize(const Value: Integer);
  begin
  if Value<>Par_ButtonSize then
    begin
    Par_ButtonSize:=Value;
    _Com.Prop(FullName+'buttonsize',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_Color(const Value: TColor);
  begin
  if Value<>Par_Color then
    begin
    Par_Color:=Value;
    _Com.Prop(FullName+'color',Ord(Value));
    end;
  end;

procedure TRtcControlScrollBar.Set_Increment(const Value: TRtcScrollBarInc);
  begin
  if Value<>Par_Increment then
    begin
    Par_Increment:=Value;
    _Com.Prop(FullName+'increment',Ord(Value));
    end;
  end;

procedure TRtcControlScrollBar.Set_Margin(const Value: Word);
  begin
  if Value<>Par_Margin then
    begin
    Par_Margin:=Value;
    _Com.Prop(FullName+'margin',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_ParentColor(const Value: Boolean);
  begin
  if Value<>Par_ParentColor then
    begin
    Par_ParentColor:=Value;
    _Com.Prop(FullName+'parentcolor',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_Position(const Value: Integer);
  begin
  if Value<>Par_Position then
    begin
    Par_Position:=Value;
    _Com.Prop(FullName+'position',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_Range(const Value: Integer);
  begin
  if Value<>Par_Range then
    begin
    Par_Range:=Value;
    _Com.Prop(FullName+'range',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_Size(const Value: Integer);
  begin
  if Value<>Par_Size then
    begin
    Par_Size:=Value;
    _Com.Prop(FullName+'size',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_Smooth(const Value: Boolean);
  begin
  if Value<>Par_Smooth then
    begin
    Par_Smooth:=Value;
    _Com.Prop(FullName+'smooth',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_Style(const Value: TRtcScrollBarStyle);
  begin
  if Value<>Par_Style then
    begin
    Par_Style:=Value;
    _Com.Prop(FullName+'style',Ord(Value));
    end;
  end;

procedure TRtcControlScrollBar.Set_ThumbSize(const Value: Integer);
  begin
  if Value<>Par_ThumbSize then
    begin
    Par_ThumbSize:=Value;
    _Com.Prop(FullName+'thumbsize',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_Tracking(const Value: Boolean);
  begin
  if Value<>Par_Tracking then
    begin
    Par_Tracking:=Value;
    _Com.Prop(FullName+'tracking',Value);
    end;
  end;

procedure TRtcControlScrollBar.Set_Visible(const Value: Boolean);
  begin
  if Value<>Par_Visible then
    begin
    Par_Visible:=Value;
    _Com.Prop(FullName+'visible',Value);
    end;
  end;

{ TRtcSizeConstraints }

procedure TRtcSizeConstraints.Assign(Source: TPersistent);
  var
    src:TRtcSizeConstraints;
  begin
  if Source is TRtcSizeConstraints then
    begin
    src:=TRtcSizeConstraints(Source);
    MaxHeight:=src.MaxHeight;
    MaxWidth:=src.MaxWidth;
    MinHeight:=src.MinHeight;
    MinWidth:=src.MinWidth;
    end;
  end;

function TRtcSizeConstraints.Get_MaxHeight: TConstraintSize;
  begin
  Result:=Par.MaxHeight;
  end;

function TRtcSizeConstraints.Get_MaxWidth: TConstraintSize;
  begin
  Result:=Par.MaxWidth;
  end;

function TRtcSizeConstraints.Get_MinHeight: TConstraintSize;
  begin
  Result:=Par.MinHeight;
  end;

function TRtcSizeConstraints.Get_MinWidth: TConstraintSize;
  begin
  Result:=Par.MinWidth;
  end;

procedure TRtcSizeConstraints.Set_MaxHeight(const Value: TConstraintSize);
  begin
  if Value<>Par.MaxHeight then
    begin
    Par.MaxHeight:=Value;
    _Com.Prop(FullName+'h2',Value);
    end;
  end;

procedure TRtcSizeConstraints.Set_MaxWidth(const Value: TConstraintSize);
  begin
  if Value<>Par.MaxWidth then
    begin
    Par.MaxWidth:=Value;
    _Com.Prop(FullName+'w2',Value);
    end;
  end;

procedure TRtcSizeConstraints.Set_MinHeight(const Value: TConstraintSize);
  begin
  if Value<>Par.MinHeight then
    begin
    Par.MinHeight:=Value;
    _Com.Prop(FullName+'h1',Value);
    end;
  end;

procedure TRtcSizeConstraints.Set_MinWidth(const Value: TConstraintSize);
  begin
  if Value<>Par.MinWidth then
    begin
    Par.MinWidth:=Value;
    _Com.Prop(FullName+'w1',Value);
    end;
  end;

{ TRtcComponent }

constructor TRtcComponent.Create(AOwner: TComponent);
  begin
  inherited;
  FCanvas:=nil;
  end;

destructor TRtcComponent.Destroy;
  begin
  if assigned(FCanvas) then FreeAndNil(FCanvas);
  inherited;
  end;

procedure TRtcComponent.Set_Tag(const Value: Longint);
  begin
  if inherited Tag<>Value then
    begin
    inherited Tag := Value;
    _Com.Prop('tag',Value);
    end;
  end;

function TRtcComponent.Get_Tag: Longint;
  begin
  Result:=inherited Tag;
  end;

function TRtcComponent.Get_Canvas: TRtcCanvas;
  begin
  if not assigned(FCanvas) then
    begin
    FCanvas:=TRtcCanvas.Create;
    FCanvas._Com:=_Com;
    FCanvas.FullName:='canvas.';
    end;
  Result:=FCanvas;
  end;

{ TRtcControl }

constructor TRtcControl.Create(AOwner: TComponent);
  begin
  inherited;
  FCanvas:=nil;

  FFnt:=TRtcFont.Create(inherited Font);
  FFnt._Com:=_Com;
  FFnt.FullName:='font.';
  FFnt.OnChange:=MyFontChange;

  FConstr:=TRtcSizeConstraints.Create;
  FConstr._Com:=_Com;
  FConstr.FullName:='cnstr.';
  FConstr.Par:=inherited Constraints;

  FAutoSize:=False;
  end;

destructor TRtcControl.Destroy;
  begin
  FAction:=nil;
  FPopupMenu:=nil;
  if assigned(FFnt) then FreeAndNil(FFnt);
  if assigned(FConstr) then FreeAndNil(FConstr);
  if assigned(FCanvas) then FreeAndNil(FCanvas);
  inherited;
  end;

function TRtcControl.Get_Font: TRtcFont;
  begin
  Result:=FFnt;
  end;

function TRtcControl.Get_Constraints: TRtcSizeConstraints;
  begin
  Result:=FConstr;
  end;

function TRtcControl.Get_Cursor: TCursor;
  begin
  Result:=inherited Cursor;
  end;

function TRtcControl.Get_HelpContext: THelpContext;
  begin
  Result:=inherited HelpContext;
  end;

function TRtcControl.Get_HelpKeyword: String;
  begin
  Result:=inherited HelpKeyword;
  end;

function TRtcControl.Get_HelpType: THelpType;
  begin
  Result:=inherited HelpType;
  end;

function TRtcControl.Get_Hint: string;
  begin
  Result:=inherited Hint;
  end;

function TRtcControl.Get_Tag: Longint;
  begin
  Result:=inherited Tag;
  end;

function TRtcControl.Get_Align: TAlign;
  begin
  Result:=inherited Align;
  end;

function TRtcControl.Get_Anchors: TAnchors;
  begin
  Result:=inherited Anchors;
  end;

function TRtcControl.Get_BiDiMode: TBiDiMode;
  begin
  Result:=inherited BiDiMode;
  end;

function TRtcControl.Get_Color: TColor;
  begin
  Result:=inherited Color;
  end;

function TRtcControl.Get_DesktopFont: Boolean;
  begin
  Result:=inherited DesktopFont;
  end;

function TRtcControl.Get_DragCursor: TCursor;
  begin
  Result:=inherited DragCursor;
  end;

function TRtcControl.Get_DragKind: TDragKind;
  begin
  Result:=inherited DragKind;
  end;

function TRtcControl.Get_DragMode: TDragMode;
  begin
  Result:=inherited DragMode;
  end;

function TRtcControl.Get_ParentBiDiMode: Boolean;
  begin
  Result:=inherited ParentBiDiMode;
  end;

function TRtcControl.Get_ParentColor: Boolean;
  begin
  Result:=inherited ParentColor;
  end;

function TRtcControl.Get_ParentFont: Boolean;
  begin
  Result:=inherited ParentFont;
  end;

function TRtcControl.Get_ParentShowHint: Boolean;
  begin
  Result:=inherited ParentShowHint;
  end;

function TRtcControl.Get_ShowHint: Boolean;
  begin
  Result:=inherited ShowHint;
  end;

function TRtcControl.Get_Visible: Boolean;
  begin
  Result:=inherited Visible;
  end;

function TRtcControl.Get_Enabled: Boolean;
  begin
  Result:=inherited Enabled;
  end;

function TRtcControl.IsAnchorsStored: Boolean;
  begin
  Result:=Anchors <> AnchorAlign[Align];
  // Result:=inherited IsAnchorsStored;
  end;

function TRtcControl.IsBiDiModeStored: Boolean;
  begin
  Result:=not ParentBiDiMode;
  // Result:=inherited IsBiDiModeStored;
  end;

function TRtcControl.IsColorStored: Boolean;
  begin
  Result:=not ParentColor;
  end;

function TRtcControl.IsFontStored: Boolean;
  begin
  Result:=not ParentFont and not DesktopFont;
  end;

function TRtcControl.IsShowHintStored: Boolean;
  begin
  Result:=not ParentShowHint;
  end;

function TRtcControl.IsVisibleStored: Boolean;
  begin
  Result:=True;
  // Result:=inherited IsVisibleStored;
  end;

function TRtcControl.IsOnClickStored: Boolean;
  begin
  Result:=True;
  // Result:=inherited IsOnClickStored;
  end;

function TRtcControl.IsEnabledStored: Boolean;
  begin
  Result:=True;
  // Result:=inherited IsEnabledStored;
  end;

procedure TRtcControl.Set_Font(const Value: TRtcFont);
  begin
  FFnt.Assign(Value);
  end;

procedure TRtcControl.Set_Constraints(const Value: TRtcSizeConstraints);
  begin
  FConstr.Assign(Value);
  end;

procedure TRtcControl.Set_Tag(const Value: Longint);
  begin
  if inherited Tag<>Value then
    begin
    inherited Tag := Value;
    _Com.Prop('tag',Value);
    end;
  end;

procedure TRtcControl.Set_Cursor(const Value: TCursor);
  begin
  if inherited Cursor<>Value then
    begin
    inherited Cursor := Value;
    _Com.Prop('cursor',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_Hint(const Value: string);
  begin
  if inherited Hint<>Value then
    begin
    inherited Hint := Value;
    _Com.Prop('hint',Value);
    end;
  end;

procedure TRtcControl.Set_HelpKeyword(const Value: String);
  begin
  if inherited HelpKeyword<>Value then
    begin
    inherited HelpKeyword := Value;
    _Com.Prop('helpkeyword',Value);
    end;
  end;

procedure TRtcControl.Set_HelpType(const Value: THelpType);
  begin
  if inherited HelpType<>Value then
    begin
    inherited HelpType := Value;
    _Com.Prop('helptype',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_HelpContext(const Value: THelpContext);
  begin
  if inherited HelpContext<>Value then
    begin
    inherited HelpContext := Value;
    _Com.Prop('helpcontext',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_Align(const Value: TAlign);
  begin
  if inherited Align<>Value then
    begin
    inherited Align:=Value;
    _Com.Prop('align',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_Anchors(const Value: TAnchors);
  var
    rec:TRtcRecord;
  begin
  if inherited Anchors<>Value then
    begin
    inherited Anchors:=Value;
    rec:=TRtcRecord.Create;
    if (akLeft in Anchors) then rec.asBoolean['l']:=True;
    if (akTop in Anchors) then rec.asBoolean['t']:=True;
    if (akRight in Anchors) then rec.asBoolean['r']:=True;
    if (akBottom in Anchors) then rec.asBoolean['b']:=True;
    _Com.Prop('anchors',rec);
    end;
  end;

procedure TRtcControl.Set_AutoSize(const Value: Boolean);
  begin
  if FAutoSize<>Value then
    begin
    FAutoSize:=Value;
    _Com.Prop('autosize',Value);
    end;
  end;

procedure TRtcControl.Set_BiDiMode(const Value: TBiDiMode);
  begin
  if inherited BiDiMode<>Value then
    begin
    inherited BiDiMode:=Value;
    _Com.Prop('bidimode',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_Color(const Value: TColor);
  begin
  if inherited Color<>Value then
    begin
    inherited Color:=Value;
    _Com.Prop('color',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_DesktopFont(const Value: Boolean);
  begin
  if inherited DesktopFont<>Value then
    begin
    inherited DesktopFont:=Value;
    _Com.Prop('desktopfont',Value);
    end;
  end;

procedure TRtcControl.Set_DragCursor(const Value: TCursor);
  begin
  if inherited DragCursor<>Value then
    begin
    inherited DragCursor:=Value;
    _Com.Prop('dragcursor',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_DragKind(const Value: TDragKind);
  begin
  if inherited DragKind<>Value then
    begin
    inherited DragKind:=Value;
    _COm.Prop('dragkind',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_DragMode(const Value: TDragMode);
  begin
  if inherited DragMode<>Value then
    begin
    inherited DragMode:=Value;
    _Com.Prop('dragmode',Ord(Value));
    end;
  end;

procedure TRtcControl.Set_ParentBiDiMode(const Value: Boolean);
  begin
  if inherited ParentBiDiMode<>Value then
    begin
    inherited ParentBiDiMode:=Value;
    _Com.Prop('parentbidimode',Value);
    end;
  end;

procedure TRtcControl.Set_ParentColor(const Value: Boolean);
  begin
  if inherited ParentColor<>Value then
    begin
    inherited ParentColor:=Value;
    _Com.Prop('parentcolor',Value);
    end;
  end;

procedure TRtcControl.Set_ParentFont(const Value: Boolean);
  begin
  if inherited ParentFont<>Value then
    begin
    inherited ParentFont:=Value;
    _Com.Prop('parentfont',Value);
    end;
  end;

procedure TRtcControl.Set_ParentShowHint(const Value: Boolean);
  begin
  if inherited ParentShowHint<>Value then
    begin
    inherited ParentShowHint:=Value;
    _Com.Prop('parentshowhint',Value);
    end;
  end;

procedure TRtcControl.Set_ShowHint(const Value: Boolean);
  begin
  if inherited ShowHint<>Value then
    begin
    inherited ShowHint:=Value;
    _Com.Prop('showhint',Value);
    end;
  end;

procedure TRtcControl.Set_Visible(const Value: Boolean);
  begin
  if inherited Visible<>Value then
    begin
    inherited Visible:=Value;
    _Com.Prop('visible',Value);
    end;
  end;

procedure TRtcControl.Set_Enabled(const Value: Boolean);
  begin
  if inherited Enabled<>Value then
    begin
    inherited Enabled:=Value;
    _Com.Prop('enabled',Value);
    end;
  end;

procedure TRtcControl.Set_OnClick(const Value: TNotifyEvent);
  begin
  FOnClick:=Value;
  if assigned(FOnClick) then
    _Com.Event('onclick',MyOnClick)
  else
    _Com.Event('onclick',nil);
  end;

procedure TRtcControl.Set_OnDblClick(const Value: TNotifyEvent);
  begin
  FOnDblClick:=Value;
  if assigned(FOnDblClick) then
    _Com.Event('ondblclick',MyOnDblClick)
  else
    _Com.Event('ondblclick',nil);
  end;

procedure TRtcControl.Set_OnMouseDown(const Value: TRtcMouseEvent);
  begin
  FOnMouseDown:=Value;
  if assigned(FOnMouseDown) then
    _Com.Event('onmousedown',MyOnMouseDown)
  else
    _Com.Event('onmousedown',nil);
  end;

procedure TRtcControl.Set_OnMouseMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseMove:=Value;
  if assigned(FOnMouseMove) then
    _Com.Event('onmousemove',MyOnMouseMove)
  else
    _Com.Event('onmousemove',nil);
  end;

procedure TRtcControl.Set_OnMouseUp(const Value: TRtcMouseEvent);
  begin
  FOnMouseUp:=Value;
  if assigned(FOnMouseUp) then
    _Com.Event('onmouseup',MyOnMouseUp)
  else
    _Com.Event('onmouseup',nil);
  end;

procedure TRtcControl.Set_OnMouseWheel(const Value: TRtcMouseWheelEvent);
  begin
  FOnMouseWheel:=Value;
  if assigned(FOnMouseWheel) then
    _Com.Event('onmousewheel',MyOnMouseWheel)
  else
    _Com.Event('onmousewheel',nil);
  end;

procedure TRtcControl.Set_OnMouseWheelDown(const Value: TRtcMouseWheelUpDownEvent);
  begin
  FOnMouseWheelDown:=Value;
  if assigned(FOnMouseWheelDown) then
    _Com.Event('onmousewheeldown',MyOnMouseWheelDown)
  else
    _Com.Event('onmousewheeldown',nil);
  end;

procedure TRtcControl.Set_OnMouseWheelUp(const Value: TRtcMouseWheelUpDownEvent);
  begin
  FOnMouseWheelUp:=Value;
  if assigned(FOnMouseWheelUp) then
    _Com.Event('onmousewheelup',MyOnMouseWheelUp)
  else
    _Com.Event('onmousewheelup',nil);
  end;

procedure TRtcControl.Set_OnResize(const Value: TNotifyEvent);
  begin
  FOnResize:=Value;
  if assigned(FOnResize) then
    _Com.Event('onresize',MyOnResize)
  else
    _Com.Event('onresize',nil);
  end;

procedure TRtcControl.Set_OnMouseDownMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseDownMove:=Value;
  if assigned(FOnMouseDownMove) then
    _Com.Event('onmousedownmove',MyOnMouseDownMove)
  else
    _Com.Event('onmousedownmove',nil);
  end;

procedure TRtcControl.Set_OnMouseUpMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseUpMove:=Value;
  if assigned(FOnMouseUpMove) then
    _Com.Event('onmouseupmove',MyOnMouseUpMove)
  else
    _Com.Event('onmouseupmove',nil);
  end;

procedure TRtcControl.Set_OnMouseEnter(const Value: TNotifyEvent);
  begin
  FOnMouseEnter := Value;
  if assigned(FOnMouseEnter) then
    _Com.Event('onmouseenter',MyOnMouseEnter)
  else
    _Com.Event('onmouseenter',nil);
  end;

procedure TRtcControl.Set_OnMouseLeave(const Value: TNotifyEvent);
  begin
  FOnMouseLeave := Value;
  if assigned(FOnMouseLeave) then
    _Com.Event('onmouseleave',MyOnMouseLeave)
  else
    _Com.Event('onmouseleave',nil);
  end;

procedure TRtcControl.Set_OnDragDrop(const Value: TRtcDragDropEvent);
  begin
  FOnDragDrop := Value;
  if assigned(FOnDragDrop) then
    _Com.Event('ondragdrop',MyOnDragDrop)
  else
    _Com.Event('ondragdrop',nil);
  end;

procedure TRtcControl.Set_OnStartDrag(const Value: TRtcStartDragEvent);
  begin
  FOnStartDrag := Value;
  if assigned(FOnStartDrag) then
    _Com.Event('onstartdrag',MyOnStartDrag)
  else
    _Com.Event('onstartdrag',nil);
  end;

procedure TRtcControl.Set_OnEndDrag(const Value: TRtcEndDragEvent);
  begin
  FOnEndDrag := Value;
  if assigned(FOnEndDrag) then
    _Com.Event('onenddrag',MyOnEndDrag)
  else
    _Com.Event('onenddrag',nil);
  end;

procedure TRtcControl.MyOnStartDrag(xData: TRtcValue);
  var
    o:TDragObject;
  begin
  if assigned(FOnStartDrag) and Visible then
    begin
    o:=nil;
    FOnStartDrag(self,o);
    end;
  end;

procedure TRtcControl.MyOnEndDrag(xData: TRtcValue);
  var
    src:TObject;
    X,Y:integer;
  begin
  if assigned(FOnEndDrag) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        if isType['o']=rtc_Record then
          src:=_Com.FindByUID(asRecord['o'])
        else
          src:=nil;
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnEndDrag(self,src,X,Y);
      end;
    end;
  end;

procedure TRtcControl.MyOnClick(xData: TRtcValue);
  begin
  if assigned(FOnClick) and Enabled and Visible then
    FOnClick(self);
  end;

procedure TRtcControl.MyOnDblClick(xData: TRtcValue);
  begin
  if assigned(FOnDblClick) and Enabled and Visible then
    FOnDblClick(self);
  end;

procedure TRtcControl.MyOnResize(xData: TRtcValue);
  begin
  if assigned(FOnResize) then
    FOnResize(self);
  end;

procedure TRtcControl.MyOnDragDrop(xData: TRtcValue);
  var
    src:TObject;
    X,Y:integer;
  begin
  if assigned(FOnDragDrop) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        if isType['o']=rtc_Record then
          src:=_Com.FindByUID(asRecord['o'])
        else
          src:=nil;
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnDragDrop(self,src,X,Y);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseDown(xData: TRtcValue);
  var
    button:TMouseButton;
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseDown) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        button:=TMouseButton(asInteger['b']);
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseDown(self,button,shift,x,y);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseUp(xData: TRtcValue);
  var
    button:TMouseButton;
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseUp) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        button:=TMouseButton(asInteger['b']);
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseUp(self,button,shift,x,y);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseMove) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseMove(self,shift,x,y);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseDownMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseDownMove) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseDownMove(self,shift,x,y);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseUpMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseUpMove) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseUpMove(self,shift,x,y);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseWheel(xData: TRtcValue);
  var
    shift:TShiftState;
    wdelta:integer;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheel) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        wdelta:=asInteger['w'];
        end;
      handled:=False;
      FOnMouseWheel(self,shift,wdelta,mpos,handled);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseWheelDown(xData: TRtcValue);
  var
    shift:TShiftState;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheelDown) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        end;
      handled:=False;
      FOnMouseWheelDown(self,shift,mpos,handled);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseWheelUp(xData: TRtcValue);
  var
    shift:TShiftState;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheelUp) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        end;
      handled:=False;
      FOnMouseWheelUp(self,shift,mpos,handled);
      end;
    end;
  end;

procedure TRtcControl.MyOnMouseEnter(xData: TRtcValue);
  begin
  if assigned(FOnMouseEnter) and Visible then
    FOnMouseEnter(self);
  end;

procedure TRtcControl.MyOnMouseLeave(xData: TRtcValue);
  begin
  if assigned(FOnMouseLeave) then
    FOnMouseLeave(self);
  end;

procedure TRtcControl.Screen2Client(var x, y: integer);
  var
    wc:TWinControl;
  begin
  wc:=Parent;
  if assigned(wc) then
    begin
    Dec(x,Left);
    Dec(y,Top);
    repeat
      if assigned(wc.Parent) then
        begin
        Dec(x,wc.Left);
        Dec(y,wc.Top);
        wc:=wc.Parent;
        end
      else
        wc:=nil;
      until not assigned(wc);
    end;
  end;

function TRtcControl.Get_Canvas: TRtcCanvas;
  begin
  if not assigned(FCanvas) then
    begin
    FCanvas:=TRtcCanvas.Create;
    FCanvas._Com:=_Com;
    FCanvas.FullName:='canvas.';
    end;
  Result:=FCanvas;
  end;

function TRtcControl.Get_Action: IRtcBasicAction;
  begin
  Result:=FAction;
  end;

function TRtcControl.Get_PopupMenu: IRtcPopupMenu;
  begin
  Result:=FPopupMenu;
  end;

procedure TRtcControl.Set_Action(const Value: IRtcBasicAction);
  begin
  if (Value<>FAction) then
    begin
    FAction:=Value;
    _Com.Link('action',Value);
    end;
  end;

procedure TRtcControl.Set_PopupMenu(const Value: IRtcPopupMenu);
  begin
  if (Value<>FPopupMenu) then
    begin
    FPopupMenu:=Value;
    _Com.Link('popupmenu',Value);
    end;
  end;

procedure TRtcControl.Notification(AComponent: TComponent; Operation: TOperation);
  begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if assigned(PopupMenu) and (AComponent = PopupMenu._Self) then PopupMenu := nil
    else if assigned(Action) and (AComponent = Action._Self) then Action := nil;
  end;

procedure TRtcControl.Hide;
  begin
  Visible:=False;
  end;

procedure TRtcControl.Show;
  begin
  Visible:=True;
  end;

procedure TRtcControl.BringToFront;
  begin
  if (csDesigning in ComponentState) then
    TControl(self).BringToFront;
  _Com.Send('tofront',nil);
  end;

procedure TRtcControl.SendToBack;
  begin
  if (csDesigning in ComponentState) then
    TControl(Self).SendToBack;
  _Com.Send('toback',nil);
  end;

procedure TRtcControl.Refresh;
  begin
  if (csDesigning in ComponentState) then
    TControl(Self).Refresh;
  _Com.Send('refresh',nil);
  end;

procedure TRtcControl.BeginDrag(Immediate: Boolean; Threshold: Integer);
  var
    rec:TRtcRecord;
  begin
  rec:=TRtcRecord.Create;
  rec.asBoolean['i']:=Immediate;
  rec.asInteger['t']:=Threshold;
  _Com.Send('begindrag',rec);
  end;

function TRtcControl.Dragging: Boolean;
  begin
  Result:=False;
  end;

procedure TRtcControl.EndDrag(Drop: Boolean);
  begin
  _Com.Send('enddrag',Drop);
  end;

procedure TRtcControl.DragDrop(Source: TObject; X, Y: Integer);
  begin
  if assigned(FOnDragDrop) then FOnDragDrop(self,Source,X,Y);
  end;

function TRtcControl.ClientToScreen(const Point: TPoint): TPoint;
  var
    LParent: TWinControl;
  begin
  Result := Point;
  Inc(Result.X, Left);
  Inc(Result.Y, Top);
  LParent := Parent;
  while LParent <> nil do
    begin
    Inc(Result.X, LParent.Left);
    Inc(Result.Y, LParent.Top);
    LParent := LParent.Parent;
    end;
  end;

function TRtcControl.ScreenToClient(const Point: TPoint): TPoint;
  var
    LParent: TWinControl;
  begin
  Result := Point;
  Dec(Result.X, Left);
  Dec(Result.Y, Top);
  LParent := Parent;
  while LParent <> nil do
    begin
    Dec(Result.X, LParent.Left);
    Dec(Result.Y, LParent.Top);
    LParent := LParent.Parent;
    end;
  end;

procedure TrtcControl.MyOnDropFiles(xData: TRtcValue);
  var
    a:integer;
    arr:TRtcArray;
    myarr:array of string;
  begin
  if assigned(FOnDropFiles) and (xData.isType=rtc_Array) then
    begin
    arr:=xData.asArray;
    try
      SetLength(myarr,arr.Count);
      for a:=0 to arr.Count-1 do
        myarr[a]:=arr.asText[a];
      FOnDropFiles(self,myarr);
    finally
      SetLength(myarr,0);
      end;
    end;
  end;

procedure TrtcControl.Set_OnDropFiles(const Value: TRtcDropFilesEvent);
  begin
  FOnDropFiles := Value;
  if assigned(FOnDropFiles) then
    _Com.Event('ondropfiles',MyOnDropFiles)
  else
    _Com.Event('ondropfiles',nil);
  end;

procedure TRtcControl.MyFontChange(Sender: TObject);
  begin
  ParentFont:=False;
  end;

{ TRtcWinControl }

constructor TRtcWinControl.Create(AOwner: TComponent);
  begin
  inherited;
  FBevelEdges:=RtcDefault_BevelEdges;
  FBevelInner:=bvNone;
  FBevelOuter:=bvRaised;
  FBevelKind:=bkNone;
  FBevelWidth:=1;
  FBorderWidth:=0;
  FCtl3D:=True;
  FImeMode:=imDontCare;
  FImeName:='';
  FParentCtl3D:=True;
  FTabOrder:=-1;
  FTabStop:=False;
  end;

destructor TRtcWinControl.Destroy;
  begin
  FAction:=nil;
  FPopupMenu:=nil;
  inherited;
  end;

function TRtcWinControl.Get_ParentBackground: Boolean;
  begin
  {$IFDEF IDE_7up}
  Result := csParentBackground in ControlStyle;
  {$ELSE}
  Result := False;
  {$ENDIF}
  end;

procedure TRtcWinControl.Set_TabOrder(const Value: TTabOrder);
  begin
  if FTabOrder<>Value then
    begin
    FTabOrder := Value;
    _Com.Prop('taborder',Ord(Value));
    end;
  end;

function TRtcWinControl.Get_TabOrder: TTabOrder;
  begin
  if (csDesigning in ComponentState) then
    begin
    Result:=_Com.TabOrder;
    if Result=-1 then
      Result:=FTabOrder
    else
      FTabOrder:=Result;
    end
  else
    Result:=FTabOrder;
  end;

function TRtcWinControl.IsCtl3DStored: Boolean;
  begin
  Result:=not ParentCtl3D;
  end;

procedure TRtcWinControl.Set_BevelEdges(const Value: TBevelEdges);
  var
    rec:TRtcRecord;
  begin
  if (FBevelEdges<>Value) then
    begin
    FBevelEdges := Value;
    rec:=TRtcRecord.Create;
    if beLeft in Value then rec.asBoolean['l']:=True;
    if beTop in Value then rec.asBoolean['t']:=True;
    if beRight in Value then rec.asBoolean['r']:=True;
    if beBottom in Value then rec.asBoolean['b']:=True;
    _Com.Prop('beveledges',rec);
    end;
  end;

procedure TRtcWinControl.Set_BevelKind(const Value: TBevelKind);
  begin
  if (FBevelKind<>Value) then
    begin
    FBevelKind := Value;
    _Com.Prop('bevelkind',Ord(Value));
    end;
  end;

procedure TRtcWinControl.Set_BevelWidth(const Value: TBevelWidth);
  begin
  if (FBevelWidth<>Value) then
    begin
    FBevelWidth := Value;
    _Com.Prop('bevelwidth',Ord(Value));
    end;
  end;

procedure TRtcWinControl.Set_BorderWidth(const Value: TBorderWidth);
  begin
  if (FBorderWidth<>Value) then
    begin
    FBorderWidth := Value;
    _Com.Prop('borderwidth',Ord(Value));
    end;
  end;

procedure TRtcWinControl.Set_Ctl3D(const Value: Boolean);
  begin
  if (FCtl3D<>Value) then
    begin
    FCtl3D := Value;
    _Com.Prop('ctl3d',Value);
    FParentCtl3D := False;
    end;
  end;

procedure TRtcWinControl.Set_ImeMode(const Value: TImeMode);
  begin
  if (FImeMode<>Value) then
    begin
    FImeMode := Value;
    _Com.Prop('imemode',Ord(Value));
    end;
  end;

procedure TRtcWinControl.Set_ImeName(const Value: TImeName);
  begin
  if (FImeName<>Value) then
    begin
    FImeName := Value;
    _Com.Prop('imename',Value);
    end;
  end;

procedure TRtcWinControl.Set_ParentBackground(const Value: Boolean);
  begin
  {$IFDEF IDE_7up}
  if (ParentBackground<>Value) then
    begin
    if Value then
      ControlStyle := ControlStyle + [csParentBackground]
    else
      ControlStyle := ControlStyle - [csParentBackground];
    _Com.Prop('parentbackground',Value);
    end;
  {$ENDIF}
  end;

procedure TRtcWinControl.Set_ParentCtl3D(const Value: Boolean);
  begin
  if (FParentCtl3D<>Value) then
    begin
    FParentCtl3D := Value;
    _Com.Prop('parentctl3d',Value);
    end;
  end;

procedure TRtcWinControl.Set_TabStop(const Value: Boolean);
  begin
  if (FTabStop<>Value) then
    begin
    FTabStop := Value;
    _Com.Prop('tabstop',Value);
    end;
  end;

procedure TRtcWinControl.Set_BevelInner(const Value: TBevelCut);
  begin
  if (FBevelInner<>Value) then
    begin
    FBevelInner := Value;
    _Com.Prop('bevelinner',Ord(Value));
    end;
  end;

procedure TRtcWinControl.Set_BevelOuter(const Value: TBevelCut);
  begin
  if (FBevelOuter<>Value) then
    begin
    FBevelOuter := Value;
    _Com.Prop('bevelouter',Ord(Value));
    end;
  end;

procedure TRtcWinControl.Set_OnKeyDown(const Value: TRtcKeyEvent);
  begin
  FOnKeyDown := Value;
  if assigned(FOnKeyDown) then
    _Com.Event('onkeydown',MyOnKeyDown)
  else
    _Com.Event('onkeydown',nil);
  end;

procedure TRtcWinControl.Set_OnKeyPress(const Value: TRtcKeyPressEvent);
  begin
  FOnKeyPress := Value;
  if assigned(FOnKeyPress) then
    _Com.Event('onkeypress',MyOnKeyPress)
  else
    _Com.Event('onkeypress',nil);
  end;

procedure TRtcWinControl.Set_OnKeyUp(const Value: TRtcKeyEvent);
  begin
  FOnKeyUp := Value;
  if assigned(FOnKeyUp) then
    _Com.Event('onkeyup',MyOnKeyUp)
  else
    _Com.Event('onkeyup',nil);
  end;

procedure TRtcWinControl.Set_OnEnter(const Value: TNotifyEvent);
  begin
  FOnEnter := Value;
  if assigned(FOnEnter) then
    _Com.Event('onenter',MyOnEnter)
  else
    _Com.Event('onenter',nil);
  end;

procedure TRtcWinControl.Set_OnExit(const Value: TNotifyEvent);
  begin
  FOnExit := Value;
  if assigned(FOnExit) then
    _Com.Event('onexit',MyOnExit)
  else
    _Com.Event('onexit',nil);
  end;

procedure TRtcWinControl.Set_OnChange(const Value: TNotifyEvent);
  begin
  FOnChange := Value;
  if assigned(FOnChange) then
    _Com.Event('onchange',MyOnChange)
  else
    _Com.Event('onchange',nil);
  end;

procedure TRtcWinControl.MyOnKeyDown(xData: TRtcValue);
  var
    key:word;
    shift:TShiftState;
  begin
  if assigned(FOnKeyDown) and Visible and Enabled then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        key:=asInteger['k'];
        shift:=RtcToShiftState(asRecord['s']);
        end;
      FOnKeyDown(self,key,shift);
      end;
    end;
  end;

procedure TRtcWinControl.MyOnKeyUp(xData: TRtcValue);
  var
    key:word;
    shift:TShiftState;
  begin
  if assigned(FOnKeyUp) and Visible and Enabled then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        key:=asInteger['k'];
        shift:=RtcToShiftState(asRecord['s']);
        end;
      FOnKeyUp(self,key,shift);
      end;
    end;
  end;

procedure TRtcWinControl.MyOnKeyPress(xData: TRtcValue);
  var
    key:char;
  begin
  if assigned(FOnKeyPress) and Visible and Enabled then
    begin
    if length(xData.asText)>0 then
      key:=(xData.asText)[1]
    else
      key:=#0;
    FOnKeyPress(self,key);
    end;
  end;

procedure TRtcWinControl.MyOnEnter(xData: TRtcValue);
  begin
  if assigned(FOnEnter) and Visible and Enabled then
    FOnEnter(self);
  end;

procedure TRtcWinControl.MyOnExit(xData: TRtcValue);
  begin
  if assigned(FOnExit) then
    FOnExit(self);
  end;

procedure TRtcWinControl.MyOnChange(xData: TRtcValue);
  begin
  if assigned(FOnChange) then
    FOnChange(self);
  end;

function TRtcWinControl.CanFocus: Boolean;
  var
    Control: TControl;
  begin
  Result := False;
  Control := Self;
  repeat
    if not (Control.Visible and Control.Enabled) then Exit;
      begin
      Control := Control.Parent;
      if Control=nil then Break;
      end;
    until Control.Parent=nil;
  Result := True;
  end;

procedure TRtcWinControl.SetFocus;
  begin
  _Com.Send('setfocus',nil);
  end;

function TRtcWinControl.Showing: Boolean;
  begin
  Result:= Visible=True;
  end;

{ TRtcParWinControl }

constructor TRtcParWinControl.Create(AOwner: TComponent);
  begin
  inherited;
  ControlStyle := RtcFormControlStyle;

  FCanvas:=nil;

  FFnt:=TRtcFont.Create(inherited Font);
  FFnt._Com:=_Com;
  FFnt.FullName:='font.';
  FFnt.OnChange:=MyFontChange;

  FConstr:=TRtcSizeConstraints.Create;
  FConstr._Com:=_Com;
  FConstr.FullName:='cnstr.';
  FConstr.Par:=inherited Constraints;

  FAutoSize:=False;

  FBevelEdges:=RtcDefault_BevelEdges;
  FBevelInner:=bvNone;
  FBevelOuter:=bvRaised;
  FBevelKind:=bkNone;
  FBevelWidth:=1;
  FBorderWidth:=0;
  FCtl3D:=True;
  FImeMode:=imDontCare;
  FImeName:='';
  FParentCtl3D:=True;
  FTabOrder:=-1;
  FTabStop:=False;
  end;

destructor TRtcParWinControl.Destroy;
  begin
  FAction:=nil;
  FPopupMenu:=nil;
  if assigned(FFnt) then FreeAndNil(FFnt);
  if assigned(FConstr) then FreeAndNil(FConstr);
  if assigned(FCanvas) then FreeAndNil(FCanvas);
  inherited;
  end;

function TRtcParWinControl.Get_Font: TRtcFont;
  begin
  Result:=FFnt;
  end;

function TRtcParWinControl.Get_Constraints: TRtcSizeConstraints;
  begin
  Result:=FConstr;
  end;

function TRtcParWinControl.Get_Cursor: TCursor;
  begin
  Result:=inherited Cursor;
  end;

function TRtcParWinControl.Get_HelpContext: THelpContext;
  begin
  Result:=inherited HelpContext;
  end;

function TRtcParWinControl.Get_HelpKeyword: String;
  begin
  Result:=inherited HelpKeyword;
  end;

function TRtcParWinControl.Get_HelpType: THelpType;
  begin
  Result:=inherited HelpType;
  end;

function TRtcParWinControl.Get_Hint: string;
  begin
  Result:=inherited Hint;
  end;

function TRtcParWinControl.Get_Tag: Longint;
  begin
  Result:=inherited Tag;
  end;

function TRtcParWinControl.Get_Align: TAlign;
  begin
  Result:=inherited Align;
  end;

function TRtcParWinControl.Get_Anchors: TAnchors;
  begin
  Result:=inherited Anchors;
  end;

function TRtcParWinControl.Get_BiDiMode: TBiDiMode;
  begin
  Result:=inherited BiDiMode;
  end;

function TRtcParWinControl.Get_Color: TColor;
  begin
  Result:=inherited Color;
  end;

function TRtcParWinControl.Get_DesktopFont: Boolean;
  begin
  Result:=inherited DesktopFont;
  end;

function TRtcParWinControl.Get_DragCursor: TCursor;
  begin
  Result:=inherited DragCursor;
  end;

function TRtcParWinControl.Get_DragKind: TDragKind;
  begin
  Result:=inherited DragKind;
  end;

function TRtcParWinControl.Get_DragMode: TDragMode;
  begin
  Result:=inherited DragMode;
  end;

function TRtcParWinControl.Get_ParentBiDiMode: Boolean;
  begin
  Result:=inherited ParentBiDiMode;
  end;

function TRtcParWinControl.Get_ParentColor: Boolean;
  begin
  Result:=inherited ParentColor;
  end;

function TRtcParWinControl.Get_ParentFont: Boolean;
  begin
  Result:=inherited ParentFont;
  end;

function TRtcParWinControl.Get_ParentShowHint: Boolean;
  begin
  Result:=inherited ParentShowHint;
  end;

function TRtcParWinControl.Get_ShowHint: Boolean;
  begin
  Result:=inherited ShowHint;
  end;

function TRtcParWinControl.Get_Visible: Boolean;
  begin
  Result:=inherited Visible;
  end;

function TRtcParWinControl.Get_Enabled: Boolean;
  begin
  Result:=inherited Enabled;
  end;

function TRtcParWinControl.IsAnchorsStored: Boolean;
  begin
  Result:=Anchors <> AnchorAlign[Align];
  // Result:=inherited IsAnchorsStored;
  end;

function TRtcParWinControl.IsBiDiModeStored: Boolean;
  begin
  Result:=not ParentBiDiMode;
  // Result:=inherited IsBiDiModeStored;
  end;

function TRtcParWinControl.IsColorStored: Boolean;
  begin
  Result:=not ParentColor;
  end;

function TRtcParWinControl.IsFontStored: Boolean;
  begin
  Result:=not ParentFont and not DesktopFont;
  end;

function TRtcParWinControl.IsShowHintStored: Boolean;
  begin
  Result:=not ParentShowHint;
  end;

function TRtcParWinControl.IsVisibleStored: Boolean;
  begin
  Result:=True;
  // Result:=inherited IsVisibleStored;
  end;

function TRtcParWinControl.IsOnClickStored: Boolean;
  begin
  Result:=True;
  // Result:=inherited IsOnClickStored;
  end;

function TRtcParWinControl.IsEnabledStored: Boolean;
  begin
  Result:=True;
  // Result:=inherited IsEnabledStored;
  end;

procedure TRtcParWinControl.Set_Font(const Value: TRtcFont);
  begin
  FFnt.Assign(Value);
  end;

procedure TRtcParWinControl.Set_Constraints(const Value: TRtcSizeConstraints);
  begin
  FConstr.Assign(Value);
  end;

procedure TRtcParWinControl.Set_Tag(const Value: Longint);
  begin
  if inherited Tag<>Value then
    begin
    inherited Tag := Value;
    _Com.Prop('tag',Value);
    end;
  end;

procedure TRtcParWinControl.Set_Cursor(const Value: TCursor);
  begin
  if inherited Cursor<>Value then
    begin
    inherited Cursor := Value;
    _Com.Prop('cursor',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_Hint(const Value: string);
  begin
  if inherited Hint<>Value then
    begin
    inherited Hint := Value;
    _Com.Prop('hint',Value);
    end;
  end;

procedure TRtcParWinControl.Set_HelpKeyword(const Value: String);
  begin
  if inherited HelpKeyword<>Value then
    begin
    inherited HelpKeyword := Value;
    _Com.Prop('helpkeyword',Value);
    end;
  end;

procedure TRtcParWinControl.Set_HelpType(const Value: THelpType);
  begin
  if inherited HelpType<>Value then
    begin
    inherited HelpType := Value;
    _Com.Prop('helptype',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_HelpContext(const Value: THelpContext);
  begin
  if inherited HelpContext<>Value then
    begin
    inherited HelpContext := Value;
    _Com.Prop('helpcontext',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_Align(const Value: TAlign);
  begin
  if inherited Align<>Value then
    begin
    inherited Align:=Value;
    _Com.Prop('align',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_Anchors(const Value: TAnchors);
  var
    rec:TRtcRecord;
  begin
  if inherited Anchors<>Value then
    begin
    inherited Anchors:=Value;
    rec:=TRtcRecord.Create;
    if (akLeft in Anchors) then rec.asBoolean['l']:=True;
    if (akTop in Anchors) then rec.asBoolean['t']:=True;
    if (akRight in Anchors) then rec.asBoolean['r']:=True;
    if (akBottom in Anchors) then rec.asBoolean['b']:=True;
    _Com.Prop('anchors',rec);
    end;
  end;

procedure TRtcParWinControl.Set_AutoSize(const Value: Boolean);
  begin
  if FAutoSize<>Value then
    begin
    FAutoSize:=Value;
    _Com.Prop('autosize',Value);
    end;
  end;

procedure TRtcParWinControl.Set_BiDiMode(const Value: TBiDiMode);
  begin
  if inherited BiDiMode<>Value then
    begin
    inherited BiDiMode:=Value;
    _Com.Prop('bidimode',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_Color(const Value: TColor);
  begin
  if inherited Color<>Value then
    begin
    inherited Color:=Value;
    _Com.Prop('color',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_DesktopFont(const Value: Boolean);
  begin
  if inherited DesktopFont<>Value then
    begin
    inherited DesktopFont:=Value;
    _Com.Prop('desktopfont',Value);
    end;
  end;

procedure TRtcParWinControl.Set_DragCursor(const Value: TCursor);
  begin
  if inherited DragCursor<>Value then
    begin
    inherited DragCursor:=Value;
    _Com.Prop('dragcursor',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_DragKind(const Value: TDragKind);
  begin
  if inherited DragKind<>Value then
    begin
    inherited DragKind:=Value;
    _COm.Prop('dragkind',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_DragMode(const Value: TDragMode);
  begin
  if inherited DragMode<>Value then
    begin
    inherited DragMode:=Value;
    _Com.Prop('dragmode',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_ParentBiDiMode(const Value: Boolean);
  begin
  if inherited ParentBiDiMode<>Value then
    begin
    inherited ParentBiDiMode:=Value;
    _Com.Prop('parentbidimode',Value);
    end;
  end;

procedure TRtcParWinControl.Set_ParentColor(const Value: Boolean);
  begin
  if inherited ParentColor<>Value then
    begin
    inherited ParentColor:=Value;
    _Com.Prop('parentcolor',Value);
    end;
  end;

procedure TRtcParWinControl.Set_ParentFont(const Value: Boolean);
  begin
  if inherited ParentFont<>Value then
    begin
    inherited ParentFont:=Value;
    _Com.Prop('parentfont',Value);
    end;
  end;

procedure TRtcParWinControl.Set_ParentShowHint(const Value: Boolean);
  begin
  if inherited ParentShowHint<>Value then
    begin
    inherited ParentShowHint:=Value;
    _Com.Prop('parentshowhint',Value);
    end;
  end;

procedure TRtcParWinControl.Set_ShowHint(const Value: Boolean);
  begin
  if inherited ShowHint<>Value then
    begin
    inherited ShowHint:=Value;
    _Com.Prop('showhint',Value);
    end;
  end;

procedure TRtcParWinControl.Set_Visible(const Value: Boolean);
  begin
  if inherited Visible<>Value then
    begin
    inherited Visible:=Value;
    _Com.Prop('visible',Value);
    end;
  end;

procedure TRtcParWinControl.Set_Enabled(const Value: Boolean);
  begin
  if inherited Enabled<>Value then
    begin
    inherited Enabled:=Value;
    _Com.Prop('enabled',Value);
    end;
  end;

procedure TRtcParWinControl.Set_OnClick(const Value: TNotifyEvent);
  begin
  FOnClick:=Value;
  if assigned(FOnClick) then
    _Com.Event('onclick',MyOnClick)
  else
    _Com.Event('onclick',nil);
  end;

procedure TRtcParWinControl.Set_OnDblClick(const Value: TNotifyEvent);
  begin
  FOnDblClick:=Value;
  if assigned(FOnDblClick) then
    _Com.Event('ondblclick',MyOnDblClick)
  else
    _Com.Event('ondblclick',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseDown(const Value: TRtcMouseEvent);
  begin
  FOnMouseDown:=Value;
  if assigned(FOnMouseDown) then
    _Com.Event('onmousedown',MyOnMouseDown)
  else
    _Com.Event('onmousedown',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseMove:=Value;
  if assigned(FOnMouseMove) then
    _Com.Event('onmousemove',MyOnMouseMove)
  else
    _Com.Event('onmousemove',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseUp(const Value: TRtcMouseEvent);
  begin
  FOnMouseUp:=Value;
  if assigned(FOnMouseUp) then
    _Com.Event('onmouseup',MyOnMouseUp)
  else
    _Com.Event('onmouseup',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseWheel(const Value: TRtcMouseWheelEvent);
  begin
  FOnMouseWheel:=Value;
  if assigned(FOnMouseWheel) then
    _Com.Event('onmousewheel',MyOnMouseWheel)
  else
    _Com.Event('onmousewheel',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseWheelDown(const Value: TRtcMouseWheelUpDownEvent);
  begin
  FOnMouseWheelDown:=Value;
  if assigned(FOnMouseWheelDown) then
    _Com.Event('onmousewheeldown',MyOnMouseWheelDown)
  else
    _Com.Event('onmousewheeldown',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseWheelUp(const Value: TRtcMouseWheelUpDownEvent);
  begin
  FOnMouseWheelUp:=Value;
  if assigned(FOnMouseWheelUp) then
    _Com.Event('onmousewheelup',MyOnMouseWheelUp)
  else
    _Com.Event('onmousewheelup',nil);
  end;

procedure TRtcParWinControl.Set_OnResize(const Value: TNotifyEvent);
  begin
  FOnResize:=Value;
  if assigned(FOnResize) then
    _Com.Event('onresize',MyOnResize)
  else
    _Com.Event('onresize',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseDownMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseDownMove:=Value;
  if assigned(FOnMouseDownMove) then
    _Com.Event('onmousedownmove',MyOnMouseDownMove)
  else
    _Com.Event('onmousedownmove',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseUpMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseUpMove:=Value;
  if assigned(FOnMouseUpMove) then
    _Com.Event('onmouseupmove',MyOnMouseUpMove)
  else
    _Com.Event('onmouseupmove',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseEnter(const Value: TNotifyEvent);
  begin
  FOnMouseEnter := Value;
  if assigned(FOnMouseEnter) then
    _Com.Event('onmouseenter',MyOnMouseEnter)
  else
    _Com.Event('onmouseenter',nil);
  end;

procedure TRtcParWinControl.Set_OnMouseLeave(const Value: TNotifyEvent);
  begin
  FOnMouseLeave := Value;
  if assigned(FOnMouseLeave) then
    _Com.Event('onmouseleave',MyOnMouseLeave)
  else
    _Com.Event('onmouseleave',nil);
  end;

procedure TRtcParWinControl.MyOnClick(xData: TRtcValue);
  begin
  if assigned(FOnClick) and Enabled and Visible then
    FOnClick(self);
  end;

procedure TRtcParWinControl.MyOnDblClick(xData: TRtcValue);
  begin
  if assigned(FOnDblClick) and Enabled and Visible then
    FOnDblClick(self);
  end;

procedure TRtcParWinControl.MyOnResize(xData: TRtcValue);
  begin
  if assigned(FOnResize) then
    FOnResize(self);
  end;

procedure TRtcParWinControl.MyOnMouseDown(xData: TRtcValue);
  var
    button:TMouseButton;
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseDown) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        button:=TMouseButton(asInteger['b']);
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseDown(self,button,shift,x,y);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnMouseUp(xData: TRtcValue);
  var
    button:TMouseButton;
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseUp) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        button:=TMouseButton(asInteger['b']);
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseUp(self,button,shift,x,y);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnMouseMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseMove) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseMove(self,shift,x,y);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnMouseDownMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseDownMove) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseDownMove(self,shift,x,y);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnMouseUpMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseUpMove) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      Screen2Client(x,y);
      FOnMouseUpMove(self,shift,x,y);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnMouseWheel(xData: TRtcValue);
  var
    shift:TShiftState;
    wdelta:integer;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheel) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        wdelta:=asInteger['w'];
        end;
      handled:=False;
      FOnMouseWheel(self,shift,wdelta,mpos,handled);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnMouseWheelDown(xData: TRtcValue);
  var
    shift:TShiftState;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheelDown) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        end;
      handled:=False;
      FOnMouseWheelDown(self,shift,mpos,handled);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnMouseWheelUp(xData: TRtcValue);
  var
    shift:TShiftState;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheelUp) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        end;
      handled:=False;
      FOnMouseWheelUp(self,shift,mpos,handled);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnMouseEnter(xData: TRtcValue);
  begin
  if assigned(FOnMouseEnter) and Visible then
    FOnMouseEnter(self);
  end;

procedure TRtcParWinControl.MyOnMouseLeave(xData: TRtcValue);
  begin
  if assigned(FOnMouseLeave) then
    FOnMouseLeave(self);
  end;

procedure TRtcParWinControl.Screen2Client(var x, y: integer);
  var
    wc:TWinControl;
  begin
  wc:=Parent;
  if assigned(wc) then
    begin
    Dec(x,Left);
    Dec(y,Top);
    repeat
      if assigned(wc.Parent) then
        begin
        Dec(x,wc.Left);
        Dec(y,wc.Top);
        wc:=wc.Parent;
        end
      else
        wc:=nil;
      until not assigned(wc);
    end;
  end;

function TRtcParWinControl.Get_Canvas: TRtcCanvas;
  begin
  if not assigned(FCanvas) then
    begin
    FCanvas:=TRtcCanvas.Create;
    FCanvas._Com:=_Com;
    FCanvas.FullName:='canvas.';
    end;
  Result:=FCanvas;
  end;

function TRtcParWinControl.Get_Action: IRtcBasicAction;
  begin
  Result:=FAction;
  end;

function TRtcParWinControl.Get_PopupMenu: IRtcPopupMenu;
  begin
  Result:=FPopupMenu;
  end;

procedure TRtcParWinControl.Set_Action(const Value: IRtcBasicAction);
  begin
  if (Value<>FAction) then
    begin
    FAction:=Value;
    _Com.Link('action',Value);
    end;
  end;

procedure TRtcParWinControl.Set_PopupMenu(const Value: IRtcPopupMenu);
  begin
  if (Value<>FPopupMenu) then
    begin
    FPopupMenu:=Value;
    _Com.Link('popupmenu',Value);
    end;
  end;

procedure TRtcParWinControl.Notification(AComponent: TComponent; Operation: TOperation);
  begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if assigned(PopupMenu) and (AComponent = PopupMenu._Self) then PopupMenu := nil
    else if assigned(Action) and (AComponent = Action._Self) then Action := nil;
  end;

function TRtcParWinControl.Get_ParentBackground: Boolean;
  begin
  {$IFDEF IDE_7up}
  Result := csParentBackground in ControlStyle;
  {$ELSE}
  Result := False;
  {$ENDIF}
  end;

procedure TRtcParWinControl.Set_TabOrder(const Value: TTabOrder);
  begin
  if FTabOrder<>Value then
    begin
    FTabOrder := Value;
    _Com.Prop('taborder',Ord(Value));
    end;
  end;

function TRtcParWinControl.Get_TabOrder: TTabOrder;
  begin
  if (csDesigning in ComponentState) then
    begin
    Result:=_Com.TabOrder;
    if Result=-1 then
      Result:=FTabOrder
    else
      FTabOrder:=Result;
    end
  else
    Result:=FTabOrder;
  end;

function TRtcParWinControl.IsCtl3DStored: Boolean;
  begin
  Result:=not ParentCtl3D;
  end;

procedure TRtcParWinControl.Set_BevelEdges(const Value: TBevelEdges);
  var
    rec:TRtcRecord;
  begin
  if (FBevelEdges<>Value) then
    begin
    FBevelEdges := Value;
    rec:=TRtcRecord.Create;
    if beLeft in Value then rec.asBoolean['l']:=True;
    if beTop in Value then rec.asBoolean['t']:=True;
    if beRight in Value then rec.asBoolean['r']:=True;
    if beBottom in Value then rec.asBoolean['b']:=True;
    _Com.Prop('beveledges',rec);
    end;
  end;

procedure TRtcParWinControl.Set_BevelKind(const Value: TBevelKind);
  begin
  if (FBevelKind<>Value) then
    begin
    FBevelKind := Value;
    _Com.Prop('bevelkind',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_BevelWidth(const Value: TBevelWidth);
  begin
  if (FBevelWidth<>Value) then
    begin
    FBevelWidth := Value;
    _Com.Prop('bevelwidth',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_BorderWidth(const Value: TBorderWidth);
  begin
  if (FBorderWidth<>Value) then
    begin
    FBorderWidth := Value;
    _Com.Prop('borderwidth',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_Ctl3D(const Value: Boolean);
  begin
  if (FCtl3D<>Value) then
    begin
    FCtl3D := Value;
    _Com.Prop('ctl3d',Value);
    FParentCtl3D := False;
    end;
  end;

procedure TRtcParWinControl.Set_ImeMode(const Value: TImeMode);
  begin
  if (FImeMode<>Value) then
    begin
    FImeMode := Value;
    _Com.Prop('imemode',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_ImeName(const Value: TImeName);
  begin
  if (FImeName<>Value) then
    begin
    FImeName := Value;
    _Com.Prop('imename',Value);
    end;
  end;

procedure TRtcParWinControl.Set_ParentBackground(const Value: Boolean);
  begin
  {$IFDEF IDE_7up}
  if (ParentBackground<>Value) then
    begin
    if Value then
      ControlStyle := ControlStyle + [csParentBackground]
    else
      ControlStyle := ControlStyle - [csParentBackground];
    _Com.Prop('parentbackground',Value);
    end;
  {$ENDIF}
  end;

procedure TRtcParWinControl.Set_ParentCtl3D(const Value: Boolean);
  begin
  if (FParentCtl3D<>Value) then
    begin
    FParentCtl3D := Value;
    _Com.Prop('parentctl3d',Value);
    end;
  end;

procedure TRtcParWinControl.Set_TabStop(const Value: Boolean);
  begin
  if (FTabStop<>Value) then
    begin
    FTabStop := Value;
    _Com.Prop('tabstop',Value);
    end;
  end;

procedure TRtcParWinControl.Set_BevelInner(const Value: TBevelCut);
  begin
  if (FBevelInner<>Value) then
    begin
    FBevelInner := Value;
    _Com.Prop('bevelinner',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_BevelOuter(const Value: TBevelCut);
  begin
  if (FBevelOuter<>Value) then
    begin
    FBevelOuter := Value;
    _Com.Prop('bevelouter',Ord(Value));
    end;
  end;

procedure TRtcParWinControl.Set_OnKeyDown(const Value: TRtcKeyEvent);
  begin
  FOnKeyDown := Value;
  if assigned(FOnKeyDown) then
    _Com.Event('onkeydown',MyOnKeyDown)
  else
    _Com.Event('onkeydown',nil);
  end;

procedure TRtcParWinControl.Set_OnKeyPress(const Value: TRtcKeyPressEvent);
  begin
  FOnKeyPress := Value;
  if assigned(FOnKeyPress) then
    _Com.Event('onkeypress',MyOnKeyPress)
  else
    _Com.Event('onkeypress',nil);
  end;

procedure TRtcParWinControl.Set_OnKeyUp(const Value: TRtcKeyEvent);
  begin
  FOnKeyUp := Value;
  if assigned(FOnKeyUp) then
    _Com.Event('onkeyup',MyOnKeyUp)
  else
    _Com.Event('onkeyup',nil);
  end;

procedure TRtcParWinControl.Set_OnEnter(const Value: TNotifyEvent);
  begin
  FOnEnter := Value;
  if assigned(FOnEnter) then
    _Com.Event('onenter',MyOnEnter)
  else
    _Com.Event('onenter',nil);
  end;

procedure TRtcParWinControl.Set_OnExit(const Value: TNotifyEvent);
  begin
  FOnExit := Value;
  if assigned(FOnExit) then
    _Com.Event('onexit',MyOnExit)
  else
    _Com.Event('onexit',nil);
  end;

procedure TRtcParWinControl.Set_OnChange(const Value: TNotifyEvent);
  begin
  FOnChange := Value;
  if assigned(FOnChange) then
    _Com.Event('onchange',MyOnChange)
  else
    _Com.Event('onchange',nil);
  end;

procedure TRtcParWinControl.Set_OnDragDrop(const Value: TRtcDragDropEvent);
  begin
  FOnDragDrop := Value;
  if assigned(FOnDragDrop) then
    _Com.Event('ondragdrop',MyOnDragDrop)
  else
    _Com.Event('ondragdrop',nil);
  end;

procedure TRtcParWinControl.Set_OnStartDrag(const Value: TRtcStartDragEvent);
  begin
  FOnStartDrag := Value;
  if assigned(FOnStartDrag) then
    _Com.Event('onstartdrag',MyOnStartDrag)
  else
    _Com.Event('onstartdrag',nil);
  end;

procedure TRtcParWinControl.Set_OnEndDrag(const Value: TRtcEndDragEvent);
  begin
  FOnEndDrag := Value;
  if assigned(FOnEndDrag) then
    _Com.Event('onenddrag',MyOnEndDrag)
  else
    _Com.Event('onenddrag',nil);
  end;

procedure TRtcParWinControl.MyOnStartDrag(xData: TRtcValue);
  var
    o:TDragObject;
  begin
  if assigned(FOnStartDrag) and Visible then
    begin
    o:=nil;
    FOnStartDrag(self,o);
    end;
  end;

procedure TRtcParWinControl.MyOnEndDrag(xData: TRtcValue);
  var
    src:TObject;
    X,Y:integer;
  begin
  if assigned(FOnEndDrag) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        if isType['o']=rtc_Record then
          src:=_Com.FindByUID(asRecord['o'])
        else
          src:=nil;
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnEndDrag(self,src,X,Y);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnDragDrop(xData: TRtcValue);
  var
    src:TObject;
    X,Y:integer;
  begin
  if assigned(FOnDragDrop) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        src:=_Com.FindByUID(asRecord['o']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnDragDrop(self,src,X,Y);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnKeyDown(xData: TRtcValue);
  var
    key:word;
    shift:TShiftState;
  begin
  if assigned(FOnKeyDown) and Visible and Enabled then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        key:=asInteger['k'];
        shift:=RtcToShiftState(asRecord['s']);
        end;
      FOnKeyDown(self,key,shift);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnKeyUp(xData: TRtcValue);
  var
    key:word;
    shift:TShiftState;
  begin
  if assigned(FOnKeyUp) and Visible and Enabled then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        key:=asInteger['k'];
        shift:=RtcToShiftState(asRecord['s']);
        end;
      FOnKeyUp(self,key,shift);
      end;
    end;
  end;

procedure TRtcParWinControl.MyOnKeyPress(xData: TRtcValue);
  var
    key:char;
  begin
  if assigned(FOnKeyPress) and Visible and Enabled then
    begin
    if length(xData.asText)>0 then
      key:=(xData.asText)[1]
    else
      key:=#0;
    FOnKeyPress(self,key);
    end;
  end;

procedure TRtcParWinControl.MyOnEnter(xData: TRtcValue);
  begin
  if assigned(FOnEnter) and Visible and Enabled then
    FOnEnter(self);
  end;

procedure TRtcParWinControl.MyOnExit(xData: TRtcValue);
  begin
  if assigned(FOnExit) then
    FOnExit(self);
  end;

procedure TRtcParWinControl.MyOnChange(xData: TRtcValue);
  begin
  if assigned(FOnChange) then
    FOnChange(self);
  end;

procedure TRtcParWinControl.Hide;
  begin
  Visible:=False;
  end;

procedure TRtcParWinControl.Show;
  begin
  Visible:=True;
  end;

procedure TRtcParWinControl.BringToFront;
  begin
  if (csDesigning in ComponentState) then
    TControl(self).BringToFront;
  _Com.Send('tofront',nil);
  end;

procedure TRtcParWinControl.SendToBack;
  begin
  if (csDesigning in ComponentState) then
    TControl(Self).SendToBack;
  _Com.Send('toback',nil);
  end;

procedure TRtcParWinControl.Refresh;
  begin
  if (csDesigning in ComponentState) then
    TControl(Self).Refresh;
  _Com.Send('refresh',nil);
  end;

procedure TRtcParWinControl.BeginDrag(Immediate: Boolean; Threshold: Integer);
  var
    rec:TRtcRecord;
  begin
  rec:=TRtcRecord.Create;
  rec.asBoolean['i']:=Immediate;
  rec.asInteger['t']:=Threshold;
  _Com.Send('begindrag',rec);
  end;

function TRtcParWinControl.Dragging: Boolean;
  begin
  Result:=False;
  end;

procedure TRtcParWinControl.EndDrag(Drop: Boolean);
  begin
  _Com.Send('enddrag',Drop);
  end;

procedure TRtcParWinControl.DragDrop(Source: TObject; X, Y: Integer);
  begin
  if assigned(FOnDragDrop) then FOnDragDrop(self,Source,X,Y);
  end;

function TRtcParWinControl.ClientToScreen(const Point: TPoint): TPoint;
  var
    LParent: TWinControl;
  begin
  Result := Point;
  Inc(Result.X, Left);
  Inc(Result.Y, Top);
  LParent := Parent;
  while LParent <> nil do
    begin
    Inc(Result.X, LParent.Left);
    Inc(Result.Y, LParent.Top);
    LParent := LParent.Parent;
    end;
  end;

function TRtcParWinControl.ScreenToClient(const Point: TPoint): TPoint;
  var
    LParent: TWinControl;
  begin
  Result := Point;
  Dec(Result.X, Left);
  Dec(Result.Y, Top);
  LParent := Parent;
  while LParent <> nil do
    begin
    Dec(Result.X, LParent.Left);
    Dec(Result.Y, LParent.Top);
    LParent := LParent.Parent;
    end;
  end;

function TRtcParWinControl.CanFocus: Boolean;
  var
    Control: TControl;
  begin
  Result := False;
  Control := Self;
  repeat
    if not (Control.Visible and Control.Enabled) then Exit;
      begin
      Control := Control.Parent;
      if Control=nil then Break;
      end;
    until Control.Parent=nil;
  Result := True;
  end;

procedure TRtcParWinControl.SetFocus;
  begin
  _Com.Send('setfocus',nil);
  end;

function TRtcParWinControl.Showing: Boolean;
  begin
  Result:= Visible=True;
  end;

procedure TRtcParWinControl.MyOnDropFiles(xData: TRtcValue);
  var
    a:integer;
    arr:TRtcArray;
    myarr:array of string;
  begin
  if assigned(FOnDropFiles) and (xData.isType=rtc_Array) then
    begin
    arr:=xData.asArray;
    try
      SetLength(myarr,arr.Count);
      for a:=0 to arr.Count-1 do
        myarr[a]:=arr.asText[a];
      FOnDropFiles(self,myarr);
    finally
      SetLength(myarr,0);
      end;
    end;
  end;

procedure TRtcParWinControl.Set_OnDropFiles(const Value: TRtcDropFilesEvent);
  begin
  FOnDropFiles := Value;
  if assigned(FOnDropFiles) then
    _Com.Event('ondropfiles',MyOnDropFiles)
  else
    _Com.Event('ondropfiles',nil);
  end;

procedure TRtcParWinControl.MyFontChange(Sender: TObject);
  begin
  ParentFont:=False;
  end;

end.
