{ 
  RealThinClient W/A Framework: General Client-side Controls
  Copyright (c) Danijel Tkalcec

  @exclude
}
unit rtcControlsCli;

interface

{$include rtcDefs.inc}

{ Some Control operations are only available with a player compiled ussing Delphi for Windows.

  Here is the list of things you should NOT use in your code to keep your application
  cross-compatible between Windows, Linux and MacOSX:

  * Files Drag and Drop feature - not implemented for Lazarus (only Delphi/Win32)
  * TControl.EndDrag method calls will be ignored
  * ScrollBars on Lazarus do not implement the following properties, values will be ignored:
    - ButtonSize
    - Color
    - Margin
    - ParentColor
    - Style
    - ThumbSize
    - Tracking
}

uses
  Classes, Types, Messages,
  {$ifdef WINDOWS}
  Windows, ShellApi,
  {$endif}

  Controls, SysUtils,
  Graphics, Forms, Menus,
  rtcInfo,

  rtcLink, rtcGraphicsCli;

type
  TRtcControlEvents=class
  private
    FOnClickID,
    FOnDblClickID,

    FOnDragDropID,
    FOnStartDragID,
    FOnEndDragID,

    FOnMouseEnterID,
    FOnMouseLeaveID,

    FOnMouseDownID,
    FOnMouseUpID,
    FOnMouseMoveID,
    FOnMouseDownMoveID,
    FOnMouseUpMoveID:longint;

    FMouseDown:integer;

  protected
    FCom:TRtcComC;

    procedure MyOnClick(Sender: TObject);
    procedure MyOnDblClick(Sender: TObject);
    procedure MyOnMouseEnter(Sender: TObject);
    procedure MyOnMouseLeave(Sender: TObject);

    procedure MyOnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure MyOnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure MyOnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure MyOnDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure MyOnDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);

    procedure MyOnStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure MyOnEndDrag(Sender, Target: TObject; X, Y: Integer);

    procedure Client2Screen(Sender:TControl; var x,y:integer);

  public

    constructor Create(c:TRtcComC);
    destructor Destroy; override;

    function OnClick(xID: Integer):TNotifyEvent;
    function OnDblClick(xID: integer):TNotifyEvent;
    function OnMouseEnter(xID: Integer):TNotifyEvent;
    function OnMouseLeave(xID: integer):TNotifyEvent;

    function OnMouseDown(xID: integer):TMouseEvent;
    function OnMouseMove(xID: integer):TMouseMoveEvent;
    function OnMouseDownMove(xID: integer):TMouseMoveEvent;
    function OnMouseUpMove(xID: integer):TMouseMoveEvent;
    function OnMouseUp(xID: integer):TMouseEvent;

    function OnDragDrop(xID: integer):TDragDropEvent;
    function OnDragOver(xID: integer):TDragOverEvent;

    function OnStartDrag(xID: integer):TStartDragEvent;
    function OnEndDrag(xID: integer):TEndDragEvent;
    end;

  TRtcWinControlEvents=class(TRtcControlEvents)
  private
    FOnActivateID,
    FOnDeactivateID,
    FOnEnterID,
    FOnExitID,
    FOnResizeID,

    FOnMouseWheelID,
    FOnMouseWheelDownID,
    FOnMouseWheelUpID,

    FOnKeyDownID,
    FOnKeyUpID,
    FOnKeyPressID: longint;

  protected
    procedure MyOnActivate(Sender: TObject);
    procedure MyOnDeactivate(Sender: TObject);
    procedure MyOnResize(Sender: TObject);

    procedure MyOnEnter(Sender: TObject);
    procedure MyOnExit(Sender: TObject);

    procedure MyOnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure MyOnKeyPress(Sender: TObject; var Key: Char);
    procedure MyOnKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);

    procedure MyOnMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure MyOnMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
    procedure MyOnMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);

  public

    function OnActivate(xID: Integer):TNotifyEvent;
    function OnDeactivate(xID: integer):TNotifyEvent;
    function OnResize(xID: integer):TNotifyEvent;

    function OnEnter(xID: Integer):TNotifyEvent;
    function OnExit(xID: integer):TNotifyEvent;

    function OnKeyDown(xID: integer):TKeyEvent;
    function OnKeyPress(xID: integer):TKeyPressEvent;
    function OnKeyUp(xID: integer):TKeyEvent;

    function OnMouseWheel(xID: integer):TMouseWheelEvent;
    function OnMouseWheelUp(xID: integer):TMouseWheelUpDownEvent;
    function OnMouseWheelDown(xID: integer):TMouseWheelUpDownEvent;
    end;

function Rtcf_ProcessComponent(c:TComponent; const xName:string; xData:TRtcValue):boolean;
function Rtcf_ProcessControlScrollBar(c:TControlScrollBar; const xName:string; xData:TRtcValue):boolean;
function Rtcf_ProcessSizeConstraints(c:TSizeConstraints; const xName:String; xData:TRtcValue):boolean;

function Rtcf_Link(c:TComponent; const xName:string; xData:TComponent):boolean;

procedure Rtcf_AcceptFiles(c:IRtcfComponentC; xID:integer; var msg:TMessage);

function ShiftStateToRtc(Shift:TShiftState):TRtcRecord;

implementation

procedure Rtcf_AcceptFiles(c:IRtcfComponentC; xID:integer; var msg:TMessage);
{$ifndef Windows}
  begin
  msg.Result:=1;
  end;
{$else}
   const
    cnMaxFileNameLen = 1024;
  var
    i,
    nCount     : integer;
    acFileName : array [0..cnMaxFileNameLen] of char;
    rec: TRtcArray;

  begin
  if (xID=0) then
    msg.Result:=1
  else
    begin
    // find out how many files we're accepting
    nCount := DragQueryFile( msg.WParam,
                             $FFFFFFFF,
                             acFileName,
                             cnMaxFileNameLen );
    rec:=TRtcArray.Create;
    try
      // query Windows one at a time for the file name
      for i := 0 to nCount-1 do
        begin
        DragQueryFile( msg.WParam, i, acFileName, cnMaxFileNameLen );
        rec.asText[rec.Count]:=acFileName;
        end;
    finally
      // let Windows know that you're done
      DragFinish( msg.WParam );
      end;
    c._Com.Event(xID,rec);
    msg.Result:=0;
    end;
  end;
 {$endif}

function ShiftStateToRtc(Shift:TShiftState):TRtcRecord;
  begin
  Result:=TRtcRecord.Create;
  if (ssShift in Shift) then Result.asBoolean['s']:=True;
  if (ssAlt	in Shift) then Result.asBoolean['a']:=True;
  if (ssCtrl in Shift) then Result.asBoolean['c']:=True;
  if (ssLeft in Shift) then Result.asBoolean['l']:=True;
  if (ssRight	in Shift) then Result.asBoolean['r']:=True;
  if (ssMiddle in Shift) then Result.asBoolean['m']:=True;
  if (ssDouble in Shift) then Result.asBoolean['d']:=True;
  end;

function Rtcf_Link(c:TComponent; const xName:string; xData:TComponent):boolean;
  function LinkControl(c:TControl):boolean;
    var
      v:boolean;
    begin
    Result:=True;
    if (xName='parent') then
      begin
      if assigned(xData) and (xData is TWinControl) then
        begin
        v:=c.Visible;
        if v then c.Visible:=False;
        c.Parent:=TWinControl(xData);
        if v then c.Visible:=True;
        end
      else
        c.Parent:=nil;
      end
    else if (xName='hostdocksite') then
      begin
      if assigned(xData) and (xData is TWinControl) then
        c.HostDockSite:=TWinControl(xData)
      else
        c.HostDockSite:=nil;
      end
    else if (xName='action') then
      begin
      if assigned(xData) and (xData is TBasicAction) then
        c.Action:=TBasicAction(xData)
      else
        c.Action:=nil;
      end
    else
      Result:=False;
    end;
  begin
  // Result:=True;
  if c is TControl then
    Result:=LinkControl(c as TControl)
  else
    Result:=False;
  end;

function Rtcf_ProcessComponent(c:TComponent; const xName:string; xData:TRtcValue):boolean;

  function ProcessWinControl(c:TWinControl):boolean;
    begin
    Result:=True;
    if xName='taborder' then
      c.TabOrder:=xData.asInteger
    else if xName='tabstop' then
      c.TabStop:=xData.asBoolean
    else if xName='setfocus' then
      c.SetFocus
    else
      Result:=False;
    end;

  function ProcessControl(c:TControl):boolean;
    procedure SetBounds;
      var
        rec:TRtcRecord;
        l,t,w,h:integer;
      begin
      if xData.isType=rtc_Record then
        begin
        rec:=xData.asRecord;
        if rec.isNull['l'] then l:=c.Left else l:=rec.asInteger['l'];
        if rec.isNull['t'] then t:=c.Top else t:=rec.asInteger['t'];
        if rec.isNull['w'] then w:=c.Width else w:=rec.asInteger['w'];
        if rec.isNull['h'] then h:=c.Height else h:=rec.asInteger['h'];
        c.SetBounds(l,t,w,h);
        end;
      end;
    procedure SetAnchors;
      var
        rec:TRtcRecord;
        a:TAnchors;
      begin
      if xData.isType=rtc_Record then
        begin
        a:=[];
        rec:=xData.asRecord;
        if not rec.isNull['l'] then Include(a,akLeft);
        if not rec.isNull['t'] then Include(a,akTop);
        if not rec.isNull['r'] then Include(a,akRight);
        if not rec.isNull['b'] then Include(a,akBottom);
        c.Anchors:=a;
        end;
      end;
    procedure SetConstraints;
      begin
      Result:=Rtcf_ProcessSizeConstraints(c.Constraints,xName,xData);
      end;
    procedure Set_Left;
      var
        wc:TWinControl;
      begin
      wc:=c.Parent;
      if assigned(wc) and Supports(wc,IRtcFormX) then
        c.Left:=xData.asInteger+(wc as IRtcFormX).GetFormLeft
      else
        c.Left:=xData.asInteger;
      end;
    procedure Set_Top;
      var
        wc:TWinControl;
      begin
      wc:=c.Parent;
      if assigned(wc) and Supports(wc,IRtcFormX) then
        c.Top:=xData.asInteger+(wc as IRtcFormX).GetFormTop
      else
        c.Top:=xData.asInteger;
      end;
    procedure Begin_Drag;
      var
        rec:TRtcRecord;
      begin
      if xData.isType=rtc_Record then
        begin
        rec:=xData.asRecord;
        c.BeginDrag(rec.asBoolean['i'],rec.asInteger['t']);
        end;
      end;
    begin
    Result:=True;
    if      xName='align' then       c.Align:=TAlign(xData.asInteger)
    else if xName='anchors' then     SetAnchors
    else if xName='bidimode' then    c.BiDiMode:=TBiDiMode(xData.asInteger)
    else if xName='bounds' then      SetBounds
    //else if xName='boundsrect' then  SetBoundsRect
    else if xName='cursor' then      c.Cursor:=TCursor(xData.asInteger)
    else if xName='dockorientation' then c.DockOrientation:=TDockOrientation(xData.asInteger)
    else if xName='enabled' then     c.Enabled:=xData.asBoolean
    else if xName='floatingdocksiteclass' then c.FloatingDockSiteClass:=TWinControlClass(GetClass(xData.asText+'C'))
    else if xName='helptype' then    c.HelpType:=THelpType(xData.asInteger)
    else if xName='helpkeyword' then c.HelpKeyword:=xData.asText
    else if xName='helpcontext' then c.HelpContext:=THelpContext(xData.asInteger)
    else if xName='hint' then        c.Hint:=xData.asText
    else if xName='left' then        Set_Left // c.Left:=xData.asInteger
    else if xName='top' then         Set_Top // c.Top:=xData.asInteger
    else if xName='height' then      c.Height:=xData.asInteger
    else if xName='width' then       c.Width:=xData.asInteger
    else if xName='showhint' then    c.ShowHint:=xData.asBoolean
    else if xName='visible' then     c.Visible:=xData.asBoolean
    else if xName='tofront' then     c.BringToFront
    else if xName='toback' then      c.SendToBack
    else if xName='refresh' then     c.Refresh
    else if xName='begindrag' then   Begin_Drag
    {$ifndef FPC}
    else if xName='enddrag' then     c.EndDrag(xData.asBoolean)
    {$endif}
    else if Copy(xName,1,6)='cnstr.' then SetConstraints
    else if c is TWinControl then
      Result:=ProcessWinControl(c as TWinControl)
    else
      Result:=False;
    end;
  begin
  Result:=True;
  if xName='tag' then
    c.Tag:=xData.asInteger
  else if c is TControl then
    Result:= ProcessControl(c as TControl)
  else
    Result:=False;
  end;

function Rtcf_ProcessSizeConstraints(c:TSizeConstraints; const xName:String; xData:TRtcValue):boolean;
  var
    prop:string;
  begin
  Result:=True;
  if pos('.',xName)>0 then // remove prefix
    prop:=Copy(xName,Pos('.',xName)+1,length(xName))
  else
    prop:=xName;

  if prop='w1' then
    c.MinWidth:=xData.asInteger
  else if prop='w2' then
    c.MaxWidth:=xData.asInteger
  else if prop='h1' then
    c.MinHeight:=xData.asInteger
  else if prop='h2' then
    c.MaxHeight:=xData.asInteger
  else
    Result:=False;
  end;

function Rtcf_ProcessControlScrollBar(c:TControlScrollBar; const xName:string; xData:TRtcValue):boolean;
  var
    prop:string;
  begin
  Result:=True;
  if pos('.',xName)>0 then // remove prefix
    prop:=Copy(xName,Pos('.',xName)+1,length(xName))
  else
    prop:=xName;

  if prop='increment' then
    c.Increment:=TScrollBarInc(xData.asInteger)
  else if prop='position' then
    c.Position:=xData.asInteger
  else if prop='range' then
    c.Range:=xData.asInteger
  else if prop='smooth' then
    c.Smooth:=xData.asBoolean
  else if prop='size' then
    c.Size:=xData.asInteger
  {$ifndef FPC}
  else if prop='buttonsize' then
    c.ButtonSize:=xData.asInteger
  else if prop='color' then
    c.Color:=TColor(xData.asInteger)
  else if prop='margin' then
    c.Margin:=xData.asInteger
  else if prop='parentcolor' then
    c.ParentColor:=xData.asBoolean
  else if prop='style' then
    c.Style:=TScrollBarStyle(xData.asInteger)
  else if prop='thumbsize' then
    c.ThumbSize:=xData.asInteger
  else if prop='tracking' then
    c.Tracking:=xData.asBoolean
  {$endif}
  else if prop='visible' then
    c.Visible:=xData.asBoolean
  else
    Result:=False;
  end;

{ TRtcControlEvents }

constructor TRtcControlEvents.Create(c:TRtcComC);
  begin
  inherited Create;
  FCom:=c;
  end;

destructor TRtcControlEvents.Destroy;
  begin
  inherited;
  end;

procedure TRtcControlEvents.MyOnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  procedure SendMouseDown;
    var
      rec:TRtcRecord;
    begin
    if Sender is TControl then
      Client2Screen(TControl(Sender),x,y);
    rec:=TRtcRecord.Create;
    rec.asInteger['b']:=Ord(Button);
    rec.asObject['s']:=ShiftStateToRtc(Shift);
    rec.asInteger['x']:=x;
    rec.asInteger['y']:=y;
    FCom.Event(FOnMouseDownID,rec);
    end;
  begin
  // This event ALWAYS has to be mapped to track mouse up/down state
  case Button of
    mbLeft: FMouseDown:=FMouseDown or 1;
    mbRight: FMouseDown:=FMouseDown or 2;
    mbMiddle: FMouseDown:=FMouseDown or 4;
    end;
  if FOnMouseDownID>0 then
    SendMouseDown;
  end;

procedure TRtcControlEvents.MyOnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  procedure SendMouseUp;
    var
      rec:TRtcRecord;
    begin
    if Sender is TControl then
      Client2Screen(TControl(Sender),x,y);
    rec:=TRtcRecord.Create;
    rec.asInteger['b']:=Ord(Button);
    rec.asObject['s']:=ShiftStateToRtc(Shift);
    rec.asInteger['x']:=x;
    rec.asInteger['y']:=y;
    FCom.Event(FOnMouseUpID,rec);
    end;
  begin
  // This event ALWAYS has to be mapped to track mouse up/down events
  case Button of
    mbLeft: FMouseDown:=FMouseDown and not 1;
    mbRight: FMouseDown:=FMouseDown and not 2;
    mbMiddle: FMouseDown:=FMouseDown and not 4;
    end;
  if FOnMouseUpID>0 then
    SendMouseUp;
  end;

procedure TRtcControlEvents.MyOnClick(Sender: TObject);
  begin
  FCom.Event(FOnClickID,nil);
  end;

procedure TRtcControlEvents.MyOnDblClick(Sender: TObject);
  begin
  FCom.Event(FOnDblClickID,nil);
  end;

procedure TRtcControlEvents.MyOnMouseEnter(Sender: TObject);
  begin
  FCom.Event(FOnMouseEnterID,nil);
  end;

procedure TRtcControlEvents.MyOnMouseLeave(Sender: TObject);
  begin
  FCom.Event(FOnMouseLeaveID,nil);
  end;

procedure TRtcControlEvents.MyOnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  procedure SendMouseMove(id:longint);
    var
      rec:TRtcRecord;
    begin
    if Sender is TControl then
      Client2Screen(TControl(Sender),x,y);
    rec:=TRtcRecord.Create;
    rec.asObject['s']:=ShiftStateToRtc(Shift);
    rec.asInteger['x']:=x;
    rec.asInteger['y']:=y;
    FCom.Event(id,rec);
    end;
  begin
  if FMouseDown>0 then
    begin
    if FOnMouseDownMoveID>0 then
      SendMouseMove(FOnMouseDownMoveID)
    else if FOnMouseMoveID>0 then
      SendMouseMove(FOnMouseMoveID);
    end
  else
    begin
    if FOnMouseUpMoveID>0 then
      SendMouseMove(FOnMouseUpMoveID)
    else if FOnMouseMoveID>0 then
      SendMouseMove(FOnMouseMoveID);
    end;
  end;

procedure TRtcControlEvents.MyOnDragDrop(Sender, Source: TObject; X, Y: Integer);
  var
    rec:TRtcRecord;
  begin
  if (FOnDragDropID>0) and (Source is TComponent) and Supports(Source,IRtcfComponentC) then
    begin
    if Sender is TControl then
      Client2Screen(TControl(Sender),x,y);
    rec:=TRtcRecord.Create;
    rec.asObject['o']:=(TComponent(Source) as IRtcfComponentC)._Com.GetUID;
    rec.asInteger['x']:=x;
    rec.asInteger['y']:=y;
    FCom.Event(FOnDragDropID,rec);
    end;
  end;

procedure TRtcControlEvents.MyOnDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
  begin
  Accept:=(FOnDragDropID>0) and (Source is TComponent) and Supports(Source,IRtcfComponentC);
  end;

procedure TRtcControlEvents.MyOnStartDrag(Sender: TObject; var DragObject: TDragObject);
  begin
  FCom.Event(FOnStartDragID,nil);
  end;

procedure TRtcControlEvents.MyOnEndDrag(Sender, Target: TObject; X, Y: Integer);
  var
    rec:TRtcRecord;
  begin
  if (FOnEndDragID>0) and (Target is TComponent) and Supports(Target,IRtcfComponentC) then
    begin
    rec:=TRtcRecord.Create;
    rec.asObject['o']:=(TComponent(Target) as IRtcfComponentC)._Com.GetUID;
    rec.asInteger['x']:=x;
    rec.asInteger['y']:=y;
    FCom.Event(FOnEndDragID,rec);
    end;
  end;

function TRtcControlEvents.OnMouseEnter(xID: Integer): TNotifyEvent;
  begin
  FOnMouseEnterID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnMouseEnter;
  end;

function TRtcControlEvents.OnMouseLeave(xID: integer): TNotifyEvent;
  begin
  FOnMouseLeaveID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnMouseLeave;
  end;

function TRtcControlEvents.OnClick(xID: Integer): TNotifyEvent;
  begin
  FOnClickID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnClick;
  end;

function TRtcControlEvents.OnDblClick(xID: integer): TNotifyEvent;
  begin
  FOnDblClickID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnDblClick;
  end;

function TRtcControlEvents.OnMouseDown(xID: integer): TMouseEvent;
  begin
  FOnMouseDownID:=xID;
  Result:=MyOnMouseDown;
  end;

function TRtcControlEvents.OnMouseUp(xID: integer): TMouseEvent;
  begin
  FOnMouseUpID:=xID;
  Result:=MyOnMouseUp;
  end;

function TRtcControlEvents.OnMouseDownMove(xID: integer): TMouseMoveEvent;
  begin
  FOnMouseDownMoveID:=xID;
  if (FOnMouseMoveID=0) and
     (FOnMouseUpMoveID=0) and
     (FOnMouseDownMoveID=0) then
    Result:=nil
  else
    Result:=MyOnMouseMove;
  end;

function TRtcControlEvents.OnMouseMove(xID: integer): TMouseMoveEvent;
  begin
  FOnMouseMoveID:=xID;
  if (FOnMouseMoveID=0) and
     (FOnMouseUpMoveID=0) and
     (FOnMouseDownMoveID=0) then
    Result:=nil
  else
    Result:=MyOnMouseMove;
  end;

function TRtcControlEvents.OnMouseUpMove(xID: integer): TMouseMoveEvent;
  begin
  FOnMouseUpMoveID:=xID;
  if (FOnMouseMoveID=0) and
     (FOnMouseUpMoveID=0) and
     (FOnMouseDownMoveID=0) then
    Result:=nil
  else
    Result:=MyOnMouseMove;
  end;

function TRtcControlEvents.OnDragDrop(xID: integer): TDragDropEvent;
  begin
  FOnDragDropID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnDragDrop;
  end;

function TRtcControlEvents.OnDragOver(xID: integer): TDragOverEvent;
  begin
  FOnDragDropID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnDragOver;
  end;

function TRtcControlEvents.OnStartDrag(xID: integer): TStartDragEvent;
  begin
  FOnStartDragID:=xID;
  if xID>0 then
    Result:=MyOnStartDrag
  else
    Result:=nil;
  end;

function TRtcControlEvents.OnEndDrag(xID: integer): TEndDragEvent;
  begin
  FOnEndDragID:=xID;
  if xID>0 then
    Result:=MyOnEndDrag
  else
    Result:=nil;
  end;

procedure TRtcControlEvents.Client2Screen(Sender:TControl; var x, y: integer);
  var
    wc:TWinControl;
  begin
  wc:=Sender.Parent;
  if assigned(wc) then
    begin
    Inc(x,Sender.Left);
    Inc(y,Sender.Top);
    repeat
      if assigned(wc.Parent) then
        begin
        Inc(x,wc.Left);
        Inc(y,wc.Top);
        wc:=wc.Parent;
        end
      else
        begin
        if Supports(wc,IRtcFormX) then
          begin
          Dec(x,(wc as IRtcFormX).GetFormLeft);
          Dec(y,(wc as IRtcFormX).GetFormTop);
          end;
        Break;
        end;
      until not assigned(wc);
    end
  else if Supports(Sender,IRtcFormX) then
    begin
    Dec(x,(Sender as IRtcFormX).GetFormLeft);
    Dec(y,(Sender as IRtcFormX).GetFormTop);
    end;
  end;

{ TRtcWinControlEvents }

procedure TRtcWinControlEvents.MyOnResize(Sender: TObject);
  begin
  // This event ALWAYS has to be mapped, to track width and height changes
  if FCom.Idle then
    begin
    if Sender is TControl then
      begin
      FCom.Prop('w',TControl(Sender).ClientWidth);
      FCom.Prop('h',TControl(Sender).ClientHeight);
      end;
    if (FOnResizeID>0) then
      FCom.Event(FOnResizeID,nil);
    end;
  end;

procedure TRtcWinControlEvents.MyOnActivate(Sender: TObject);
  begin
  FCom.Event(FOnActivateID,nil);
  end;

procedure TRtcWinControlEvents.MyOnDeactivate(Sender: TObject);
  begin
  FCom.Event(FOnDeactivateID,nil);
  end;

procedure TRtcWinControlEvents.MyOnEnter(Sender: TObject);
  begin
  FCom.Event(FOnEnterID,nil);
  end;

procedure TRtcWinControlEvents.MyOnExit(Sender: TObject);
  begin
  FCom.Event(FOnExitID,nil);
  end;

procedure TRtcWinControlEvents.MyOnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  var
    rec:TRtcRecord;
  begin
  rec:=TRtcRecord.Create;
  rec.asInteger['k']:=Key;
  rec.asObject['s']:=ShiftStateToRtc(Shift);
  FCom.Event(FOnKeyDownID,rec);
  end;

procedure TRtcWinControlEvents.MyOnKeyPress(Sender: TObject; var Key: Char);
  begin
  FCom.Event(FOnKeyPressID,Key);
  end;

procedure TRtcWinControlEvents.MyOnKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
  var
    rec:TRtcRecord;
  begin
  rec:=TRtcRecord.Create;
  rec.asInteger['k']:=key;
  rec.asObject['s']:=ShiftStateToRtc(Shift);
  FCom.Event(FOnKeyUpID,rec);
  end;

procedure TRtcWinControlEvents.MyOnMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
  var
    rec:TRtcRecord;
  begin
  rec:=TRtcRecord.Create;
  rec.asObject['s']:=ShiftStateToRtc(Shift);
  rec.asInteger['w']:=Ord(WheelDelta);
  rec.asInteger['x']:=MousePos.X;
  rec.asInteger['y']:=MousePos.Y;
  FCom.Event(FOnMouseWheelID,rec);
  end;

procedure TRtcWinControlEvents.MyOnMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
  var
    rec:TRtcRecord;
  begin
  rec:=TRtcRecord.Create;
  rec.asObject['s']:=ShiftStateToRtc(Shift);
  rec.asInteger['x']:=MousePos.X;
  rec.asInteger['y']:=MousePos.Y;
  FCom.Event(FOnMouseWheelUpID,rec);
  end;

procedure TRtcWinControlEvents.MyOnMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
  var
    rec:TRtcRecord;
  begin
  rec:=TRtcRecord.Create;
  rec.asObject['s']:=ShiftStateToRtc(Shift);
  rec.asInteger['x']:=MousePos.X;
  rec.asInteger['y']:=MousePos.Y;
  FCom.Event(FOnMouseWheelDownID,rec);
  end;

function TRtcWinControlEvents.OnActivate(xID: Integer): TNotifyEvent;
  begin
  FOnActivateID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnActivate;
  end;

function TRtcWinControlEvents.OnDeactivate(xID: integer): TNotifyEvent;
  begin
  FOnDeactivateID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnDeactivate;
  end;

function TRtcWinControlEvents.OnEnter(xID: Integer): TNotifyEvent;
  begin
  FOnEnterID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnEnter;
  end;

function TRtcWinControlEvents.OnExit(xID: integer): TNotifyEvent;
  begin
  FOnExitID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnExit;
  end;

function TRtcWinControlEvents.OnResize(xID: integer): TNotifyEvent;
  begin
  FOnResizeID:=xID;
  Result:=MyOnResize;
  end;

function TRtcWinControlEvents.OnKeyDown(xID: integer): TKeyEvent;
  begin
  FOnKeyDownID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnKeyDown;
  end;

function TRtcWinControlEvents.OnKeyPress(xID: integer): TKeyPressEvent;
  begin
  FOnKeyPressID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnKeyPress;
  end;

function TRtcWinControlEvents.OnKeyUp(xID: integer): TKeyEvent;
  begin
  FOnKeyUpID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnKeyUp;
  end;

function TRtcWinControlEvents.OnMouseWheel(xID: integer): TMouseWheelEvent;
  begin
  FOnMouseWheelID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnMouseWheel;
  end;

function TRtcWinControlEvents.OnMouseWheelDown(xID: integer): TMouseWheelUpDownEvent;
  begin
  FOnMouseWheelDownID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnMouseWheelDown;
  end;

function TRtcWinControlEvents.OnMouseWheelUp(xID: integer): TMouseWheelUpDownEvent;
  begin
  FOnMouseWheelUpID:=xID;
  if xID=0 then
    Result:=nil
  else
    Result:=MyOnMouseWheelUp;
  end;

end.
