{
  RealThinClient W/A Framework: Server-side Application, Form and DataModule implementations
  Copyright (c) Danijel Tkalcec

  @exclude
}
unit rtcForms;

interface

uses
  rtcTrashcan, rtcLog,
  Windows, Classes, SysUtils, Types, Graphics,
  {$IFNDEF NoForms} Controls {$ELSE} _Controls {$ENDIF},
  rtcZLib, rtcInfo,

  rtcBase,
  rtcGraphics, rtcControls;

type
  TrtcApplication=class(TRtcBaseApplication)
  private
    FInfo:TRtcInfo;
    FUserID:AnsiString;

    function Get_Info: TRtcInfo;
    function Get_UserID: AnsiString;
    procedure Set_UserID(const Value: AnsiString);

  protected
    procedure _Send(Data:TRtcValueObject); override;

  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

    property UserID:AnsiString read Get_UserID write Set_UserID;
    property Info:TRtcInfo read Get_Info;
    end;

  TrtcDataModule=class(TRtcBaseDataModule)
  private
    procedure Set_Tag(const Value: Longint);
    function Get_Tag: Longint;

    function Get_Application: TrtcApplication;

  protected
    procedure _Process(const xName:string; xData:TRtcValue); override;

  public
    constructor CreateNew(AOwner:TComponent; Dummy:integer=0); override;
    destructor Destroy; override;

    property Application:TrtcApplication read Get_Application;

  published
    property Tag: Longint read Get_Tag write Set_Tag default 0;
    end;

  TrtcForm=class(TRtcBaseForm)
  private
    FAutoScroll:boolean;
    FHorzBar,FVertBar:TRtcControlScrollBar;

    FAction: IRtcBasicAction;
    FActiveControl: TRtcWinControl;
    FMenu: IRtcMainMenu;
    FPopupMenu: IRtcPopupMenu;

    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnActivate: TNotifyEvent;
    FOnDeactivate: TNotifyEvent;

    FOnDragDrop: TRtcDragDropEvent;
    FOnDropFiles: TRtcDropFilesEvent;

    FOnResize: TNotifyEvent;
    FOnKeyDown: TRtcKeyEvent;
    FOnKeyUp: TRtcKeyEvent;
    FOnKeyPress: TRtcKeyPressEvent;
    FOnMouseDown: TRtcMouseEvent;
    FOnMouseUp: TRtcMouseEvent;
    FOnMouseMove: TRtcMouseMoveEvent;
    FOnMouseUpMove: TRtcMouseMoveEvent;
    FOnMouseDownMove: TRtcMouseMoveEvent;
    FOnMouseWheel: TRtcMouseWheelEvent;
    FOnMouseWheelUp: TRtcMouseWheelUpDownEvent;
    FOnMouseWheelDown: TRtcMouseWheelUpDownEvent;

    procedure Set_Caption(const Value: string);
    procedure Set_AutoScroll(const Value: Boolean);
    procedure Set_AutoSize(const Value: Boolean);
    procedure Set_BorderIcons(const Value: TRtcBorderIcons);
    procedure Set_BorderStyle(const Value: TRtcFormBorderStyle);
    procedure Set_BorderWidth(const Value: TBorderWidth);
    procedure Set_Color(const Value: TColor);
    procedure Set_Ctl3D(const Value: Boolean);
    procedure Set_DragKind(const Value: TDragKind);
    procedure Set_DragMode(const Value: TDragMode);
    procedure Set_Enabled(const Value: Boolean);
    procedure Set_FormStyle(const Value: TRtcFormStyle);
    procedure Set_Cursor(const Value: TCursor);
    procedure Set_Hint(const Value: string);
    procedure Set_HelpKeyword(const Value: String);
    procedure Set_HelpType(const Value: THelpType);
    procedure Set_Tag(const Value: Longint);
    procedure Set_HelpContext(const Value: THelpContext);
    procedure Set_BiDiMode(const Value: TBiDiMode);
    procedure Set_Position(const Value: TRtcPosition);
    procedure Set_ShowHint(const Value: Boolean);
    procedure Set_TransparentColor(const Value: Boolean);
    procedure Set_TransparentColorValue(const Value: TColor);
    procedure Set_WindowState(const Value: TRtcWindowState);
    procedure Set_KeyPreview(const Value: Boolean);

    procedure Set_OnDragDrop(const Value: TRtcDragDropEvent);

    function Get_AutoScroll: Boolean;
    function Get_AutoSize: Boolean;
    function Get_BiDiMode: TBiDiMode;
    function Get_BorderIcons: TRtcBorderIcons;
    function Get_BorderStyle: TRtcFormBorderStyle;
    function Get_BorderWidth: TBorderWidth;
    function Get_Caption: string;
    function Get_Color: TColor;
    function Get_Ctl3D: Boolean;
    function Get_Cursor: TCursor;
    function Get_DragKind: TDragKind;
    function Get_DragMode: TDragMode;
    function Get_Enabled: Boolean;
    function Get_FormStyle: TRtcFormStyle;
    function Get_HelpContext: THelpContext;
    function Get_HelpKeyword: String;
    function Get_HelpType: THelpType;
    function Get_Hint: string;
    function Get_KeyPreview: Boolean;
    function Get_Position: TRtcPosition;
    function Get_ShowHint: Boolean;
    function Get_Tag: Longint;
    function Get_TransparentColor: Boolean;
    function Get_TransparentColorValue: TColor;
    function Get_WindowState: TRtcWindowState;

    procedure Set_HorzScrollBar(const Value: TRtcControlScrollBar);
    procedure Set_VertScrollBar(const Value: TRtcControlScrollBar);

    function Get_HorzScrollBar: TRtcControlScrollBar;
    function Get_VertScrollBar: TRtcControlScrollBar;

    function Get_Align: TAlign;
    function Get_AlphaBlend: Boolean;
    function Get_AlphaBlendValue: byte;

    procedure Set_Align(const Value: TAlign);
    procedure Set_AlphaBlend(const Value: Boolean);
    procedure Set_AlphaBlendValue(const Value: byte);

    function IsForm: Boolean;

    function Get_ScreenSnap: Boolean;
    function Get_SnapBuffer: integer;
    procedure Set_ScreenSnap(const Value: Boolean);
    procedure Set_SnapBuffer(const Value: integer);

    function Get_DefaultMonitor: TRtcDefaultMonitor;
    procedure Set_DefaultMonitor(const Value: TRtcDefaultMonitor);

    function Get_Application: TrtcApplication;

    procedure Set_OnClick(const Value: TNotifyEvent);
    procedure Set_OnDblClick(const Value: TNotifyEvent);
    procedure Set_OnActivate(const Value: TNotifyEvent);
    procedure Set_OnDeactivate(const Value: TNotifyEvent);
    procedure Set_OnResize(const Value: TNotifyEvent);

    procedure Set_OnKeyDown(const Value: TRtcKeyEvent);
    procedure Set_OnKeyPress(const Value: TRtcKeyPressEvent);
    procedure Set_OnKeyUp(const Value: TRtcKeyEvent);
    procedure Set_OnMouseDown(const Value: TRtcMouseEvent);
    procedure Set_OnMouseDownMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnMouseMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnMouseUp(const Value: TRtcMouseEvent);
    procedure Set_OnMouseUpMove(const Value: TRtcMouseMoveEvent);
    procedure Set_OnMouseWheel(const Value: TRtcMouseWheelEvent);
    procedure Set_OnMouseWheelDown(const Value: TRtcMouseWheelUpDownEvent);
    procedure Set_OnMouseWheelUp(const Value: TRtcMouseWheelUpDownEvent);

    function Get_Action: IRtcBasicAction;
    function Get_ActiveControl: TRtcWinControl;
    function Get_Menu: IRtcMainMenu;
    function Get_PopupMenu: IRtcPopupMenu;

    procedure Set_Action(const Value: IRtcBasicAction);
    procedure Set_ActiveControl(const Value: TRtcWinControl);
    procedure Set_Menu(const Value: IRtcMainMenu);
    procedure Set_PopupMenu(const Value: IRtcPopupMenu);
    procedure Set_OnDropFiles(const Value: TRtcDropFilesEvent);

  protected
    procedure MyOnClick(xData:TRtcValue);
    procedure MyOnDblClick(xData:TRtcValue);
    procedure MyOnActivate(xData:TRtcValue);
    procedure MyOnDeactivate(xData:TRtcValue);
    procedure MyOnResize(xData:TRtcValue);

    procedure MyOnDragDrop(xData:TRtcValue);
    procedure MyOnDropFiles(xData:TRtcValue);

    procedure MyOnKeyDown(xData:TRtcValue);
    procedure MyOnKeyUp(xData:TRtcValue);
    procedure MyOnKeyPress(xData:TRtcValue);

    procedure MyOnMouseDown(xData:TRtcValue);
    procedure MyOnMouseUp(xData:TRtcValue);
    procedure MyOnMouseMove(xData:TRtcValue);
    procedure MyOnMouseDownMove(xData:TRtcValue);
    procedure MyOnMouseUpMove(xData:TRtcValue);

    procedure MyOnMouseWheel(xData:TRtcValue);
    procedure MyOnMouseWheelUp(xData:TRtcValue);
    procedure MyOnMouseWheelDown(xData:TRtcValue);
    
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

  public
    constructor CreateNew(AOwner:TComponent; Dummy:integer=0); override;
    destructor Destroy; override;

    property Application:TrtcApplication read Get_Application;

  published // Properties

    property Action: IRtcBasicAction read Get_Action write Set_Action;
    property ActiveControl: TRtcWinControl read Get_ActiveControl write Set_ActiveControl;
    property Menu: IRtcMainMenu read Get_Menu write Set_Menu;
    property PopupMenu: IRtcPopupMenu read Get_PopupMenu write Set_PopupMenu;

    property Align: TAlign read Get_Align write Set_Align default alNone;

    property ScreenSnap: Boolean read Get_ScreenSnap write Set_ScreenSnap default False;
    property SnapBuffer: integer read Get_SnapBuffer write Set_SnapBuffer default 10;

    property DefaultMonitor: TRtcDefaultMonitor read Get_DefaultMonitor write Set_DefaultMonitor default dmActiveForm;
    property AutoSize: Boolean read Get_AutoSize write Set_AutoSize default False;
    property AutoScroll: Boolean read Get_AutoScroll write Set_AutoScroll default True;
    property BorderStyle: TRtcFormBorderStyle read Get_BorderStyle write Set_BorderStyle default bsSizeable;
    property BorderIcons: TRtcBorderIcons read Get_BorderIcons write Set_BorderIcons default [biSystemMenu, biMinimize, biMaximize];
    property BorderWidth: TBorderWidth read Get_BorderWidth write Set_BorderWidth default 0;
    property Ctl3D: Boolean read Get_Ctl3D write Set_Ctl3D default True;
    property DragKind: TDragKind read Get_DragKind write Set_DragKind default dkDrag;
    property DragMode: TDragMode read Get_DragMode write Set_DragMode default dmManual;
    property Enabled: Boolean read Get_Enabled write Set_Enabled default True;
    property FormStyle: TRtcFormStyle read Get_FormStyle write Set_FormStyle default fsNormal;
    property KeyPreview: Boolean read Get_KeyPreview write Set_KeyPreview default False;
    property BiDiMode: TBiDiMode read Get_BiDiMode write Set_BiDiMode default bdLeftToRight;
    property WindowState: TRtcWindowState read Get_WindowState write Set_WindowState default wsNormal;

    property ShowHint: Boolean read Get_ShowHint write Set_ShowHint default False;
    property Position: TRtcPosition read Get_Position write Set_Position default poDesigned;

    property TransparentColor: Boolean read Get_TransparentColor write Set_TransparentColor default False;
    property TransparentColorValue: TColor read Get_TransparentColorValue write Set_TransparentColorValue default 0;

    property AlphaBlend: Boolean read Get_AlphaBlend write Set_AlphaBlend default False;
    property AlphaBlendValue: byte read Get_AlphaBlendValue write Set_AlphaBlendValue default 255;

    property Caption: string read Get_Caption write Set_Caption;
    property Color: TColor read Get_Color write Set_Color default clWindow;

    property Tag: Longint read Get_Tag write Set_Tag default 0;
    property Cursor: TCursor read Get_Cursor write Set_Cursor default crDefault;
    property Hint: string read Get_Hint write Set_Hint;
    property HelpType: THelpType read Get_HelpType write Set_HelpType default htContext;
    property HelpKeyword: String read Get_HelpKeyword write Set_HelpKeyword;
    property HelpContext: THelpContext read Get_HelpContext write Set_HelpContext default 0;

    property HorzScrollBar: TRtcControlScrollBar read Get_HorzScrollBar write Set_HorzScrollBar stored IsForm;
    property VertScrollBar: TRtcControlScrollBar read Get_VertScrollBar write Set_VertScrollBar stored IsForm;

  // Events
    property OnClick:TNotifyEvent read FOnClick write Set_OnClick;
    property OnDblClick:TNotifyEvent read FOnDblClick write Set_OnDblClick;
    property OnActivate:TNotifyEvent read FOnActivate write Set_OnActivate;
    property OnDeactivate:TNotifyEvent read FOnDeactivate write Set_OnDeactivate;
    property OnResize:TNotifyEvent read FOnResize write Set_OnResize;

    property OnDragDrop: TRtcDragDropEvent read FOnDragDrop write Set_OnDragDrop;
    property OnDropFiles: TRtcDropFilesEvent read FOnDropFiles write Set_OnDropFiles;

    property OnKeyDown:TRtcKeyEvent read FOnKeyDown write Set_OnKeyDown;
    property OnKeyUp:TRtcKeyEvent read FOnKeyUp write Set_OnKeyUp;
    property OnKeyPress:TRtcKeyPressEvent read FOnKeyPress write Set_OnKeyPress;

    property OnMouseDown:TRtcMouseEvent read FOnMouseDown write Set_OnMouseDown;
    property OnMouseDownMove:TRtcMouseMoveEvent read FOnMouseDownMove write Set_OnMouseDownMove;
    property OnMouseMove:TRtcMouseMoveEvent read FOnMouseMove write Set_OnMouseMove;
    property OnMouseUp:TRtcMouseEvent read FOnMouseUp write Set_OnMouseUp;
    property OnMouseUpMove:TRtcMouseMoveEvent read FOnMouseUpMove write Set_OnMouseUpMove;

    property OnMouseWheel:TRtcMouseWheelEvent read FOnMouseWheel write Set_OnMouseWheel;
    property OnMouseWheelUp:TRtcMouseWheelUpDownEvent read FOnMouseWheelUp write Set_OnMouseWheelUp;
    property OnMouseWheelDown:TRtcMouseWheelUpDownEvent read FOnMouseWheelDown write Set_OnMouseWheelDown;
    end;

implementation

{ TRtcApplication }

constructor TrtcApplication.Create(AOwner: TComponent);
  begin
  inherited;
  FUserID:='';
  FInfo:=TRtcInfo.Create;
  end;

destructor TrtcApplication.Destroy;
  begin
  FUserID:='';
  FreeAndNil(FInfo);
  inherited;
  end;

function TrtcApplication.Get_Info: TRtcInfo;
  begin
  Result:=FInfo;
  end;

function TrtcApplication.Get_UserID: AnsiString;
  begin
  Result:=FUserID;
  end;

procedure TrtcApplication.Set_UserID(const Value: AnsiString);
  begin
  FUserID:=Value;
  end;

procedure TrtcApplication._Send(Data: TRtcValueObject);
  begin
  // Log(Data.toCode,'TrtcFormServer');
  inherited;
  end;

{ TRtcDataModule }

constructor TrtcDataModule.CreateNew(AOwner: TComponent; Dummy:integer=0);
  begin
  inherited;
  end;

destructor TrtcDataModule.Destroy;
  begin
  inherited;
  end;

function TrtcDataModule.Get_Application: TrtcApplication;
  begin
  if _App<>nil then
    Result:=TrtcApplication(_App)
  else
    Result:=nil;
  end;

function TrtcDataModule.Get_Tag: Longint;
  begin
  Result:=inherited Tag;
  end;

procedure TrtcDataModule.Set_Tag(const Value: Longint);
  begin
  if inherited Tag<>Value then
    begin
    inherited Tag := Value;
    _Com.Prop('tag',Value);
    end;
  end;

procedure TrtcDataModule._Process(const xName: string; xData: TRtcValue);
  begin
  // Execute Form event "xName" with parameters "xData"
  end;

{ TRtcForm }

constructor TrtcForm.CreateNew(AOwner: TComponent; Dummy:integer=0);
  begin
  inherited;
  FHorzBar:=TRtcControlScrollBar.Create;
  FHorzBar._Com:=_Com;
  FHorzBar.FullName:='hscroll.';
  FHorzBar.AutoScroll:=True;

  FVertBar:=TRtcControlScrollBar.Create;
  FVertBar._Com:=_Com;
  FVertBar.FullName:='vscroll.';
  FVertBar.AutoScroll:=True;

  FAutoScroll:=True;
  end;

destructor TrtcForm.Destroy;
  begin
  _Detach;
  FAction:=nil;
  FMenu:=nil;
  FPopupMenu:=nil;
  FreeAndNil(FHorzBar);
  FreeAndNil(FVertBar);
  inherited;
  end;

procedure TrtcForm.Set_Tag(const Value: Longint);
  begin
  if inherited Tag<>Value then
    begin
    inherited Tag := Value;
    _Com.Prop('tag',Value);
    end;
  end;

procedure TrtcForm.Set_Cursor(const Value: TCursor);
  begin
  if inherited Cursor<>Value then
    begin
    inherited Cursor := Value;
    _Com.Prop('cursor',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_Hint(const Value: string);
  begin
  if inherited Hint<>Value then
    begin
    inherited Hint := Value;
    _Com.Prop('hint',Value);
    end;
  end;

procedure TrtcForm.Set_HelpKeyword(const Value: String);
  begin
  if inherited HelpKeyword<>Value then
    begin
    inherited HelpKeyword := Value;
    _Com.Prop('helpkeyword',Value);
    end;
  end;

procedure TrtcForm.Set_HelpType(const Value: THelpType);
  begin
  if inherited HelpType<>Value then
    begin
    inherited HelpType := Value;
    _Com.Prop('helptype',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_HelpContext(const Value: THelpContext);
  begin
  if inherited HelpContext<>Value then
    begin
    inherited HelpContext := Value;
    _Com.Prop('helpcontext',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_Caption(const Value: string);
  begin
  if inherited Caption<>Value then
    begin
    inherited Caption := Value;
    _Com.Prop('caption',Value);
    end;
  end;

procedure TrtcForm.Set_AutoScroll(const Value: Boolean);
  begin
  if FAutoScroll<>Value then
    begin
    FVertBar.AutoScroll:=Value;
    FHorzBar.AutoScroll:=Value;
    FAutoScroll := Value;
    _Com.Prop('autoscroll',Value);
    end;
  end;

procedure TrtcForm.Set_AutoSize(const Value: Boolean);
  begin
  if inherited AutoSize<>Value then
    begin
    inherited AutoSize := Value;
    _Com.Prop('autosize',Value);
    end;
  end;

procedure TrtcForm.Set_BorderIcons(const Value: TRtcBorderIcons);
  var
    rec:TRtcRecord;
  begin
  if inherited BorderIcons<>Value then
    begin
    inherited BorderIcons := Value;
    rec:=TRtcRecord.Create;
    if biSystemMenu in Value then rec.asBoolean['sy']:=True;
    if biMinimize in Value then rec.asBoolean['mi']:=True;
    if biMaximize in Value then rec.asBoolean['ma']:=True;
    if biHelp in Value then rec.asBoolean['he']:=True;
    _Com.Prop('bordericons',rec);
    end;
  end;

procedure TrtcForm.Set_BorderStyle(const Value: TRtcFormBorderStyle);
  begin
  if inherited BorderStyle<>Value then
    begin
    inherited BorderStyle := Value;
    _Com.Prop('borderstyle',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_BorderWidth(const Value: TBorderWidth);
  begin
  if inherited BorderWidth<>Value then
    begin
    inherited BorderWidth := Value;
    _Com.Prop('borderwidth',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_Color(const Value: TColor);
  begin
  if inherited Color<>Value then
    begin
    inherited Color := Value;
    _Com.Prop('color',Ord(value));
    end;
  end;

procedure TrtcForm.Set_Ctl3D(const Value: Boolean);
  begin
  if inherited Ctl3D<>Value then
    begin
    inherited Ctl3D := Value;
    _Com.Prop('ctl3d',Value);
    end;
  end;

procedure TrtcForm.Set_DragKind(const Value: TDragKind);
  begin
  if inherited DragKind<>Value then
    begin
    inherited DragKind := Value;
    _Com.Prop('dragkind',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_DragMode(const Value: TDragMode);
  begin
  if inherited DragMode<>Value then
    begin
    inherited DragMode := Value;
    _Com.Prop('dragmode',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_Enabled(const Value: Boolean);
  begin
  if inherited Enabled<>Value then
    begin
    inherited Enabled := Value;
    _Com.Prop('enabled',Value);
    end;
  end;

procedure TrtcForm.Set_FormStyle(const Value: TRtcFormStyle);
  begin
  if inherited FormStyle<>Value then
    begin
    inherited FormStyle := Value;
    _Com.Prop('formstyle',Value);
    end;
  end;

procedure TrtcForm.Set_BiDiMode(const Value: TBiDiMode);
  begin
  if inherited BiDiMode<>Value then
    begin
    inherited BiDiMode := Value;
    _Com.Prop('bidimode',Ord(value));
    end;
  end;

procedure TrtcForm.Set_Position(const Value: TRtcPosition);
  begin
  if inherited Position<>Value then
    begin
    inherited Position := Value;
    _Com.Prop('position',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_ShowHint(const Value: Boolean);
  begin
  if inherited ShowHint<>Value then
    begin
    inherited ShowHint := Value;
    _Com.Prop('showhint',Value);
    end;
  end;

procedure TrtcForm.Set_TransparentColor(const Value: Boolean);
  begin
  if inherited TransparentColor<>Value then
    begin
    inherited TransparentColor := Value;
    _Com.Prop('transparentcolor',Value);
    end;
  end;

procedure TrtcForm.Set_TransparentColorValue(const Value: TColor);
  begin
  if inherited TransparentColorValue<>Value then
    begin
    inherited TransparentColorValue := Value;
    _Com.Prop('transparentcolorvalue',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_WindowState(const Value: TRtcWindowState);
  begin
  if inherited WindowState<>Value then
    begin
    inherited WindowState := Value;
    _Com.Prop('windowstate',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_KeyPreview(const Value: Boolean);
  begin
  if inherited KeyPreview<>Value then
    begin
    inherited KeyPreview:=Value;
    _Com.Prop('keypreview',Value);
    end;
  end;

function TrtcForm.Get_AutoScroll: Boolean;
  begin
  Result:=FAutoScroll;
  end;

function TrtcForm.Get_AutoSize: Boolean;
  begin
  Result:=inherited AutoSize;
  end;

function TrtcForm.Get_BiDiMode: TBiDiMode;
  begin
  Result:=inherited BiDiMode;
  end;

function TrtcForm.Get_BorderIcons: TRtcBorderIcons;
  begin
  Result:=inherited BorderIcons;
  end;

function TrtcForm.Get_BorderStyle: TRtcFormBorderStyle;
  begin
  Result:=inherited BorderStyle;
  end;

function TrtcForm.Get_BorderWidth: TBorderWidth;
  begin
  Result:=inherited BorderWidth;
  end;

function TrtcForm.Get_Caption: string;
  begin
  Result:=inherited Caption;
  end;

function TrtcForm.Get_Color: TColor;
  begin
  Result:=inherited Color;
  end;

function TrtcForm.Get_Ctl3D: Boolean;
  begin
  Result:=inherited Ctl3D;
  end;

function TrtcForm.Get_Cursor: TCursor;
  begin
  Result:=inherited Cursor;
  end;

function TrtcForm.Get_DragKind: TDragKind;
  begin
  Result:=inherited DragKind;
  end;

function TrtcForm.Get_DragMode: TDragMode;
  begin
  Result:=inherited DragMode;
  end;

function TrtcForm.Get_Enabled: Boolean;
  begin
  Result:=inherited Enabled;
  end;

function TrtcForm.Get_FormStyle: TRtcFormStyle;
  begin
  Result:=inherited FormStyle;
  end;

function TrtcForm.Get_HelpContext: THelpContext;
  begin
  Result:=inherited HelpContext;
  end;

function TrtcForm.Get_HelpKeyword: String;
  begin
  Result:=inherited HelpKeyword;
  end;

function TrtcForm.Get_HelpType: THelpType;
  begin
  Result:=inherited HelpType;
  end;

function TrtcForm.Get_Hint: string;
  begin
  Result:=inherited Hint;
  end;

function TrtcForm.Get_KeyPreview: Boolean;
  begin
  Result:=inherited KeyPreview;
  end;

function TrtcForm.Get_Position: TRtcPosition;
  begin
  Result:=inherited Position;
  end;

function TrtcForm.Get_ShowHint: Boolean;
  begin
  Result:=inherited ShowHint;
  end;

function TrtcForm.Get_Tag: Longint;
  begin
  Result:=inherited Tag;
  end;

function TrtcForm.Get_TransparentColor: Boolean;
  begin
  Result:=inherited TransparentColor;
  end;

function TrtcForm.Get_TransparentColorValue: TColor;
  begin
  Result:=inherited TransparentColorValue;
  end;

function TrtcForm.Get_WindowState: TRtcWindowState;
  begin
  Result:=inherited WindowState;
  end;

function TrtcForm.Get_Align: TAlign;
  begin
  Result:=inherited Align;
  end;

procedure TrtcForm.Set_Align(const Value: TAlign);
  begin
  if Value<>inherited Align then
    begin
    inherited Align:=Value;
    _Com.Prop('align',Ord(Value));
    end;
  end;

procedure TrtcForm.Set_HorzScrollBar(const Value: TRtcControlScrollBar);
  begin
  FHorzBar.Assign(Value);
  end;

procedure TrtcForm.Set_VertScrollBar(const Value: TRtcControlScrollBar);
  begin
  FVertBar.Assign(Value);
  end;

function TrtcForm.Get_HorzScrollBar: TRtcControlScrollBar;
  begin
  Result:=FHorzBar;
  end;

function TrtcForm.Get_VertScrollBar: TRtcControlScrollBar;
  begin
  Result:=FVertBar;
  end;

function TrtcForm.IsForm: Boolean;
  begin
  Result:=not IsControl;
  end;

function TrtcForm.Get_AlphaBlend: Boolean;
  begin
  Result:=inherited AlphaBlend;
  end;

function TrtcForm.Get_AlphaBlendValue: byte;
  begin
  Result:=inherited AlphaBlendValue;
  end;

procedure TrtcForm.Set_AlphaBlend(const Value: Boolean);
  begin
  if Value<>inherited AlphaBlend then
    begin
    inherited AlphaBlend:=Value;
    _Com.Prop('alphablend',Value);
    end;
  end;

procedure TrtcForm.Set_AlphaBlendValue(const Value: byte);
  begin
  if Value<>inherited AlphaBlendValue then
    begin
    inherited AlphaBlendValue:=Value;
    _Com.Prop('alphablendvalue',Value);
    end;
  end;

function TrtcForm.Get_ScreenSnap: Boolean;
  begin
  Result:=inherited ScreenSnap;
  end;

function TrtcForm.Get_SnapBuffer: integer;
  begin
  Result:=inherited SnapBuffer;
  end;

procedure TrtcForm.Set_ScreenSnap(const Value: Boolean);
  begin
  if Value<>inherited ScreenSnap then
    begin
    inherited ScreenSnap:=Value;
    _Com.Prop('screensnap',Value);
    end;
  end;

procedure TrtcForm.Set_SnapBuffer(const Value: integer);
  begin
  if Value<>inherited SnapBuffer then
    begin
    inherited SnapBuffer:=Value;
    _Com.Prop('snapbuffer',Value);
    end;
  end;

function TrtcForm.Get_DefaultMonitor: TRtcDefaultMonitor;
  begin
  Result:=inherited DefaultMonitor;
  end;

procedure TrtcForm.Set_DefaultMonitor(const Value: TRtcDefaultMonitor);
  begin
  if Value<>inherited DefaultMonitor then
    begin
    inherited DefaultMonitor:=Value;
    _Com.Prop('defaultmonitor',Ord(Value));
    end;
  end;

function TrtcForm.Get_Application: TrtcApplication;
  begin
  if _App<>nil then
    Result:=TrtcApplication(_App)
  else
    Result:=nil;
  end;

procedure TrtcForm.Set_OnClick(const Value: TNotifyEvent);
  begin
  FOnClick := Value;
  if assigned(FOnClick) then
    _Com.Event('onclick',MyOnClick)
  else
    _Com.Event('onclick',nil);
  end;

procedure TrtcForm.MyOnClick(xData: TRtcValue);
  begin
  if assigned(FOnClick) then
    FOnClick(self);
  end;

procedure TrtcForm.Set_OnDblClick(const Value: TNotifyEvent);
  begin
  FOnDblClick := Value;
  if assigned(FOnDblClick) then
    _Com.Event('ondblclick',MyOnDblClick)
  else
    _Com.Event('ondblclick',nil);
  end;

procedure TrtcForm.MyOnDblClick(xData: TRtcValue);
  begin
  if assigned(FOnDblClick) then
    FOnDblClick(self);
  end;

procedure TrtcForm.Set_OnActivate(const Value: TNotifyEvent);
  begin
  FOnActivate := Value;
  if assigned(FOnActivate) then
    _Com.Event('onactivate',MyOnActivate)
  else
    _Com.Event('onactivate',nil);
  end;

procedure TrtcForm.MyOnActivate(xData: TRtcValue);
  begin
  if assigned(FOnActivate) then
    FOnActivate(self);
  end;

procedure TrtcForm.Set_OnResize(const Value: TNotifyEvent);
  begin
  FOnResize:=Value;
  if assigned(FOnResize) then
    _Com.Event('onresize',MyOnResize)
  else
    _Com.Event('onresize',nil);
  end;

procedure TrtcForm.MyOnResize(xData: TRtcValue);
  begin
  if assigned(FOnResize) then
    FOnResize(self);
  end;

procedure TrtcForm.Set_OnDeactivate(const Value: TNotifyEvent);
  begin
  FOnDeactivate := Value;
  if not assigned(FOnDeactivate) then
    _Com.Event('ondeactivate',nil)
  else
    _Com.Event('ondeactivate',MyOnDeactivate);
  end;

procedure TrtcForm.MyOnDeactivate(xData: TRtcValue);
  begin
  if assigned(FOnDeactivate) then
    FOnDeactivate(self);
  end;

procedure TrtcForm.Set_OnKeyDown(const Value: TRtcKeyEvent);
  begin
  FOnKeyDown := Value;
  if not assigned(FOnKeyDown) then
    _Com.Event('onkeydown',nil)
  else
    _Com.Event('onkeydown',MyOnKeyDown);
  end;

procedure TrtcForm.MyOnKeyDown(xData: TRtcValue);
  var
    key:word;
    shift:TShiftState;
  begin
  if assigned(FOnKeyDown) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        key:=asInteger['k'];
        shift:=RtcToShiftState(asRecord['s']);
        end;
      FOnKeyDown(self,key,shift);
      end;
    end;
  end;

procedure TrtcForm.Set_OnKeyUp(const Value: TRtcKeyEvent);
  begin
  FOnKeyUp := Value;
  if not assigned(FOnKeyUp) then
    _Com.Event('onkeyup',nil)
  else
    _Com.Event('onkeyup',MyOnKeyUp);
  end;

procedure TrtcForm.MyOnKeyUp(xData: TRtcValue);
  var
    key:word;
    shift:TShiftState;
  begin
  if assigned(FOnKeyUp) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        key:=asInteger['k'];
        shift:=RtcToShiftState(asRecord['s']);
        end;
      FOnKeyUp(self,key,shift);
      end;
    end;
  end;

procedure TrtcForm.Set_OnKeyPress(const Value: TRtcKeyPressEvent);
  begin
  FOnKeyPress := Value;
  if not assigned(FOnKeyPress) then
    _Com.Event('onkeypress',nil)
  else
    _Com.Event('onkeypress',MyOnKeyPress);
  end;

procedure TrtcForm.MyOnKeyPress(xData: TRtcValue);
  var
    key:char;
  begin
  if assigned(FOnKeyPress) then
    begin
    if length(xData.asText)>0 then
      key:=(xData.asText)[1]
    else
      key:=#0;
    FOnKeyPress(self,key);
    end;
  end;

procedure TrtcForm.Set_OnMouseDown(const Value: TRtcMouseEvent);
  begin
  FOnMouseDown := Value;
  if not assigned(FOnMouseDown) then
    _Com.Event('onmousedown',nil)
  else
    _Com.Event('onmousedown',MyOnMouseDown);
  end;

procedure TrtcForm.MyOnMouseDown(xData: TRtcValue);
  var
    button:TMouseButton;
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseDown) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        button:=TMouseButton(asInteger['b']);
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnMouseDown(self,button,shift,x,y);
      end;
    end;
  end;

procedure TrtcForm.Set_OnMouseUp(const Value: TRtcMouseEvent);
  begin
  FOnMouseUp := Value;
  if not assigned(FOnMouseUp) then
    _Com.Event('onmouseup',nil)
  else
    _Com.Event('onmouseup',MyOnMouseUp);
  end;

procedure TrtcForm.MyOnMouseUp(xData: TRtcValue);
  var
    button:TMouseButton;
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseUp) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        button:=TMouseButton(asInteger['b']);
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnMouseUp(self,button,shift,x,y);
      end;
    end;
  end;

procedure TrtcForm.Set_OnMouseMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseMove := Value;
  if not assigned(FOnMouseMove) then
    _Com.Event('onmousemove',nil)
  else
    _Com.Event('onmousemove',MyOnMouseMove);
  end;

procedure TrtcForm.MyOnMouseMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseMove) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnMouseMove(self,shift,x,y);
      end;
    end;
  end;

procedure TrtcForm.Set_OnMouseDownMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseDownMove := Value;
  if not assigned(FOnMouseDownMove) then
    _Com.Event('onmousedownmove',nil)
  else
    _Com.Event('onmousedownmove',MyOnMouseDownMove);
  end;

procedure TrtcForm.MyOnMouseDownMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseDownMove) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnMouseDownMove(self,shift,x,y);
      end;
    end;
  end;

procedure TrtcForm.Set_OnMouseUpMove(const Value: TRtcMouseMoveEvent);
  begin
  FOnMouseUpMove := Value;
  if not assigned(FOnMouseUpMove) then
    _Com.Event('onmouseupmove',nil)
  else
    _Com.Event('onmouseupmove',MyOnMouseUpMove);
  end;

procedure TrtcForm.MyOnMouseUpMove(xData: TRtcValue);
  var
    shift:TShiftState;
    x,y:integer;
  begin
  if assigned(FOnMouseUpMove) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnMouseUpMove(self,shift,x,y);
      end;
    end;
  end;

procedure TrtcForm.Set_OnMouseWheel(const Value: TRtcMouseWheelEvent);
  begin
  FOnMouseWheel := Value;
  if not assigned(FOnMouseWheel) then
    _Com.Event('onmousewheel',nil)
  else
    _Com.Event('onmousewheel',MyOnMouseWheel);
  end;

procedure TrtcForm.MyOnMouseWheel(xData: TRtcValue);
  var
    shift:TShiftState;
    wdelta:integer;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheel) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        wdelta:=asInteger['w'];
        end;
      handled:=False;
      FOnMouseWheel(self,shift,wdelta,mpos,handled);
      end;
    end;
  end;

procedure TrtcForm.Set_OnMouseWheelDown(const Value: TRtcMouseWheelUpDownEvent);
  begin
  FOnMouseWheelDown := Value;
  if not assigned(FOnMouseWheelDown) then
    _Com.Event('onmousewheeldown',nil)
  else
    _Com.Event('onmousewheeldown',MyOnMouseWheelDown);
  end;

procedure TrtcForm.MyOnMouseWheelDown(xData: TRtcValue);
  var
    shift:TShiftState;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheelDown) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        end;
      handled:=False;
      FOnMouseWheelDown(self,shift,mpos,handled);
      end;
    end;
  end;

procedure TrtcForm.Set_OnMouseWheelUp(const Value: TRtcMouseWheelUpDownEvent);
  begin
  FOnMouseWheelUp := Value;
  if not assigned(FOnMouseWheelUp) then
    _Com.Event('onmousewheelup',nil)
  else
    _Com.Event('onmousewheelup',MyOnMouseWheelUp);
  end;

procedure TrtcForm.MyOnMouseWheelUp(xData: TRtcValue);
  var
    shift:TShiftState;
    mpos:TPoint;
    handled:boolean;
  begin
  if assigned(FOnMouseWheelUp) then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        shift:=RtcToShiftState(asRecord['s']);
        mpos.x:=asInteger['x'];
        mpos.y:=asInteger['y'];
        end;
      handled:=False;
      FOnMouseWheelUp(self,shift,mpos,handled);
      end;
    end;
  end;

function TrtcForm.Get_Action: IRtcBasicAction;
  begin
  Result:=FAction;
  end;

function TrtcForm.Get_ActiveControl: TRtcWinControl;
  begin
  Result:=FActiveControl;
  end;

function TrtcForm.Get_Menu: IRtcMainMenu;
  begin
  Result:=FMenu;
  end;

function TrtcForm.Get_PopupMenu: IRtcPopupMenu;
  begin
  Result:=FPopupMenu;
  end;

procedure TrtcForm.Set_Action(const Value: IRtcBasicAction);
  begin
  if Value<>FAction then
    begin
    FAction:=Value;
    _Com.Link('action',Value);
    end;
  end;

procedure TrtcForm.Set_ActiveControl(const Value: TRtcWinControl);
  begin
  if Value<>FActiveControl then
    begin
    FActiveControl:=Value;
    _Com.Link('activecontrol',Value);
    end;
  end;

procedure TrtcForm.Set_Menu(const Value: IRtcMainMenu);
  {var
    h:integer;}
  begin
  if Value<>FMenu then
    begin
    FMenu:=Value;
    _Com.Link('menu',Value);
    {if (csDesigning in ComponentState) and assigned(Parent) then
      begin
      h:=ClientHeight;
      if assigned(Value) then
        begin
        if Parent.HandleAllocated then
          begin
          if Windows.GetMenu(Parent.Handle) <> Value.Handle then
            Windows.SetMenu(Parent.Handle, Value.Handle);
          Value.WindowHandle := Parent.Handle;
          end;
        end
      else
        begin
        if Parent.HandleAllocated then
          Windows.SetMenu(Parent.Handle, 0);
        end;
      ClientHeight:=h;
      end;}
    end;
  end;

procedure TrtcForm.Set_PopupMenu(const Value: IRtcPopupMenu);
  begin
  if Value<>FPopupMenu then
    begin
    FPopupMenu:=Value;
    _Com.Link('popupmenu',Value);
    end;
  end;

procedure TrtcForm.Notification(AComponent: TComponent; Operation: TOperation);
  begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
    if assigned(PopupMenu) and (AComponent = PopupMenu._Self) then
      PopupMenu := nil
    else if assigned(Action) and (AComponent = Action._Self) then
      Action := nil
    else if AComponent = ActiveControl then
      ActiveControl := nil
    else if assigned(Menu) and (AComponent = Menu._Self) then
      Menu := nil;
  end;

procedure TrtcForm.Set_OnDragDrop(const Value: TRtcDragDropEvent);
  begin
  FOnDragDrop := Value;
  if assigned(FOnDragDrop) then
    _Com.Event('ondragdrop',MyOnDragDrop)
  else
    _Com.Event('ondragdrop',nil);
  end;

procedure TrtcForm.MyOnDragDrop(xData: TRtcValue);
  var
    src:TObject;
    X,Y:integer;
  begin
  if assigned(FOnDragDrop) and Visible then
    begin
    if xData.isType=rtc_Record then
      begin
      with xData.asRecord do
        begin
        src:=_Com.FindByUID(asRecord['o']);
        x:=asInteger['x'];
        y:=asInteger['y'];
        end;
      FOnDragDrop(self,src,X,Y);
      end;
    end;
  end;

procedure TrtcForm.MyOnDropFiles(xData: TRtcValue);
  var
    a:integer;
    arr:TRtcArray;
    myarr:array of string;
  begin
  if assigned(FOnDropFiles) and (xData.isType=rtc_Array) then
    begin
    arr:=xData.asArray;
    try
      SetLength(myarr,arr.Count);
      for a:=0 to arr.Count-1 do
        myarr[a]:=arr.asText[a];
      FOnDropFiles(self,myarr);
    finally
      SetLength(myarr,0);
      end;
    end;
  end;

procedure TrtcForm.Set_OnDropFiles(const Value: TRtcDropFilesEvent);
  begin
  FOnDropFiles := Value;
  if assigned(FOnDropFiles) then
    _Com.Event('ondropfiles',MyOnDropFiles)
  else
    _Com.Event('ondropfiles',nil);
  end;

end.
