{
  RealThinClient W/A Framework: Server-side Graphic Types and Classes
  Copyright (c) Danijel Tkalcec

  @exclude
}
unit rtcGraphics;

interface

{$INCLUDE rtcDefs.inc}

uses
  rtcTrashcan, rtcLog, SysUtils,
  Windows, Graphics, Classes, Messages, Types,
  {$IFNDEF NoForms} Controls {$ELSE} _Controls {$ENDIF},
  rtcInfo, rtcSyncObjs,

  rtcLink;

type
  TColor = Graphics.TColor;
  TFont = Graphics.TFont;
  TPoint = Windows.TPoint;
  TFontPitch = Graphics.TFontPitch;
  TBitmap = Graphics.TBitmap;

const
  clSystemColor = $FF000000;

  clScrollBar = TColor(clSystemColor or COLOR_SCROLLBAR);
  clBackground = TColor(clSystemColor or COLOR_BACKGROUND);
  clActiveCaption = TColor(clSystemColor or COLOR_ACTIVECAPTION);
  clInactiveCaption = TColor(clSystemColor or COLOR_INACTIVECAPTION);
  clMenu = TColor(clSystemColor or COLOR_MENU);
  clWindow = TColor(clSystemColor or COLOR_WINDOW);
  clWindowFrame = TColor(clSystemColor or COLOR_WINDOWFRAME);
  clMenuText = TColor(clSystemColor or COLOR_MENUTEXT);
  clWindowText = TColor(clSystemColor or COLOR_WINDOWTEXT);
  clCaptionText = TColor(clSystemColor or COLOR_CAPTIONTEXT);
  clActiveBorder = TColor(clSystemColor or COLOR_ACTIVEBORDER);
  clInactiveBorder = TColor(clSystemColor or COLOR_INACTIVEBORDER);
  clAppWorkSpace = TColor(clSystemColor or COLOR_APPWORKSPACE);
  clHighlight = TColor(clSystemColor or COLOR_HIGHLIGHT);
  clHighlightText = TColor(clSystemColor or COLOR_HIGHLIGHTTEXT);
  clBtnFace = TColor(clSystemColor or COLOR_BTNFACE);
  clBtnShadow = TColor(clSystemColor or COLOR_BTNSHADOW);
  clGrayText = TColor(clSystemColor or COLOR_GRAYTEXT);
  clBtnText = TColor(clSystemColor or COLOR_BTNTEXT);
  clInactiveCaptionText = TColor(clSystemColor or COLOR_INACTIVECAPTIONTEXT);
  clBtnHighlight = TColor(clSystemColor or COLOR_BTNHIGHLIGHT);
  cl3DDkShadow = TColor(clSystemColor or COLOR_3DDKSHADOW);
  cl3DLight = TColor(clSystemColor or COLOR_3DLIGHT);
  clInfoText = TColor(clSystemColor or COLOR_INFOTEXT);
  clInfoBk = TColor(clSystemColor or COLOR_INFOBK);
  clHotLight = TColor(clSystemColor or COLOR_HOTLIGHT);
  clGradientActiveCaption = TColor(clSystemColor or COLOR_GRADIENTACTIVECAPTION);
  clGradientInactiveCaption = TColor(clSystemColor or COLOR_GRADIENTINACTIVECAPTION);
{$IFDEF IDE_7up}
  clMenuHighlight = TColor(clSystemColor or COLOR_MENUHILIGHT);
  clMenuBar = TColor(clSystemColor or COLOR_MENUBAR);
{$ENDIF}

  clBlack = TColor($000000);
  clMaroon = TColor($000080);
  clGreen = TColor($008000);
  clOlive = TColor($008080);
  clNavy = TColor($800000);
  clPurple = TColor($800080);
  clTeal = TColor($808000);
  clGray = TColor($808080);
  clSilver = TColor($C0C0C0);
  clRed = TColor($0000FF);
  clLime = TColor($00FF00);
  clYellow = TColor($00FFFF);
  clBlue = TColor($FF0000);
  clFuchsia = TColor($FF00FF);
  clAqua = TColor($FFFF00);
  clLtGray = TColor($C0C0C0);
  clDkGray = TColor($808080);
  clWhite = TColor($FFFFFF);
  StandardColorsCount = 16;

  clMoneyGreen = TColor($C0DCC0);
  clSkyBlue = TColor($F0CAA6);
  clCream = TColor($F0FBFF);
  clMedGray = TColor($A4A0A0);
  ExtendedColorsCount = 4;

  clNone = TColor($1FFFFFFF);
  clDefault = TColor($20000000);

const
  RtcDefaultPixelsPerInch:integer=96;

type
  TRtcFont=class(TPersistent)
  private
    {FCharset: TFontCharset;
    FColor: TColor;
    FHeight: Integer;
    FName: TFontName;
    FPitch: TFontPitch;
    FStyle: TFontStyles;}

    FCom:TRtcCom;
    FFullName: string;
    FOnChange: TNotifyEvent;
    FUpdating: integer;
    FParent: TFont;
    FAuto: boolean;

    procedure Set_Charset(const Value: TFontCharset);
    procedure Set_Color(const Value: TColor);
    procedure Set_Height(const Value: Integer);
    procedure Set_Name(const Value: TFontName);
    procedure Set_Pitch(const Value: TFontPitch);
    procedure Set_Size(const Value: Integer);
    procedure Set_Style(const Value: TFontStyles);

    function Get_Charset: TFontCharset;
    function Get_Color: TColor;
    function Get_Height: Integer;
    function Get_Name: TFontName;
    function Get_Pitch: TFontPitch;
    function Get_Size: Integer;
    function Get_Style: TFontStyles;

  public
    constructor Create(AParent:TFont);
    destructor Destroy; override;

    procedure Reset;

    procedure Assign(Source: TPersistent); override;

    property FullName:string read FFullName write FFullName;
    property _Com:TRtcCom read FCom write FCom;

    property OnChange: TNotifyEvent read FOnChange write FOnChange;

    procedure BeginUpdate;
    procedure EndUpdate;

  published
    property Charset: TFontCharset read Get_Charset write Set_Charset default DEFAULT_CHARSET;
    property Color: TColor read Get_Color write Set_Color default clWindowText;
    property Height: Integer read Get_Height write Set_Height;
    property Name: TFontName read Get_Name write Set_Name;
    property Pitch: TFontPitch read Get_Pitch write Set_Pitch default fpDefault;
    property Size: Integer read Get_Size write Set_Size stored False;
    property Style: TFontStyles read Get_Style write Set_Style default [];
    end;

  TRtcPen = class(TPersistent)
  private
    FCom:TRtcCom;
    FFullName: string;

    FColor: TColor;
    FMode: TPenMode;
    FStyle: TPenStyle;
    FWidth: Integer;
    FOnChange: TNotifyEvent;
    function Get_Mode: TPenMode;

  protected
    function Get_Color: TColor;
    function Get_Width: Integer;
    function Get_Style: TPenStyle;

    procedure Set_Color(Value: TColor);
    procedure Set_Mode(Value: TPenMode);
    procedure Set_Style(Value: TPenStyle);
    procedure Set_Width(Value: Integer);
  public
    constructor Create;
    destructor Destroy; override;

    procedure Reset;

    property FullName:string read FFullName write FFullName;
    property _Com:TRtcCom read FCom write FCom;

    procedure Assign(Source: TPersistent); override;

    property OnChange: TNotifyEvent read FOnChange write FOnChange;

  published
    property Color: TColor read Get_Color write Set_Color default clBlack;
    property Mode: TPenMode read Get_Mode write Set_Mode default pmCopy;
    property Style: TPenStyle read Get_Style write Set_Style default psSolid;
    property Width: Integer read Get_Width write Set_Width default 1;
  end;

  TRtcBrush = class(TPersistent)
  private
    FCom:TRtcCom;
    FFullName: string;

    FColor: TColor;
    FStyle: TBrushStyle;
    FOnChange: TNotifyEvent;

  protected
    function Get_Color: TColor;
    function Get_Style: TBrushStyle;

    procedure Set_Bitmap(Value: TBitmap);
    procedure Set_Color(Value: TColor);
    procedure Set_Style(Value: TBrushStyle);

  public
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;

    procedure Reset;

    property FullName:string read FFullName write FFullName;
    property _Com:TRtcCom read FCom write FCom;

    property Bitmap: TBitmap write Set_Bitmap;

    property OnChange: TNotifyEvent read FOnChange write FOnChange;

  published
    property Color: TColor read Get_Color write Set_Color default clWhite;
    property Style: TBrushStyle read Get_Style write Set_Style default bsSolid;
  end;

  TRtcCanvas = class(TPersistent)
  private
    FCom:TRtcCom;
    FFullName: string;

    FFont: TRtcFont;
    FPen: TRtcPen;
    FBrush: TRtcBrush;

    FCopyMode: TCopyMode;
    FTextFlags: Longint;

    FOnChange: TNotifyEvent;
    FOnChanging: TNotifyEvent;

    procedure SetPenPos(Value: TPoint);
    procedure SetPixel(X, Y: Integer; Value: TColor);

    procedure Set_Brush(Value: TRtcBrush);
    procedure Set_Font(Value: TRtcFont);
    procedure Set_Pen(Value: TRtcPen);
    procedure Set_CopyMode(const Value: TCopyMode);

    function Get_Brush: TRtcBrush;
    function Get_Font: TRtcFont;
    function Get_Pen: TRtcPen;
    function Get_CopyMode: TCopyMode;

    procedure Set_Com(const Value: TRtcCom);
    procedure Set_FullName(const Value: string);

    procedure Changed;
    procedure Changing;

    function GetTextFlags: Longint;
    procedure SetTextFlags(const Value: Longint);

  public
    constructor Create;
    destructor Destroy; override;

    property FullName:string read FFullName write Set_FullName;
    property _Com:TRtcCom read FCom write Set_Com;

    procedure Arc(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
    procedure BrushCopy(const Dest: TRect; Bitmap: TBitmap; const Source: TRect; Color: TColor);
    procedure Chord(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
    procedure CopyRect(const Dest: TRect; const Source: TRect);
    procedure Draw(X, Y: Integer; Graphic: TGraphic);
    procedure DrawFocusRect(const Rect: TRect);
    procedure Ellipse(X1, Y1, X2, Y2: Integer); overload;
    procedure Ellipse(const Rect: TRect); overload;
    procedure FillRect(const Rect: TRect);
    procedure FloodFill(X, Y: Integer; Color: TColor; FillStyle: TFillStyle);
    procedure FrameRect(const Rect: TRect);
    procedure LineTo(X, Y: Integer);
    procedure MoveTo(X, Y: Integer);
    procedure Pie(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
    procedure Polygon(const Points: array of TPoint);
    procedure Polyline(const Points: array of TPoint);
    procedure PolyBezier(const Points: array of TPoint);
    procedure PolyBezierTo(const Points: array of TPoint);
    procedure Rectangle(X1, Y1, X2, Y2: Integer); overload;
    procedure Rectangle(const Rect: TRect); overload;
    procedure RoundRect(X1, Y1, X2, Y2, X3, Y3: Integer);
    procedure StretchDraw(const Rect: TRect; Graphic: TGraphic);

    procedure TextOut(X, Y: Integer; const Text: string);
    procedure TextRect(Rect: TRect; X, Y: Integer; const Text: string);

    procedure Refresh;

    property PenPos: TPoint write SetPenPos;
    property Pixels[X, Y: Integer]: TColor write SetPixel;
    property TextFlags: Longint read GetTextFlags write SetTextFlags;

    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnChanging: TNotifyEvent read FOnChanging write FOnChanging;

  published
    property Brush: TRtcBrush read Get_Brush write Set_Brush;
    property CopyMode: TCopyMode read Get_CopyMode write Set_CopyMode default cmSrcCopy;
    property Font: TRtcFont read Get_Font write Set_Font;
    property Pen: TRtcPen read Get_Pen write Set_Pen;
  end;

implementation

{ TRtcFont }

constructor TRtcFont.Create(AParent:TFont);
  begin
  inherited Create;
  if not assigned(AParent) then
    begin
    FParent:=TFont.Create;
    FAuto:=True;
    end
  else
    begin
    FParent:=AParent;
    FAuto:=False;
    end;
  FUpdating:=0;
  end;

destructor TRtcFont.Destroy;
  begin
  if FAuto then FParent.Free;
  inherited;
  end;

procedure TRtcFont.Reset;
  begin
  FParent.Charset:=DEFAULT_CHARSET;
  FParent.Color:=clWindowText;
  FParent.Size:=8;
  FParent.Name:='Arial';
  FParent.Pitch:=fpDefault;
  FParent.Style:=[];
  if assigned(FOnChange) then FOnChange(self);
  end;

procedure TRtcFont.Assign(Source: TPersistent);
  var
    src:TRtcFont;
  begin
  if Source is TRtcFont then
    begin
    src:=TRtcFont(Source);
    Charset:=src.Charset;
    Name:=src.Name;
    Color:=src.Color;
    Pitch:=src.Pitch;
    Style:=src.Style;
    if src.Height>0 then
      Height:=src.Height
    else
      Size:=src.Size;
    if assigned(FOnChange) and (FUpdating=0) then
      FOnChange(self);
    end;
  end;

function TRtcFont.Get_Charset: TFontCharset;
  begin
  Result:=FParent.Charset;
  end;

function TRtcFont.Get_Color: TColor;
  begin
  Result:=FParent.Color;
  end;

function TRtcFont.Get_Height: Integer;
  begin
  Result:=FParent.Height;
  end;

function TRtcFont.Get_Name: TFontName;
  begin
  Result:=FParent.Name;
  end;

function TRtcFont.Get_Pitch: TFontPitch;
  begin
  Result:=FParent.Pitch;
  end;

function TRtcFont.Get_Size: Integer;
  begin
  Result:=FParent.Size;
  end;

function TRtcFont.Get_Style: TFontStyles;
  begin
  Result:=FParent.Style;
  end;

procedure TRtcFont.Set_Charset(const Value: TFontCharset);
  begin
  if Value<>FParent.Charset then
    begin
    FParent.Charset := Value;
    if FUpdating=0 then
      begin
      _Com.Prop(FullName+'charset',Ord(Value));
      if assigned(FOnChange) then
        FOnChange(self);
      end;
    end;
  end;

procedure TRtcFont.Set_Color(const Value: TColor);
  begin
  if Value<>FParent.Color then
    begin
    FParent.Color := Value;
    if FUpdating=0 then
      begin
      _Com.Prop(FullName+'color',Ord(Value));
      if assigned(FOnChange) then
        FOnChange(self);
      end;
    end;
  end;

procedure TRtcFont.Set_Height(const Value: Integer);
  begin
  if Value<>FParent.Height then
    begin
    FParent.Height := Value;
    if FUpdating=0 then
      begin
      _Com.Prop(FullName+'size',FParent.Size);
      if assigned(FOnChange) then
        FOnChange(self);
      end;
    end;
  end;

procedure TRtcFont.Set_Name(const Value: TFontName);
  begin
  if Value<>FParent.Name then
    begin
    FParent.Name := Value;
    if FUpdating=0 then
      begin
      _Com.Prop(FullName+'name',Value);
      if assigned(FOnChange) then
        FOnChange(self);
      end;
    end;
  end;

procedure TRtcFont.Set_Pitch(const Value: TFontPitch);
  begin
  if Value<>FParent.Pitch then
    begin
    FParent.Pitch := Value;
    if FUpdating=0 then
      begin
      _Com.Prop(FullName+'pitch',Ord(Value));
      if assigned(FOnChange) then
        FOnChange(self);
      end;
    end;
  end;

procedure TRtcFont.Set_Size(const Value: Integer);
  begin
  if Value<>FParent.Size then
    begin
    FParent.Size:=Value;
    if FUpdating=0 then
      begin
      _Com.Prop(FullName+'size',FParent.Size);
      if assigned(FOnChange) then
        FOnChange(self);
      end;
    end;
  end;

procedure TRtcFont.Set_Style(const Value: TFontStyles);
  var
    rec:TRtcRecord;
  begin
  if Value<>FParent.Style then
    begin
    FParent.Style := Value;

    if FUpdating=0 then
      begin
      rec:=TRtcRecord.Create;
      if fsBold in Value then rec.asBoolean['b']:=True;
      if fsItalic in Value then rec.asBoolean['i']:=True;
      if fsUnderline in Value then rec.asBoolean['u']:=True;
      if fsStrikeout in Value then rec.asBoolean['s']:=True;
      _Com.Prop(FullName+'style',rec);
      if assigned(FOnChange) then
        FOnChange(self);
      end;
    end;
  end;

procedure TRtcFont.BeginUpdate;
  begin
  Inc(FUpdating);
  end;

procedure TRtcFont.EndUpdate;
  begin
  Dec(FUpdating);
  end;

{ TRtcPen }

constructor TRtcPen.Create;
  begin
  inherited;
  FColor:=clBlack;
  FWidth:=1;
  FStyle:=psSolid;
  FMode:=pmCopy;
  end;

destructor TRtcPen.Destroy;
  begin
  inherited;
  end;

procedure TRtcPen.Reset;
  begin
  FColor:=clBlack;
  FWidth:=1;
  FStyle:=psSolid;
  FMode:=pmCopy;
  if assigned(FOnChange) then FOnChange(self);
  end;

procedure TRtcPen.Assign(Source: TPersistent);
  var
    src:TRtcPen;
  begin
  if Source is TRtcPen then
    begin
    src:=TRtcPen(Source);
    Color:=src.Color;
    Width:=src.Width;
    Style:=src.Style;
    Mode:=src.Mode;
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

function TRtcPen.Get_Color: TColor;
  begin
  Result:=FColor;
  end;

function TRtcPen.Get_Mode: TPenMode;
  begin
  Result:=FMode;
  end;

function TRtcPen.Get_Style: TPenStyle;
  begin
  Result:=FStyle;
  end;

function TRtcPen.Get_Width: Integer;
  begin
  Result:=FWidth;
  end;

procedure TRtcPen.Set_Color(Value: TColor);
  begin
  if Value<>FColor then
    begin
    FColor := Value;
    _Com.Prop(FullName+'color',Ord(Value));
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

procedure TRtcPen.Set_Mode(Value: TPenMode);
  begin
  if Value<>FMode then
    begin
    FMode := Value;
    _Com.Prop(FullName+'mode',Ord(Value));
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

procedure TRtcPen.Set_Style(Value: TPenStyle);
  begin
  if Value<>FStyle then
    begin
    FStyle := Value;
    _Com.Prop(FullName+'style',Ord(Value));
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

procedure TRtcPen.Set_Width(Value: Integer);
  begin
  if Value<>FWidth then
    begin
    FWidth := Value;
    _Com.Prop(FullName+'width',Value);
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

{ TRtcBrush }

constructor TRtcBrush.Create;
  begin
  inherited;
  FColor:=clWhite;
  FStyle:=bsSolid;
  end;

destructor TRtcBrush.Destroy;
  begin
  inherited;
  end;

procedure TRtcBrush.Reset;
  begin
  FColor:=clWhite;
  FStyle:=bsSolid;
  if assigned(FOnChange) then FOnChange(self);
  end;

procedure TRtcBrush.Assign(Source: TPersistent);
  var
    src:TRtcBrush;
  begin
  if Source is TRtcBrush then
    begin
    src:=TRtcBrush(Source);
    Color:=src.Color;
    Style:=src.Style;
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

function TRtcBrush.Get_Color: TColor;
  begin
  Result:=FColor;
  end;

function TRtcBrush.Get_Style: TBrushStyle;
  begin
  Result:=FStyle;
  end;

procedure TRtcBrush.Set_Bitmap(Value: TBitmap);
  var
    st:TRtcValue;
    bs:TRtcByteStream;
  begin
  if assigned(Value) then
    begin
    bs:=TRtcByteStream.Create;
    Value.SaveToStream(bs.GetByteStream);
    st:=TRtcValue.Create;
    st.asObject:=FCom.Cache(bs);
    _Com.Prop(FullName+'bitmap',st);
    if assigned(FOnChange) then FOnChange(self);
    end
  else
    begin
    _Com.Prop(FullName+'bitmap',nil);
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

procedure TRtcBrush.Set_Color(Value: TColor);
  begin
  if Value<>FColor then
    begin
    FColor := Value;
    _Com.Prop(FullName+'color',Ord(Value));
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

procedure TRtcBrush.Set_Style(Value: TBrushStyle);
  begin
  if Value<>FStyle then
    begin
    FStyle := Value;
    _Com.Prop(FullName+'style',Ord(Value));
    if assigned(FOnChange) then FOnChange(self);
    end;
  end;

{ TRtcCanvas }

constructor TRtcCanvas.Create;
  begin
  inherited;
  FFont:=TRtcFont.Create(nil);
  FPen:=TRtcPen.Create;
  FBrush:=TRtcBrush.Create;
  FCopyMode:=cmSrcCopy;
  FTextFlags:=0;
  end;

destructor TRtcCanvas.Destroy;
  begin
  FreeAndNil(FFont);
  FreeAndNil(FPen);
  FreeAndNil(FBrush);
  inherited;
  end;

procedure TRtcCanvas.Set_Com(const Value: TRtcCom);
  begin
  FCom := Value;
  FPen._Com:=Value;
  FBrush._Com:=Value;
  FFont._Com:=Value;
  end;

procedure TRtcCanvas.Set_FullName(const Value: string);
  begin
  FFullName := Value;
  FPen.FullName := Value+'pen.';
  FBrush.FullName := Value+'brush.';
  FFont.FullName := Value+'font.';
  end;

procedure TRtcCanvas.Refresh;
  begin
  Changing;
  FPen.Reset;
  FBrush.Reset;
  FFont.Reset;
  FCopyMode:=cmSrcCopy;
  FTextFlags:=0;
  _Com.Send(FullName+'refresh',nil);
  Changed;
  end;

procedure TRtcCanvas.Changed;
  begin
  if assigned(FOnChange) then FOnChange(self);
  end;

procedure TRtcCanvas.Changing;
  begin
  if assigned(FOnChanging) then FOnChanging(self);
  end;

function TRtcCanvas.Get_Brush: TRtcBrush;
  begin
  Result:=FBrush;
  end;

function TRtcCanvas.Get_CopyMode: TCopyMode;
  begin
  Result:=FCopyMode;
  end;

function TRtcCanvas.Get_Font: TRtcFont;
  begin
  Result:=FFont;
  end;

function TRtcCanvas.Get_Pen: TRtcPen;
  begin
  Result:=FPen;
  end;

procedure TRtcCanvas.Set_Brush(Value: TRtcBrush);
  begin
  FBrush.Assign(Value);
  end;

procedure TRtcCanvas.Set_Font(Value: TRtcFont);
  begin
  FFont.Assign(Value);
  end;

procedure TRtcCanvas.Set_Pen(Value: TRtcPen);
  begin
  FPen.Assign(Value);
  end;

procedure TRtcCanvas.Set_CopyMode(const Value: TCopyMode);
  begin
  if FCopyMode<>Value then
    begin
    FCopyMode:=Value;
    _Com.Prop(FullName+'copymode',Ord(Value));
    end;
  end;

function TRtcCanvas.GetTextFlags: Longint;
  begin
  Result:=FTextFlags;
  end;

procedure TRtcCanvas.SetTextFlags(const Value: Longint);
  begin
  if FTextFlags<>Value then
    begin
    FTextFlags:=Value;
    _Com.Prop(FullName+'tflags',Value);
    end;
  end;

procedure TRtcCanvas.Arc(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x1']:=x1;
  rec.asInteger['y1']:=y1;
  rec.asInteger['x2']:=x2;
  rec.asInteger['y2']:=y2;
  rec.asInteger['x3']:=x3;
  rec.asInteger['y3']:=y3;
  rec.asInteger['x4']:=x4;
  rec.asInteger['y4']:=y4;
  _Com.Send(FullName+'arc',rec);
  Changed;
  end;

procedure TRtcCanvas.Chord(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x1']:=x1;
  rec.asInteger['y1']:=y1;
  rec.asInteger['x2']:=x2;
  rec.asInteger['y2']:=y2;
  rec.asInteger['x3']:=x3;
  rec.asInteger['y3']:=y3;
  rec.asInteger['x4']:=x4;
  rec.asInteger['y4']:=y4;
  _Com.Send(FullName+'chord',rec);
  Changed;
  end;

procedure TRtcCanvas.BrushCopy(const Dest: TRect; Bitmap: TBitmap; const Source: TRect; Color: TColor);
  var
    rec:TRtcRecord;
    bs:TRtcByteStream;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  with rec.NewRecord('d') do
    begin
    asInteger['l']:=Dest.Left;
    asInteger['t']:=Dest.Top;
    asInteger['r']:=Dest.Right;
    asInteger['b']:=Dest.Bottom;
    end;
  if assigned(Bitmap) then
    begin
    bs:=TRtcByteStream.Create;
    Bitmap.SaveToStream(bs.GetByteStream);
    rec.asObject['bmp']:=FCom.Cache(bs);
    end;
  with rec.NewRecord('s') do
    begin
    asInteger['l']:=Source.Left;
    asInteger['t']:=Source.Top;
    asInteger['r']:=Source.Right;
    asInteger['b']:=Source.Bottom;
    end;
  rec.asInteger['c']:=Ord(Color);
  _Com.Send(FullName+'brushcopy',rec);
  Changed;
  end;

procedure TRtcCanvas.CopyRect(const Dest: TRect; const Source: TRect);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  with rec.NewRecord('d') do
    begin
    asInteger['l']:=Dest.Left;
    asInteger['t']:=Dest.Top;
    asInteger['r']:=Dest.Right;
    asInteger['b']:=Dest.Bottom;
    end;
  with rec.NewRecord('s') do
    begin
    asInteger['l']:=Source.Left;
    asInteger['t']:=Source.Top;
    asInteger['r']:=Source.Right;
    asInteger['b']:=Source.Bottom;
    end;
  _Com.Send(FullName+'copyrect',rec);
  Changed;
  end;

procedure TRtcCanvas.Draw(X, Y: Integer; Graphic: TGraphic);
  var
    rec:TRtcRecord;
    bs:TRtcByteStream;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x']:=X;
  rec.asInteger['y']:=Y;
  if assigned(Graphic) then
    begin
    if Graphic is TBitmap then
      rec.asText['cls']:='bmp'
    else if Graphic is TIcon then
      rec.asText['cls']:='ico'
    else if Graphic is TMetafile then
      rec.asText['cls']:='wmf'
    else
      rec.asText['cls']:=Graphic.ClassName;
    bs:=TRtcByteStream.Create;
    Graphic.SaveToStream(bs.GetByteStream);
    rec.asObject['img']:=FCom.Cache(bs);
    end;
  _Com.Send(FullName+'draw',rec);
  Changed;
  end;

procedure TRtcCanvas.StretchDraw(const Rect: TRect; Graphic: TGraphic);
  var
    rec:TRtcRecord;
    bs:TRtcByteStream;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=Rect.Left;
  rec.asInteger['t']:=Rect.Top;
  rec.asInteger['r']:=Rect.Right;
  rec.asInteger['b']:=Rect.Bottom;
  if assigned(Graphic) then
    begin
    if Graphic is TBitmap then
      rec.asText['cls']:='bmp'
    else if Graphic is TIcon then
      rec.asText['cls']:='ico'
    else if Graphic is TMetafile then
      rec.asText['cls']:='wmf'
    else
      rec.asText['cls']:=Graphic.ClassName;
    bs:=TRtcByteStream.Create;
    Graphic.SaveToStream(bs.GetByteStream);
    rec.asObject['img']:=FCom.Cache(bs);
    end;
  _Com.Send(FullName+'sdraw',rec);
  Changed;
  end;

procedure TRtcCanvas.DrawFocusRect(const Rect: TRect);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=Rect.Left;
  rec.asInteger['t']:=Rect.Top;
  rec.asInteger['r']:=Rect.Right;
  rec.asInteger['b']:=Rect.Bottom;
  _Com.Send(FullName+'focusrect',rec);
  Changed;
  end;

procedure TRtcCanvas.Ellipse(X1, Y1, X2, Y2: Integer);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=X1;
  rec.asInteger['t']:=Y1;
  rec.asInteger['r']:=X2;
  rec.asInteger['b']:=Y2;
  _Com.Send(FullName+'ellipse',rec);
  Changed;
  end;

procedure TRtcCanvas.Ellipse(const Rect: TRect);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=Rect.Left;
  rec.asInteger['t']:=Rect.Top;
  rec.asInteger['r']:=Rect.Right;
  rec.asInteger['b']:=Rect.Bottom;
  _Com.Send(FullName+'ellipse',rec);
  Changed;
  end;

procedure TRtcCanvas.FillRect(const Rect: TRect);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=Rect.Left;
  rec.asInteger['t']:=Rect.Top;
  rec.asInteger['r']:=Rect.Right;
  rec.asInteger['b']:=Rect.Bottom;
  _Com.Send(FullName+'fillrect',rec);
  Changed;
  end;

procedure TRtcCanvas.FloodFill(X, Y: Integer; Color: TColor; FillStyle: TFillStyle);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x']:=X;
  rec.asInteger['y']:=Y;
  rec.asInteger['c']:=Ord(Color);
  rec.asInteger['s']:=Ord(FillStyle);
  _Com.Send(FullName+'fill',rec);
  Changed;
  end;

procedure TRtcCanvas.FrameRect(const Rect: TRect);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=Rect.Left;
  rec.asInteger['t']:=Rect.Top;
  rec.asInteger['r']:=Rect.Right;
  rec.asInteger['b']:=Rect.Bottom;
  _Com.Send(FullName+'framerect',rec);
  Changed;
  end;

procedure TRtcCanvas.LineTo(X, Y: Integer);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x']:=X;
  rec.asInteger['y']:=Y;
  _Com.Send(FullName+'line',rec);
  Changed;
  end;

procedure TRtcCanvas.MoveTo(X, Y: Integer);
  var
    rec:TRtcRecord;
  begin
  rec:=TRtcRecord.Create;
  rec.asInteger['x']:=X;
  rec.asInteger['y']:=Y;
  // Using "Prop" instead of "Send" here to avoid duplocates
  // in case move is called multiple times in a row
  _Com.Prop(FullName+'move',rec);
  end;

procedure TRtcCanvas.SetPenPos(Value: TPoint);
  begin
  MoveTo(Value.X,Value.Y);
  end;

procedure TRtcCanvas.Pie(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x1']:=x1;
  rec.asInteger['y1']:=y1;
  rec.asInteger['x2']:=x2;
  rec.asInteger['y2']:=y2;
  rec.asInteger['x3']:=x3;
  rec.asInteger['y3']:=y3;
  rec.asInteger['x4']:=x4;
  rec.asInteger['y4']:=y4;
  _Com.Send(FullName+'pie',rec);
  Changed;
  end;

procedure TRtcCanvas.Rectangle(const Rect: TRect);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=Rect.Left;
  rec.asInteger['t']:=Rect.Top;
  rec.asInteger['r']:=Rect.Right;
  rec.asInteger['b']:=Rect.Bottom;
  _Com.Send(FullName+'rect',rec);
  Changed;
  end;

procedure TRtcCanvas.Rectangle(X1, Y1, X2, Y2: Integer);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=X1;
  rec.asInteger['t']:=Y1;
  rec.asInteger['r']:=X2;
  rec.asInteger['b']:=Y2;
  _Com.Send(FullName+'rect',rec);
  Changed;
  end;

procedure TRtcCanvas.RoundRect(X1, Y1, X2, Y2, X3, Y3: Integer);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x1']:=X1;
  rec.asInteger['y1']:=Y1;
  rec.asInteger['x2']:=X2;
  rec.asInteger['y2']:=Y2;
  rec.asInteger['x3']:=X3;
  rec.asInteger['y3']:=Y3;
  _Com.Send(FullName+'roundrect',rec);
  Changed;
  end;

procedure TRtcCanvas.SetPixel(X, Y: Integer; Value: TColor);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x']:=X;
  rec.asInteger['y']:=Y;
  rec.asInteger['c']:=Ord(Value);
  _Com.Send(FullName+'pixel',rec);
  Changed;
  end;

procedure TRtcCanvas.TextOut(X, Y: Integer; const Text: string);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['x']:=X;
  rec.asInteger['y']:=Y;
  rec.asText['s']:=Text;
  _Com.Send(FullName+'text',rec);
  Changed;
  end;

procedure TRtcCanvas.TextRect(Rect: TRect; X, Y: Integer; const Text: string);
  var
    rec:TRtcRecord;
  begin
  Changing;
  rec:=TRtcRecord.Create;
  rec.asInteger['l']:=Rect.Left;
  rec.asInteger['t']:=Rect.Top;
  rec.asInteger['r']:=Rect.Right;
  rec.asInteger['b']:=Rect.Bottom;
  rec.asInteger['x']:=X;
  rec.asInteger['y']:=Y;
  rec.asText['s']:=Text;
  _Com.Send(FullName+'textrect',rec);
  Changed;
  end;

procedure TRtcCanvas.PolyBezier(const Points: array of TPoint);
  var
    rec:TRtcDataSet;
    a:integer;
  begin
  Changing;
  rec:=TRtcDataSet.Create;
  for a:=0 to length(Points)-1 do
    begin
    rec.Append;
    rec.asInteger['x']:=Points[a].X;
    rec.asInteger['y']:=Points[a].Y;
    end;
  _Com.Send(FullName+'polyb',rec);
  Changed;
  end;

procedure TRtcCanvas.PolyBezierTo(const Points: array of TPoint);
  var
    rec:TRtcDataSet;
    a:integer;
  begin
  Changing;
  rec:=TRtcDataSet.Create;
  for a:=0 to length(Points)-1 do
    begin
    rec.Append;
    rec.asInteger['x']:=Points[a].X;
    rec.asInteger['y']:=Points[a].Y;
    end;
  _Com.Send(FullName+'polybto',rec);
  Changed;
  end;

procedure TRtcCanvas.Polygon(const Points: array of TPoint);
  var
    rec:TRtcDataSet;
    a:integer;
  begin
  Changing;
  rec:=TRtcDataSet.Create;
  for a:=0 to length(Points)-1 do
    begin
    rec.Append;
    rec.asInteger['x']:=Points[a].X;
    rec.asInteger['y']:=Points[a].Y;
    end;
  _Com.Send(FullName+'poly',rec);
  Changed;
  end;

procedure TRtcCanvas.Polyline(const Points: array of TPoint);
  var
    rec:TRtcDataSet;
    a:integer;
  begin
  Changing;
  rec:=TRtcDataSet.Create;
  for a:=0 to length(Points)-1 do
    begin
    rec.Append;
    rec.asInteger['x']:=Points[a].X;
    rec.asInteger['y']:=Points[a].Y;
    end;
  _Com.Send(FullName+'polyl',rec);
  Changed;
  end;

end.
