{ 
  RealThinClient W/A Framework: Interfaces between Client and Server components
  Copyright (c) Danijel Tkalcec

  @exclude
}
unit rtcLink;

interface

{$include rtcDefs.inc}

uses
  Classes, Graphics,
  {$IFNDEF NoForms} Controls {$ELSE} _Controls {$ENDIF},
  rtcInfo;

type
  TRtcReceiveEvent=procedure(const xData:TRtcValueObject) of object;
  TRtcExecuteEvent=procedure(xData:TRtcValue) of object;

  TRtcCom=class; // forward

  { TRtcBaseComponent and TRtcBaseControl implement the _Com function.
    Components implementing the actual functionality need to implement
      the _Process and _ClassName functions }
  IrtcfComponent=interface
    ['{9A207B85-1EB7-4D3F-8148-FA11CE62DBE8}']
    procedure _Process(const xName:string; xData:TRtcValue); // Process property data
    function _Com:TRtcCom; // "TRtcCom" object: created in constructor, destroyed in destructor
    function _ClassName:string; // name of the client-side class without the "TRtc" prefix and "C" sufix ('Label' for 'TRtcLabelC', 'Form' for 'TRtcFormC')
    function _Self:TComponent;
    end;

  (* Server-side communications inteerface *)
  TRtcCom=class
  public
    // Used by "TWinControl" descendants to retreive the correct tab order at design-time
    function TabOrder:TTabOrder; virtual; abstract;

    { Calls where order is important and every call is recorded }

    // Update components property or call components method by name
    procedure Send(const xName: string; xData: TRtcValueObject); overload; virtual; abstract;
    procedure Send(const xName: string; const xData:Variant); overload; virtual; abstract;
    // Link component to another component
    procedure Link(const xName: string; xComp: TComponent); overload; virtual; abstract;
    procedure Link(const xName: string; xComp: IRtcfComponent); overload; virtual;

    { Calls where order is not important and calls with same name will override old calls with that name }

    // Set event accessible remotely (xMethod = NIL to remove)
    procedure Event(const xName: string; xMethod:TRtcExecuteEvent); virtual; abstract;
    // Set property on remote component
    procedure Prop(const xName:string; xData:TRtcValueObject); overload; virtual; abstract;
    procedure Prop(const xName:string; const xData:Variant); overload; virtual; abstract;

    // Send object theough cache, return cache ID (assign the value where cached data has to be read from)
    function Cache(xData:TRtcValueObject):TRtcValueObject; overload; virtual; abstract;
    function Cache(const xData:Variant):TRtcValueObject; overload; virtual; abstract;

    // Call this to force sending all prepared data to the Client
    procedure UpdateNow; virtual; abstract;

    function GetUID:TRtcRecord; virtual; abstract;
    function FindByUID(UID:TRtcRecord):TComponent; virtual; abstract;
    end;

  (* Client-side communications inteerface *)
  TRtcComC=class
  public
    { Calls where order is important and every call is recorded }

    // Set remote components property or call remote components method by name
    procedure Send(const xName: string; xData: TRtcValueObject); overload; virtual; abstract;
    procedure Send(const xName: string; xData: Variant); overload; virtual; abstract;
    // Call remote components event
    procedure Event(const xID:integer; xData:TRtcValueObject); overload; virtual; abstract;
    procedure Event(const xID:integer; xData:Variant); overload; virtual; abstract;

    { Calls where order is not important and calls with same name will override old calls with that name }

    // Set remote components property
    procedure Prop(const xName: string; xData: TRtcValueObject); overload; virtual; abstract;
    procedure Prop(const xName: string; xData: Variant); overload; virtual; abstract;

    // call this to force sending all prepared data to the Server
    procedure UpdateNow; virtual; abstract;

    // Application is currently Idle? (we are currently not processing received data)
    function Idle:boolean; virtual; abstract;

    // Get this components Unique Identifier
    function GetUID:TRtcRecord; virtual; abstract;
    function FindByUID(UID:TRtcRecord):TComponent; virtual; abstract;
    end;

  IrtcfComponentX=interface
    ['{631CF209-A0A5-4C97-B340-DA6118788517}']
    function _Process(const xName:string; xData:TRtcValue):boolean; // Process property data
    function _Link(const xName:string; xData:TComponent):boolean; // Link component through property
    procedure _Paint(Can:TCanvas);
    function _Self:TComponent;

    procedure _BeginLoad;
    procedure _EndLoad;
    function _Loading:boolean;
    end;

  IrtcfComponentC=interface(IrtcfComponentX)
    ['{8422D3EC-6FEE-4F06-885D-72D95014CF75}']
    procedure _SetCom(Com:TRtcComC); // Set "TRtcComC" object (called after constructor, destroy in destructor)
    function _SetEvent(const xName:string; xID:longint):boolean; // Set event ID (0 = clear event)
    function _Com:TRtcComC; // Get "TRtcComC" object
    end;

  IrtcFormX=interface(IrtcfComponentX)
    ['{D4F00E07-4035-4ABB-9411-4043B8B93082}']
    function GetFormLeft:integer;
    function GetFormTop:integer;
    end;

implementation

{ TRtcCom }

procedure TRtcCom.Link(const xName: string; xComp: IRtcfComponent);
  var
    dummy:TComponent;
  begin
  if assigned(xComp) then
    Link(xName,xComp._Self)
  else
    begin
    dummy:=nil;
    Link(xName,dummy);
    end;
  end;

end.
