{
  @html(<b>)
  Encryption/Decryption Plug-in
  @html(</b>)
  - Copyright (c) Danijel Tkalcec
  @html(<br><br>)

  This unit defines a Plug-in class, which should be extended when implementing
  third-party plug-ins for SSL/SSH encryption/decryption using RTC SDK components.
}
unit rtcPlugins;

interface

{$include rtcDefs.inc}

uses
  Classes;

type
  TRtcConnID = longint;

  { @abstract(Cryptography Plugin)
    This is a basic class for any kind of third-party encryption/decryption
    plug-ins used by rtcHttpClient and rtcHttpServer components. }
  TRtcCryptPlugin = class(TComponent)
  public
    { Called after a new connection was established.
      OutData = data which has to be sent out immediately. }
    procedure AfterConnect(ID:TRtcConnID; var OutData:AnsiString); overload; virtual;
    procedure AfterConnect(var ConnCryptObj:TObject; ID:TRtcConnID; var OutData:AnsiString); overload; virtual;

    { Called before we do a graceful disconnect, in case some data has to be sent out. }
    procedure BeforeDisconnect(ID:TRtcConnID; var OutData:AnsiString); overload; virtual;
    procedure BeforeDisconnect(var ConnCryptObj:TObject; ID:TRtcConnID; var OutData:AnsiString); overload; virtual;

    { Called after a connection was closed. }
    procedure AfterDisconnect(ID:TRtcConnID); overload; virtual;
    procedure AfterDisconnect(var ConnCryptObj:TObject; ID:TRtcConnID); overload; virtual;

    { Called when data arrived.
      InData = data received from recipient (decode that!)
      OutData = data prepared by "decoder" for sending back to recipient (encoded data)
      OutPlainText = decrypted input data (for use by application) }
    procedure DataReceived(ID:TRtcConnID; const InData:AnsiString; var OutData:AnsiString; var OutPlainText:AnsiString); overload; virtual;
    procedure DataReceived(var ConnCryptObj:TObject; ID:TRtcConnID; const InData:AnsiString; var OutData:AnsiString; var OutPlainText:AnsiString); overload; virtual;

    { Called when data needs to be sent.
      InData = application data which we want to be encoded for sending
      OutData = encoded data which should be sent out }
    procedure DataToSend(ID:TRtcConnID; const InData:AnsiString; var OutData:AnsiString); overload; virtual;
    procedure DataToSend(var ConnCryptObj:TObject; ID:TRtcConnID; const InData:AnsiString; var OutData:AnsiString); overload; virtual;
    end;

implementation

{ TRtcCryptPlugin }

{ TRtcCryptPlugin }

procedure TRtcCryptPlugin.AfterConnect(var ConnCryptObj: TObject; ID: TRtcConnID; var OutData: AnsiString);
  begin
  // can be left as if is now if the other AfterConnect method is implemented.
  AfterConnect(ID,OutData);
  end;

procedure TRtcCryptPlugin.AfterConnect(ID: TRtcConnID; var OutData: AnsiString);
  begin
  // can be left empty if the other AfterConnect method is implemented.
  end;

procedure TRtcCryptPlugin.AfterDisconnect(var ConnCryptObj: TObject; ID: TRtcConnID);
  begin
  // can be left as it is now if the other AfterDisconnect method is implemented.
  AfterDisconnect(ID);
  end;

procedure TRtcCryptPlugin.AfterDisconnect(ID: TRtcConnID);
  begin
  // can be left empty if the other AfterDisconnect method is implemented.
  end;

procedure TRtcCryptPlugin.BeforeDisconnect(var ConnCryptObj: TObject; ID: TRtcConnID; var OutData: AnsiString);
  begin
  // can be left as it is now if the other BeforeDisconnect method is implemented.
  BeforeDisconnect(ID,OutData);
  end;

procedure TRtcCryptPlugin.BeforeDisconnect(ID: TRtcConnID; var OutData: AnsiString);
  begin
  // can be left empty if the other BeforeDisconnect method is implemented.
  end;

procedure TRtcCryptPlugin.DataReceived(var ConnCryptObj: TObject; ID: TRtcConnID; const InData: AnsiString; var OutData, OutPlainText: AnsiString);
  begin
  // can be left as it is now if the other DataReceived method is implemented.
  DataReceived(ID,InData,OutData,OutPlainText);
  end;

procedure TRtcCryptPlugin.DataReceived(ID: TRtcConnID; const InData: AnsiString; var OutData, OutPlainText: AnsiString);
  begin
  // can be left empty if the other DataReceived method is implemented.
  end;

procedure TRtcCryptPlugin.DataToSend(var ConnCryptObj: TObject; ID: TRtcConnID; const InData: AnsiString; var OutData: AnsiString);
  begin
  // can be left as it is now if the other DataToSend method is implemented.
  DataToSend(ID,InData,OutData);
  end;

procedure TRtcCryptPlugin.DataToSend(ID: TRtcConnID; const InData: AnsiString; var OutData: AnsiString);
  begin
  // can be left empty if the other DataToSend method is implemented.
  end;

end.
