{
  RealThinClient SDK - UNIX support
  Copyright (c) Danijel Tkalcec
  @exclude
}

unit rtcUnixSockUtils;

interface

uses
  BaseUnix, Sockets, netdb;
  
procedure SetSocketNonBlocking(sk: Integer);
function GetSocketError(sk: Integer): Integer;

function ResolveAddrStr(const AddrStr: AnsiString): Integer;
function ResolveSockAddr(const AddrStr: AnsiString; Port: Word; var addr: SockAddr): Boolean;

implementation

function ResolveSockAddr(const AddrStr: AnsiString; Port: Word; var addr: SockAddr): Boolean;
begin
  try
    FillChar(addr, sizeof(addr), 0);
    addr.sa_family := AF_INET;
    addr.sin_port := NToHs(Port);
    Integer(addr.sin_addr) := ResolveAddrStr(AddrStr);
    Result := true;
  except
    Result := false;
  end;
end;

function ResolveAddrStr(const AddrStr: AnsiString): Integer;
var
  h : THostEntry;
begin
  Result := Integer(StrToNetAddr(AddrStr));
  if (Result = 0) then
    if (netdb.ResolveHostByName(AddrStr, h)) then begin
      Result := Integer(h.Addr.s_addr);
    end;
end;

procedure SetSocketNonBlocking(sk: Integer);
begin
  FpFcntl(sk, F_SetFl, O_NONBLOCK);
end;

function GetSocketError(sk: Integer): Integer;
var
  err : Integer;
  len : Integer;
begin
  err := 0;
  len := SizeOf(err);
  Result := fpgetsockopt(sk, SOL_SOCKET, SO_ERROR, @err, @len );
  //writeln('fpgetsockopt = ', Result);
  Result := err;
end;


finalization
 // writeln('...rtcUnixSocketUtils finalized');

end.

