{
  RealThinClient SDK - UNIX support
  Copyright (c) Danijel Tkalcec
  @exclude
}

unit rtcUnixWinAPI;
// this is Windows wrapper module.
// it also uses wrapper classes at: rtcLinuxWinClasses
//
// the default windows implementation is console, given at: rtcLinuxWinConsole
//
// if any other implementation is required (LCL for example), than


interface

{$include rtcDefs.inc}

uses
{$IFDEF UNIX}
  cthreads,
{$ENDIF}
  BaseUnix, Unix,
  //UnixDebug, UnixUndebug,
  Classes, SysUtils, SyncObjs,
  rtcUnixWinCommon,
  rtcUnixWinClasses,
  // the unit is used as a base Windows wrapping implementation
  rtcUnixWinConsole
  // if LCL gui is used!
  {$ifdef LCLCarbon}, rtcUnixWinLCL{$endif}
  {$ifdef LCLGtk}, rtcUnixWinLCL{$endif}
  {$ifdef LCLGtk2}, rtcUnixWinLCL{$endif}
  {$ifdef LCLQt}, rtcUnixWinLCL{$endif}
  ;

procedure InitializeCriticalSection(var CS: TRTLCriticalSection);
procedure DeleteCriticalSection(CS: TRTLCriticalSection);
procedure EnterCriticalSection(CS: TRTLCriticalSection);
procedure LeaveCriticalSection(CS: TRTLCriticalSection);

const
  WS_EX_TOOLWINDOW = 0;
  WS_POPUP = 0;
  
// x, y, Width, Height, Style, Parent, Menu, Instance - ignored
function CreateWindowEx(ExStyle : LongWord;
  ClassName, WindowName: PAnsiChar;
  Style: LongWord; x, y, Width, Height: Integer;
  Parent: HWND; Menu: HMenu; Instance: THINSTANCE;
   Param : Pointer): HWND;
   
function UnregisterClass(lpClassName: PAnsiChar; hInstance: LongWord): Boolean;
function RegisterClass(const lpWndClass: TWndClass): ATOM;
function GetClassInfo(hInstance: LongWord; lpClassName: PAnsiChar;
  var lpWndClass: TWndClass): LongBool;


// any other nIndex is ignored
function SetWindowLong(Wnd: HWND; nIndex, NewLong: Integer): LongWord;
function DestroyWindow(Wnd: HWND): Boolean;
function DefWindowProc(ahWnd: HWND; auMsg, awParam, alParam: Longword): LongWord; {stdcall}

// parameters MsgFilterMin, MsgFilter Max are ignored
function GetMessage(var msg: TMsg; wnd: HWND; MsgFilterMin, MsgFilterMax: Integer): Boolean;
function PeekMessage(var msg: TMsg; wnd: HWND; MsgFilterMin, MsgFilterMax, RemoveMsg: Integer): Boolean;

function PostMessage(wnd:HWND; msgID: Integer; WParam,LParam: Integer): Boolean;

// rtcThrPool
function TranslateMessage(var msg: TMsg): Boolean;
function DispatchMessage(var msg : TMsg): LongWord;

function GetTickCount: LongWord;

function PostThreadMessage(ThreadID: integer; msgID, wParam, lParam: LongWord): Boolean;

function SetTimer(Wnd:HWND; nIDEvent: Integer; uElapse: LongWord;
  TimerFunc: TTimerProc): Integer;
  
function KillTimer(wnd: HWND; TimeID: Integer): Boolean;

// rtcDataSrv ActiveX/COM
function CoCreateGuid(Var G: TGUID): LongWord; // HResult


implementation

{}
var
  DbgLvl: Byte;

function CreateWindowEx(ExStyle : LongWord;
  ClassName, WindowName: PAnsiChar;
  Style: LongWord; x, y, Width, Height: Integer;
  Parent: HWND; Menu: HMenu; Instance: THINSTANCE;
   Param : Pointer): HWND;
var
  w   : TEmuWindow;
  cl  : TEmuWindowInfo;
begin
//  writeln('rtcUnixWindows.CreateWindowEx');
  Result := 0;
  cl := GetRegisteredClass(ClassName);
  if not Assigned(cl) then Exit;
  
  if GetCurrentThreadId = MainThreadID
    then w := CreateEmuWindow(cl, Integer(Param))  // the latest registered Window-wrapper implementation
    else w := CreateEmuConsoleWindow(cl, Integer(Param));

  w.userid := EmuWindowToHWND(w);
  w.Styles := Style;
  w.ExStyles := ExStyle;
  Result := EmuWindowToHWND(w);
end;

function DestroyWindow(Wnd: HWND): Boolean;
var
  ew : TEmuWindow;
begin
  ew := HWNDToEmuWindow(Wnd);
  Result := Assigned(ew);
  if Result then ew.Free;
end;

function DefWindowProc(ahWnd: HWND; auMsg, awParam, alParam: Longword): LongWord;
begin
  // handle WM_DESTROY? ;)
  Result := 0;
end;

function SetWindowLong(Wnd: HWND; nIndex, NewLong: Integer): LongWord;
var
  em : TEmuWindow;
begin
  Result := 0;
  em := HWNDToEmuWindow(Wnd);
  if not Assigned(em) then Exit;
  case nIndex of
    GWL_USERDATA: begin
      Result := em.LongValue;
      em.LongValue := NewLong;
    end;
  else
    Result := 0;
  end;
end;

procedure EnterCriticalSection(CS: TRTLCriticalSection);
var
  o : TObject;
begin
  o := TObject(CS);
  try
    if Assigned(o) and (o is TCriticalSection) then
      TCriticalSection(o).Enter;
  except
  end;
end;

procedure LeaveCriticalSection(CS: TRTLCriticalSection);
var
  o : TObject;
begin
  o := TObject(CS);
  try
    if Assigned(o) and (o is TCriticalSection) then
      TCriticalSection(o).Leave;
  except
  end;
end;

procedure DeleteCriticalSection(CS: Integer);
var
  o : TObject;
begin
  o := TObject(CS);
  try
    if Assigned(o) and (o is TCriticalSection) then
      o.Free;
  except
  end;
end;

procedure InitializeCriticalSection(var CS: Integer);
var
  c : TCriticalSection;
begin
  c := TCriticalSection.Create;
  CS := Integer(c)
end;

// posting can be done to any clas of TEmuWindow
function PostMessage(wnd:HWND; msgID: Integer; WParam,LParam: Integer): Boolean;
var
  emu : TEmuWindow;
begin
  emu := HWNDToEmuWindow(wnd);
  Result :=false;
  if not Assigned(emu) then begin
    //DebugLine('Emu window is not found, Cannot Post. wnd = ' + IntToStr(wnd));
    Exit;
  end;

  //DebugLine('Emu window found posting! msgID = %d, wPAram=%d,  lParam=%d', [msgID, wParam, lParam]);
  emu.PostMessage(msgID, WParam, lParam);
  Result := true;
end;


// you cannot get or peek message any non console window
// since only Console window provides access to message thread
// any further implementation should not change this.
function GetMessage(var msg: TMsgStruct; wnd: HWND; MsgFilterMin, MsgFilterMax: Integer): Boolean;
var
  em : TEmuWindow;
  cns : TConsoleEmuWindow;
begin
  if wnd <> 0 then begin
    em := HWNDToEmuWindow(wnd);
    Result := Assigned(em) and IsConsoleWindow(em, cns);
    if not Result then Exit;
  end else
    cns := nil;
  
  Result := EmuGetMessage(cns, msg);
end;

function PeekMessage(var msg: TMsg; wnd: HWND; MsgFilterMin, MsgFilterMax, RemoveMsg: Integer): Boolean;
var
  em : TEmuWindow;
  cns : TConsoleEmuWindow;
begin
  if wnd <> 0 then begin
    em := HWNDToEmuWindow(wnd);
    Result := Assigned(em) and IsConsoleWindow(em, cns);
    if not Result then Exit;
  end else
    cns := nil;

  Result := EmuPeekMessage(cns, msg, RemoveMsg = PM_REMOVE);
end;

function TranslateMessage(var msg: TMsg): Boolean;
begin
  // does nothing actually
  result := true;
end;

function DispatchMessage(var msg : TMsg): LongWord;
var
  em : TEmuWindow;
  cns : TConsoleEmuWindow;
begin
  Result := 0;
  em := HWNDToEmuWindow(msg.wnd);
  if not (Assigned(em) and IsConsoleWindow(em, cns)) then Exit;
  EmuDispatchMessage(msg, cns);
end;

// posting also required win api thread usage
function PostThreadMessage(ThreadID: integer; msgID, wParam, lParam: LongWord): Boolean;
begin
  Result:=EmuPostThreadMessage(ThreadID, msgID, wParam, lParam);
end;

function GetTickCount: LongWord;
var
  b: tms;
begin
  Result := fpTimes(b);
end;

function SetTimer(Wnd:HWND; nIDEvent: Integer; uElapse: LongWord;
  TimerFunc: TTimerProc): Integer;
var
  em : TEmuWindow;
begin
  Result := 0;
  if (wnd = 0) then Exit;
  em := HWNDToEmuWindow(wnd);
  if not assigned(em) then begin
    Exit;
  end;

  Result := em.CreateTimer(nIDEvent, uElapse, TimerFunc);
end;

function KillTimer(wnd: HWND; TimeID: Integer): Boolean;
var
  em : TEmuWindow;
begin
  Result := true;
  if (wnd = 0) then Exit;
  em := HWNDToEmuWindow(wnd);
  if not assigned(em) then Exit;

  Result := em.KillTimer(timeID);
end;


function UnregisterClass(lpClassName: PAnsiChar; hInstance: LongWord): Boolean;
begin
  Result := rtcUnixWinClasses.UnRegisterClass(lpClassName);
end;

function RegisterClass(const lpWndClass: TWndClass): ATOM;
var
  cl  : TEmuWindowInfo;
  nm  : AnsiString;
begin
  Result := 0;
  nm := lpWndClass.lpszClassName;
  cl := rtcUnixWinClasses.GetRegisteredClass(nm);
  if Assigned(cl) then Exit;
  cl := rtcUnixWinClasses.RegisterClass(nm);
  cl.Proc := lpWndClass.lpfnWndProc;
  if not Assigned(cl) then Exit;
  Result := cl.Atom;
end;

function GetClassInfo(hInstance: LongWord; lpClassName: PAnsiChar;
  var lpWndClass: TWndClass): LongBool;
begin
  Result := false;
end;

function CoCreateGuid(Var G: TGUID): LongWord;
begin
  SysUtils.CreateGUID(G);
  Result := 0;
end;

finalization
//  writeln('...rtcUnixWindows finalized');

end.
