{
  RealThinClient SDK - UNIX support
  Copyright (c) Danijel Tkalcec
  @exclude
}

unit rtcUnixWinClasses; 
// this is WinAPI Window emulation unit;
//

{$include rtcDefs.inc}

interface

uses
{$IFDEF UNIX}
  cthreads,
{$ENDIF}
  Classes, SysUtils, SyncObjs, rtcUnixWinCommon;
  
type

  { TEmuWindowClass }
  TEmuWindowInfo = class(TObject)
    WinClassName  : AnsiString;
    Proc          : TWindowProc;
    Atom          : Word; // huh?
    constructor Create;
  end;

  { TEmuWindow }

  TEmuWindow = class(TObject)
  protected
    fLock          : TCriticalSection;
    procedure Lock;
    procedure Unlock;

  public
    WinClassName  : AnsiString;
    WinName       : AnsiString;
    Parent        : TEmuWindow;
    Inst          : Integer;
    EmuWinProc    : TWindowProc;
    LongValue     : LongWord;
    Styles        : Integer;
    ExStyles      : Integer;

    UserID        : Integer;
    constructor Create(const Class_Name: AnsiString; Proc: TWindowProc; CreateParam: Integer);
    destructor Destroy; override;

    procedure PostMessage(msg: Integer; wParam, lParam: Integer); virtual; abstract;

    function CreateTimer(nIDEvent: Integer; uElapse: LongWord; TimerFunc: TTimerProc) : Integer; virtual; abstract;
    function KillTimer(TimeID: Integer): Boolean; virtual; abstract;
  end;
  

procedure InitWindowSystem;
  
function GetRegisteredClass(const WinClassName: AnsiString): TEmuWindowInfo;
function RegisterClass(const WinClassName: AnsiString): TEmuWindowInfo;
function UnRegisterClass(const WinClassName: AnsiString): Boolean;

function HWNDToEmuWindow(w: HWND): TEmuWindow;
function EmuWindowToHWND(em: TEmuWindow): HWND;

var
  CreateEmuWindow : function (AInfo: TEmuWindowInfo; CreateParam: Integer): TEmuWindow = nil;

implementation

var
  AtomID     : Word = 1;
  ClassLock  : TCriticalSection = nil;
  WinClasses : TStringList = nil;
  
  WinLock : TCriticalSection = nil;
  WinList : TList;
  

function EmuWindowToHWND(em: TEmuWindow): HWND;
begin
  Result := Integer(em);
end;

function HWNDToEmuWindow(w: HWND): TEmuWindow;
begin
  try
    if TObject(w) is TEmuWindow
      then Result := TEmuWindow(w)
      else Result := nil;
  except
    Result:=nil;
  end;
end;

procedure AddWindow(win: TEmuWindow);
begin
  WinLock.Enter;
  try
    WinList.Add(win);
  finally
    WinLock.Leave;
  end;
end;

procedure RemoveWindow(win: TEmuWindow);
var
  i   : integer;
begin
  WinLock.Enter;
  try
    i := WinList.IndexOf(win);
    if i < 0 then Exit;
    WinList.Delete(i);
    win.Free;
  finally
    WinLock.Leave;
  end;
end;

  
(*function CreateLCLWindow(AInfo: TEmuWindowInfo; CreateParam: Integer): TEmuWindow;
begin
{  Result := nil;
  if not Assigned(AInfo) then Exit;
  Result := TEmuWindow.Create(AClass.WinClassName, AClass.Proc, CreateParam);
  }
end;*)

function FindRegisteredClass(const WinClassName: AnsiString; var Index: Integer; var WinClass: TEmuWindowInfo): Boolean;
begin
  ClassLock.Enter;
  try
    //todo: use hash table!
    Index := WinClasses.IndexOf(WinClassName);
    Result := Index >= 0;
    if Result then WinClass := WinClasses.Objects[Index] as TEmuWindowInfo;
  finally
    ClassLock.Leave;
  end;
end;

function GetRegisteredClass(const WinClassName: AnsiString): TEmuWindowInfo;
var
  idx : Integer;
begin
  Result := nil;
  if not FindRegisteredClass(WinClassName, idx, Result) then
    Result := nil;
end;


function RegisterClass(const WinClassName: AnsiString): TEmuWindowInfo;
begin
  Result := nil;
  if not Assigned(ClassLock) or not Assigned(WinClasses) then Exit;
  ClassLock.Enter;
  try

    Result := GetRegisteredClass(WinClassName);
    if not Assigned(Result) then begin
      Result := TEmuWindowInfo.Create;
      Result.WinClassName := WinClassName;
      WinClasses.AddObject(WinClassName, Result);
    end;
  finally
    ClassLock.Leave;
  end;
end;

function UnRegisterClass(const WinClassName: AnsiString): Boolean;
var
  idx : Integer;
  cl  : TEmuWindowInfo;
begin
  ClassLock.Enter;
  try
    cl := nil;
    idx := -1;
    Result := FindRegisteredClass(WinClassName, idx, cl);
    if not Result then Exit;
    if idx >= 0 then WinClasses.Delete(idx);
    cl.Free;
  finally
    ClassLock.Leave;
  end;
end;

procedure ClearClasses;
var
  i : Integer;
begin
  ClassLock.Enter;
  try
    for i := 0 to WinClasses.Count - 1 do begin
      if Assigned(WinClasses.Objects[i]) then
        WinClasses.Objects[i].Free;
    end;
    WinClasses.Clear;
  finally
    ClassLock.Leave;
  end;
end;


procedure ClearWinList;
var
  i   : Integer;
begin
  for i := 0 to WinList.Count - 1 do begin
  end;
end;

procedure InitWindowSystem;
begin
  ClassLock := TCriticalSection.Create;
  WinClasses := TStringList.Create;
  WinLock := TCriticalSection.Create;
  WinList := TList.Create;
end;

procedure UninitWindowSystem;
begin
  ClearWinList;
  WinLock.Free;
  WinList.Free;

  ClearClasses;
  WinClasses.Free;
  ClassLock.Free;
  ClassLock := nil;
end;

{ TEmuWindow }

procedure TEmuWindow.Lock;
begin
  fLock.Enter;
end;

procedure TEmuWindow.Unlock;
begin
  fLock.Leave;
end;

constructor TEmuWindow.Create(const Class_Name: AnsiString; Proc: TWindowProc; CreateParam: Integer);
begin
  inherited Create;
  fLock := TCriticalSection.create;
  WinClassName := Class_Name;
  EmuWinProc := Proc;
  AddWindow(Self);
end;

destructor TEmuWindow.Destroy;
begin
  fLock.Free;
  inherited Destroy;
end;

{ TEmuWindowClass }

constructor TEmuWindowInfo.Create;
begin
  Atom := AtomID;
  inc(AtomID);
end;

initialization
  InitWindowSystem;

finalization
  UninitWindowSystem;
 // writeln('...rtcUnixWinSockets finalized');
end.

