{
  RealThinClient SDK - UNIX support
  Copyright (c) Danijel Tkalcec
  @exclude
}

unit rtcUnixWinConsole;
// the unim emulates windows messaging system for console/service/deamon application
// Since Unix OS-es does not provide thread-base messaging mechanism
// it has to be implemented for Windows Message queue emulation

// nixMsgQueue and nixWinMsgQueue is used to achive this goal

// todo: timer

interface

{$include rtcDefs.inc}

{.$define DebugWinConsole}

uses
{$IFDEF UNIX}
  cthreads,
{$ENDIF}
  Classes, SysUtils, rtcUnixWinCommon, rtcUnixWinClasses, rtcUnixWinMsgQueue, rtcUnixThreadTimer
  {$ifdef DebugWinConsole}
  ,UnixDebug
  {$endif}
  ;

type
  TConsoleEmuWindow = class;

  { TConsoleTimer }

  TConsoleTimer = class(TThreadedTimer)
  protected
    procedure DoTimer; override;
  public
    TimerProc : TTimerProc;
    TimerID   : LongWord;
    Owner     : TConsoleEmuWindow;
    constructor Create(AOwner: TConsoleEmuWindow; ATimerID: LongWord; AProc: TTimerProc);
  end;

  { TConsoleEmuWindow }

  TConsoleEmuWindow = class(TEmuWindow)
  protected
    Target  : integer;

    fTimers : TList;
    procedure WinMsgProc(hwnd: Integer; msg, wParam, lParam: LongWord);

    procedure ClearTimers;

    function GetTimerByID(AID: Integer): TConsoleTimer;
  public
    constructor Create(const AClassName: AnsiString; Proc: TWindowProc; CreateParam: Integer);
    destructor Destroy; override;

    procedure PostMessage(msg: Integer; wParam, lParam: Integer); override;

    function CreateTimer(nIDEvent: Integer; uElapse: LongWord; TimerFunc: TTimerProc) : Integer; override;
    function KillTimer(TimerID: Integer): Boolean; override;

  end;

function CreateEmuConsoleWindow(AInfo: TEmuWindowInfo; CreateParam: Integer): TEmuWindow;
function IsConsoleWindow(emu: TEmuWindow; var ConsoleWindow: TConsoleEmuWindow): Boolean;

function EmuGetMessage(win: TConsoleEmuWindow; var msg: TMsgStruct): Boolean;
function EmuPeekMessage(win: TConsoleEmuWindow; var msg: TMsgStruct; RemoveMessage: Boolean): Boolean;
function EmuDispatchMessage(var msg: TMsgStruct; TargetedWindow: TConsoleEmuWindow): LongWord;
function EmuPostThreadMessage(ThreadID: integer; msgID, wParam, lParam: LongWord): Boolean;

implementation

{$ifdef DebugWinConsole}
var
  DbgLVL: byte;
{$endif}

function IsConsoleWindow(emu: TEmuWindow; var ConsoleWindow: TConsoleEmuWindow): Boolean;
begin
  Result := Assigned(emu) and (emu is TConsoleEmuWindow);
  if Result then
    ConsoleWindow := TConsoleEmuWindow(emu);
end;

function CreateEmuConsoleWindow(AInfo: TEmuWindowInfo; CreateParam: Integer): TEmuWindow;
begin
  Result := nil;
  if not Assigned(AInfo) then Exit;
  Result := TConsoleEmuWindow.create(AInfo.WinClassName, AInfo.Proc, CreateParam);
end;

{ TConsoleEmuWindow }

procedure TConsoleEmuWindow.WinMsgProc(hwnd: Integer; msg, wParam, lParam: LongWord);
var
  tt : TConsoleTimer;
begin
  try
    if Assigned(EmuWinProc) then begin
      EmuWinProc(userid,msg,wParam, lParam);
    end;
    Lock;
    try
      tt := GetTimerByID(wParam);
      if Assigned(tt) then begin
        try
          if Assigned(tt.TimerProc) then
            tt.TimerProc(Self.UserID, WM_TIMER, wParam, tt.Interval);
        except
        end;
      end;
    finally
      Unlock;
    end;
  except
  end;
end;

procedure TConsoleEmuWindow.ClearTimers;
var
  i : integer;
  tt  : TConsoleTimer;
begin
  Lock;
  try
    for i := 0 to fTimers.Count - 1 do begin
      tt := TConsoleTimer(fTimers[i]);
      fTimers[i] := nil;
      tt.Enabled := false;
      tt.Free;
    end;
    fTimers.Clear;
  finally
    Unlock;
  end;
end;

function TConsoleEmuWindow.GetTimerByID(AID: Integer): TConsoleTimer;
var
  cns   : TConsoleTimer;
  i     : Integer;
begin
  Lock;
  try
    for i := 0 to fTimers.Count - 1 do begin
      cns := TConsoleTimer(fTimers[i]);
      if cns.TimerID = AID then begin
        Result := cns;
        Exit;
      end;
    end;
    Result := nil;
  finally
    Unlock;
  end;
end;

constructor TConsoleEmuWindow.Create(const AClassName: AnsiString; Proc: TWindowProc; CreateParam: Integer);
begin
  inherited Create(ClassName, Proc, CreateParam);
  Target := winmsgCreateTarget(WinMsgProc);
  winmsgSetTagObject(Target, Self);
  fTimers := TList.Create;
end;

destructor TConsoleEmuWindow.Destroy;
begin
  ClearTimers;
  fTimers.Free;
  if Target <> 0 then winmsgCloseTarget(Target);
  inherited Destroy;
end;

procedure TConsoleEmuWindow.PostMessage(msg: Integer; wParam, lParam: Integer);
begin
  {$ifdef DebugWinConsole} DebugLine('[ConsoleWin.PostMesage] lParam = %d, wParam = %d', [lParam, wParam], DbgLVL);{$endif}
  winmsgPostMessage(Target, msg, wParam, lParam);
end;

function TConsoleEmuWindow.CreateTimer(nIDEvent: Integer; uElapse: LongWord;
  TimerFunc: TTimerProc): Integer;
var
  tt  : TConsoleTimer;
begin
  Lock;
  try
    if nIDEvent <> 0 then begin
      tt := GetTimerByID(nIDEvent);
    end else begin
      nIDEvent := fTimers.Count;
      tt := nil;
      repeat
        inc(nIDEvent);
        tt := GetTimerByID(nIDEvent);
      until not Assigned(tt) or (nIDEvent <> 0);

      if nIDEvent = 0 then begin
        // error. out of resources!
        Result := 0;
        Exit;
      end;
    end;

    if not Assigned(tt) then begin
      tt := TConsoleTimer.Create(Self, nIDEvent, TimerFunc);
      fTimers.Add(tt);
    end else
      tt.TimerProc := TimerFunc;

    tt.Interval := uElapse;
    tt.Enabled := true;

    Result := tt.TimerID;
  finally
    Unlock;
  end;
end;

function TConsoleEmuWindow.KillTimer(TimerID: Integer): Boolean;
var
  tt  : TConsoleTimer;
  i   : integer;
begin
  Lock;
  try
    tt := GetTimerByID(TimerID);
    Result:=Assigned(tt);
    if not Result then Exit;

    tt.Enabled := false;
    i := fTimers.IndexOf(tt);
    if i>=0 then fTimers.Delete(i);
  finally
    Unlock;
  end;
end;

procedure UpdateMsgToEmuWindow(cns : TConsoleEmuWindow; var msg: TMsgStruct);
begin
  if not Assigned(cns) then Exit;
  msg.wnd := cns.UserID;
end;

function EmuGetMessage(win: TConsoleEmuWindow; var msg: TMsgStruct): Boolean;
begin
  if Assigned(win) then
    Result := winmsgGetMessage(msg, win.Target)
  else
    Result := winmsgGetMessage(msg, 0);

  {$ifdef DebugWinConsole} DebugLine('[EmuGetWindowMessage] lParam = %d, wParam = %d', [msg.lParam, msg.wParam], DbgLVL);{$endif}
  if Result and (msg.wnd <> 0) then
    UpdateMsgToEmuWindow( TConsoleEmuWindow(winmsgGetTagObject(msg.wnd)), msg);
end;

function EmuPeekMessage(win: TConsoleEmuWindow; var msg: TMsgStruct; RemoveMessage: Boolean ): Boolean;
begin
  if Assigned(win) then
    Result := winmsgPeekMessage(msg, win.Target, RemoveMessage)
  else
    Result := winmsgPeekMessage(msg, 0, RemoveMessage);
    
  if Result and (msg.wnd <> 0) then
    UpdateMsgToEmuWindow( TConsoleEmuWindow(winmsgGetTagObject(msg.wnd)), msg);
end;

function EmuDispatchMessage(var msg: TMsgStruct; TargetedWindow: TConsoleEmuWindow ): LongWord;
var
  m : TMsgStruct;
begin

  m := msg;
  if Assigned(TargetedWindow) then m.wnd := TargetedWindow.Target
  else m.wnd := 0;
  
  if winmsgDispatchMessage(m) then Result := 1
  else Result := 0;
end;

function EmuPostThreadMessage(ThreadID: integer; msgID, wParam, lParam: LongWord): Boolean;
begin
  {$ifdef DebugWinConsole} DebugLine('[EmuPostThreadMessage] lParam = %d, wParam = %d', [lParam, wParam], DbgLVL);{$endif}
  Result := winmsgPostThreadMessage(ThreadID, msgId, wParam, lParam);
end;

{ TConsoleTimer }

constructor TConsoleTimer.Create(AOwner: TConsoleEmuWindow; ATimerID: LongWord; AProc: TTimerProc);
begin
  Owner := AOwner;
  TimerID := ATimerID;
  TimerProc := AProc;
  inherited Create;
end;

procedure TConsoleTimer.DoTimer;
begin
  inherited DoTimer;
  Owner.PostMessage(WM_TIMER, TimerID, Integer(@TimerProc));
end;

initialization
{$ifdef DebugWinConsole}
  DbgLVL:=RegisterDebugLVL;
{$endif}

  if not Assigned(CreateEmuWindow) then
    CreateEmuWindow := CreateEmuConsoleWindow;



end.

