{
  RealThinClient SDK - UNIX support
  Copyright (c) Danijel Tkalcec
  @exclude
}

unit rtcUnixWinLCL;
// the unim emulates windows messaging system,
// based on current LCL Widgetset.
// The message queue and invisible windows are provided by current WidgetSet
// and operating system GUI (gtk, qt (untested), carbon);
//
// It is required for non-Windows gui applications only
// Windows API should not be emulated through the LCL

interface

{$include rtcDefs.inc}

{.$define WinLCLDebug}

uses
  Classes, SysUtils, Forms, rtcUnixWinCommon, rtcUnixWinClasses, LMessages, LclIntf, ExtCtrls
  {$ifdef WinLCLDebug}, rtcUnixDebug{$endif}
  ;


type
  TLCLEmuWindow = class;

  { TLCLWindow }

  TLCLWindow = class(TForm)
  protected
    fEmu  : TLCLEmuWindow;
    procedure WndProc(var Msg: TLMessage); override;
  public
    constructor CreateWithWindow(AOwner: TComponent; AEmu: TLCLEmuWindow);
  end;

  TWinTimer = class(TTimer)
  public
    Busy     : Boolean;
    CallProc : TTimerProc;
    EventID  : Integer;
  end;

  { TLCLEmuWindow }

  TLCLEmuWindow = class(TEmuWindow)
  protected
    LclWin  : TLCLWindow;
    Owned   : Boolean;

    // timer objects are freed with window destroyed
    Timers  : TList;

    procedure ProcMsgFromLcl(msg: Integer; wParam, lParam: Integer);

    procedure AllocTimer(var tm: TWinTimer; var timerID: Integer);
    procedure TimerProc(Sender: TObject);

    procedure ReleaseLCLForm;
  public
    constructor Create(const Class_Name: AnsiString; Proc: TWindowProc; CreateParam: Integer);
    destructor Destroy; override;

    procedure AssignForm(AForm: TForm);

    procedure PostMessage(msg: Integer; wParam, lParam: Integer); override;

    function CreateTimer(nIDEvent: Integer; uElapse: LongWord; TimerFunc: TTimerProc) : Integer; override;
    function KillTimer(TimeID: Integer): Boolean; override;

  end;

function CreateLCLEmuWindow(AClass: TEmuWindowInfo; CreateParam: Integer): TEmuWindow;
function CreateWindowAroundForm(Form: TForm; CreateParam: Integer): TEmuWindow;

implementation

{$ifdef WinLCLDebug}
var
  DbgLVL : Byte;
{$endif}

function CreateLCLEmuWindow(AClass: TEmuWindowInfo; CreateParam: Integer): TEmuWindow;
begin
  Result := nil;
  if not Assigned(AClass) then Exit;
  Result := TLCLEmuWindow.create(AClass.WinClassName, AClass.Proc, CreateParam);
end;

function CreateWindowAroundForm(Form: TForm; CreateParam: Integer): TEmuWindow;
begin
//  Result := TLCLEmuWindow.create(AClass.WinClassName, AClass.Proc, CreateParam);
end;


{ TLCLWindow }

procedure TLCLWindow.WndProc(var Msg: TLMessage);
begin
  try
    if Assigned(fEmu) then
      fEmu.ProcMsgFromLcl(Msg.Msg, Msg.WParam, Msg.LParam);
  except
  end;
  inherited WndProc(Msg);
end;

constructor TLCLWindow.CreateWithWindow(AOwner: TComponent; AEmu: TLCLEmuWindow);
begin
  fEmu := AEmu;
  inherited Create(AOwner);
end;

{ TLCLEmuWindow }

procedure TLCLEmuWindow.ProcMsgFromLcl(msg: Integer; wParam, lParam : Integer);
begin
  if Assigned(EmuWinProc) then begin
    try
      {$ifdef WinLCLDebug}  DebugLine('[LCLEmuWin] Process Msg; W = %s; L = %s', [IntTohex(wParam, 8), IntToHex(lParam, 8)], DbgLVL);{$endif}
      EmuWinProc(userid, msg, wParam, lParam);
    except
      {$ifdef WinLCLDebug}  DebugLine('[LCLEmuWin] TLCLEmuWindow.ProcMsgFromLcl exception', DbgLVL);{$endif}
    end;
  end;
end;

procedure TLCLEmuWindow.AllocTimer(var tm: TWinTimer; var timerID: Integer);
var
  i : Integer;
begin
  Lock;
  try
    for i := 0 to Timers.Count - 1 do begin
      tm := TWinTimer(Timers[i]);
      if not tm.Busy then begin
        tm.Busy := true;
        TimerID := i;
        Exit;
      end;
    end;
    tm := TWinTimer.Create(nil);
    timerID := timers.Add(tm);
    tm.OnTimer := TimerProc;
  finally
    Unlock;
  end;
end;

procedure TLCLEmuWindow.TimerProc(Sender: TObject);
var
  win : TWinTimer;
begin
  win := TWinTimer(Sender);
  if Assigned(win) and Assigned(win.CallProc) then
    win.CallProc(UserID, WM_TIMER, win.EventID, GetTickCount) ;
end;

procedure TLCLEmuWindow.ReleaseLCLForm;
begin
  if Owned then LclWin.Free;
end;

constructor TLCLEmuWindow.Create(const Class_Name: AnsiString;
  Proc: TWindowProc; CreateParam: Integer);
begin
  inherited Create(Class_Name, Proc, CreateParam);
  LclWin := TLCLWindow.CreateWithWindow(nil, Self);
  LclWin.HandleNeeded;
  Owned :=true;
  Timers := TList.Create;
end;

destructor TLCLEmuWindow.Destroy;
begin
  ReleaseLCLForm;
  Timers.Free;
  inherited Destroy;
end;

procedure TLCLEmuWindow.AssignForm(AForm: TForm);
begin
  ReleaseLCLForm;
//  LclWin := AForm;
  Owned := false;
end;

procedure TLCLEmuWindow.PostMessage(msg: Integer; wParam, lParam: Integer);
begin
  {$ifdef WinLCLDebug}DebugLine('[LCLEmuWin] Post Msg; W = %s; L = %s', [IntTohex(wParam, 8), IntToHex(lParam, 8)], DbgLVL);{$endif}
  LclIntf.PostMessage(LclWin.Handle, msg, wParam, lParam);
end;

function TLCLEmuWindow.CreateTimer(nIDEvent: Integer; uElapse: LongWord;
  TimerFunc: TTimerProc): Integer;
var
  tm  : TWinTimer;
begin
  try
    AllocTimer(tm, Result);
    inc(Result);
    tm.CallProc := TimerFunc;
    tm.EventID := nIDEvent;
    tm.Interval := uElapse;
    tm.Enabled := true;
  except
    Result := 0;
  end;
end;

function TLCLEmuWindow.KillTimer(TimeID: Integer): Boolean;
begin
  Result := false;
  Lock;
  try
    dec(TimeID);
    if (TimeID < 0) or (TimeID >= Timers.Count) then Exit;
    with TWinTimer(Timers[TimeID]) do begin
      CallProc := nil;
      Enabled := false;
      Busy := false;
    end;
    Result := true;
  finally
    Unlock;
  end;
end;


initialization
{$ifdef WinLCLDebug}
  DbgLVL := RegisterDebugLVL;
{$endif}
  rtcUnixWinClasses.CreateEmuWindow := CreateLCLEmuWindow;

finalization
//  writeln('...rtcUnixWinLCL finalized');
end.

