{
  RealThinClient SDK - UNIX support
  Copyright (c) Danijel Tkalcec
  @exclude
}

unit rtcUnixWinSockets;

// base Unix unit to provide interface the same as WinSockets provide.
// Though, WinSockets are almost closed to unix sockets there're
// a lot of small differences, which must be wrapped.

{$INCLUDE rtcDefs.inc}

interface

{.$define NixSockDebug}
uses
//todo: remove
  SysUtils,
  BaseUnix, Sockets, netdb, rtcUnixSockWrapper, rtcUnixSockUtils
  {$ifdef NixSockDebug}
  ,UnixDebug
  {$endif}
  ;


type
  TSocket = Integer;
  HWND    = LongWord;

const
  SOCKET_ERROR = -1;

  //this constants will be used in WSocket_rtc
(*  SOL_SOCKET    = Sockets.SOL_SOCKET;

  SO_KEEPALIVE = Sockets.SO_KEEPALIVE;
  SO_REUSEADDR  = Sockets.SO_REUSEADDR;
  SO_LINGER     = Sockets.SO_LINGER;
  SO_BROADCAST  = Sockets.SO_BROADCAST;
  SO_SNDTIMEO  = Sockets.SO_SNDTIMEO;          { socket sending timeout }
  SO_RCVTIMEO  = Sockets.SO_RCVTIMEO;          { socket receiving timeout }
*)
  
  FD_READ_BIT = 0;
  FD_READ = (1  shl  FD_READ_BIT);
  FD_WRITE_BIT = 1;
  FD_WRITE = (1  shl  FD_WRITE_BIT);
  FD_OOB_BIT = 2;
  FD_OOB = (1  shl  FD_OOB_BIT);
  FD_ACCEPT_BIT = 3;
  FD_ACCEPT = (1  shl  FD_ACCEPT_BIT);
  FD_CONNECT_BIT = 4;
  FD_CONNECT = (1  shl  FD_CONNECT_BIT);
  FD_CLOSE_BIT = 5;
  FD_CLOSE = (1  shl  FD_CLOSE_BIT);
  FD_QOS_BIT = 6;
  FD_QOS = (1  shl  FD_QOS_BIT);
  FD_GROUP_QOS_BIT = 7;
  FD_GROUP_QOS = (1  shl  FD_GROUP_QOS_BIT);
  FD_MAX_EVENTS = 8;
  FD_ALL_EVENTS = ((1  shl  FD_MAX_EVENTS) - 1);
  
  SD_RECEIVE = SHUT_RD;
  SD_SEND = SHUT_WR;
  SD_BOTH = SHUT_RDWR;

type
  TWinLinger = record
    l_onoff: Word;
    l_linger: Word;
  end;
  PWinLinger = ^TWinLinger;

  PWinInAddr = ^TWinInAddr;
  TWinInAddr = record
    case integer of
      //0: (S_un_b: SunB);
      //1: (S_un_w: SunW);
      2: (S_addr: LongWord);
  end;

  winsockaddr_in = packed record
    case Integer of
      0: (sin_family: Word;
          sin_port: Word;
          sin_addr: TWinInAddr;
          sin_zero: array[0..7] of Char);
      1: (sa_family: Word;
          sa_data: array[0..13] of Char)
  end;

  PWinSOCKADDR = ^TWinSockAddr;

  TWinSockAddrIn = winsockaddr_in;
  TWinSockAddr = winsockaddr_in;

  PServEnt = ^TServEnt;
  servent = packed record
    s_name: PChar;
    s_aliases: ^PChar;
    s_port: Word;
    s_proto: PChar;
  end;
  TServEnt = servent;

  PProtoEnt = ^TProtoEnt;
  protoent = packed record
    p_name: PChar;
    p_aliases: ^Pchar;
    p_proto: Smallint;
  end;
  TProtoEnt = protoent;

  PHostEnt = ^THostEnt;
  hostent = packed record
    h_name: PChar;
    h_aliases: ^PChar;
    h_addrtype: Smallint;
    h_length: Smallint;
    case Byte of
      0: (h_addr_list: ^PChar);
      1: (h_addr: ^PChar)
  end;
  THostEnt = hostent;

function nixwWSAASyncSelect(s: TSocket; wnd: HWND; wMsg, Event: Integer): Integer;stdcall;

function nixwOpenSocket(af, struct, protocol: Longint): TSocket; stdcall;
function nixwConnect(s: TSocket; var name: TWinSockAddr; namelen: Integer): Integer; stdcall;

function nixwClosesocket(s: TSocket): Integer; stdcall;

procedure nixwWSASetLastError(iError: Integer); stdcall;
function nixwWSAGetLastError: Integer; stdcall;

function nixwWSACancelAsyncRequest(hAsyncTaskHandle: THandle): Integer; stdcall;

function nixwGetServByName(name, proto: PChar): PServEnt; stdcall;
function nixwGetProtoByName(name: PChar): PProtoEnt; stdcall;
function nixwGetHostByName(name: PChar): PHostEnt; stdcall;
function nixwGetHostByAddr(addr: Pointer; len, Struct: Integer): PHostEnt; stdcall;
function nixwGetHostName(name: PChar; len: Integer): Integer; stdcall;


function nixwShutdown(s: TSocket; how: Integer): Integer; stdcall;
function nixwSetSockOpt(s: TSocket; level, optname: Integer; optval: PChar; optlen: Integer): Integer; stdcall;
function nixwGetSockOpt(s: TSocket; level, optname: Integer;
  optval: PChar; var optlen: Integer): Integer; stdcall;

function nixwSend(s: TSocket; var Buf; len, flags: Integer): Integer; stdcall;
function nixwSendTo(s: TSocket; var Buf; len, flags: Integer; var addrto: TWinSockAddr; tolen: Integer): Integer; stdcall;

function nixwRecv(s: TSocket; var Buf; len, flags: Integer): Integer; stdcall;
function nixwRecvFrom (s: TSocket; var Buf; len, flags: Integer;
  var from: TWinSockAddr; var fromlen: Integer): Integer; stdcall;

function nixwListen(s: TSocket; backlog: Integer): Integer; stdcall;
function nixwIoctlSocket(s: TSocket; cmd: DWORD; var arg: LongWord): Integer; stdcall;

function nixwntohs(netshort: Word): Word; stdcall;
function nixwntohl(netlong: LongWord): LongWord; stdcall;
function nixwInet_ntoa(inaddr: TWinInAddr): PChar; stdcall;
function nixwInet_addr(cp: PChar): LongWord; stdcall;

function nixwGetSockName(s: TSocket; var name: TWinSockAddr; var namelen: Integer): Integer; stdcall;
function nixwGetPeerName(s: TSocket; var name: TWinSockAddr; var namelen: Integer): Integer; stdcall;

function nixwBind(s: TSocket; var addr: TWinSockAddr; namelen: Integer): Integer; stdcall;
function nixwAccept(s: TSocket; addr: PWinSockAddr; addrlen: PInteger): TSocket; stdcall;

implementation


uses
  rtcUnixAsyncSock;

{$ifdef NixSockDebug}
var
  DbgLVL : byte;
{$endif}

type
  THostEntRecord = record
    ent       : hostent; // win entry

    name      : AnsiString;
    alisases  : AnsiString;
    addr      : Integer;
    addrptr   : array [0..1] of PInteger; // last addrptr must be nil
  end;

  TProtoEntRecord = record
    ent     : protoent;
    fpEnt   : TProtocolEntry;
  end;

  TServEntRecord = record
    ent     : servent;
    fpEnt   : TServiceEntry;
  end;

threadvar
  WSASocketError  : Integer;
  TempString      : AnsiString;
  lHostEnt        : THostEntRecord;
  lServEnt        : TServEntRecord;
  lProtoEnt       : TProtoEntRecord;


procedure WSAUpdateSocketError;
begin
  WSASocketError := socketerror;
  if WSASocketError = ESysEAGAIN then WSASocketError := EsockEWOULDBLOCK;
end;

procedure WinToNixAddr(const Win: TWinSockAddr; var Nix: SockAddr);
begin
  nix.sa_family := win.sa_family;
  nix.sin_port := win.sin_port;
  nix.sin_addr.s_addr := win.sin_addr.s_addr;
end;

procedure NixToWinAddr(const Nix: SockAddr; var Win: TWinSockAddr);
begin
  win.sa_family := nix.sa_family;
  win.sin_port := nix.sin_port;
  win.sin_addr.s_addr := nix.sin_addr.s_addr;
end;


const
  AsyncProcSetFlag : array [TAsyncProc] of Integer = (
    FD_CONNECT, FD_CLOSE, FD_READ, FD_WRITE, FD_ACCEPT
  );

procedure EntryToHostEntRecord(const entry: THostEntry; var Rec: THostEntRecord);
begin
  Rec.name := entry.Name;
  Rec.alisases := entry.Aliases;
  rec.addr := LongInt(entry.Addr);
  Rec.ent.h_length := 4;
  Rec.ent.h_addrtype := AF_INET; //??
  Rec.ent.h_addr := @Rec.addrptr; //??
  Rec.addrptr[0] := @Rec.addr;
  Rec.addrptr[1] := nil;
end;

function EventsToAProcSet(event: Integer): TAsyncProcSet;
var
  flag : TAsyncProc;
begin
  Result := [];
  if event = 0 then Exit;

  for flag := low(TAsyncProc) to high(TAsyncProc) do
    if (AsyncProcSetFlag[flag] and event) > 0 then Include(Result, flag);
end;


function nixwOpenSocket(af, struct, protocol: Longint): TSocket; stdcall;
var
  sk  : integer;
  man : TSocketManager;
begin
  {$ifdef NixSockDebug} DebugLine('[nixwOpenSocket] opening socket', DbgLVL); {$endif}
  sk := fpsocket(af, struct, protocol);
  if sk < -1 then begin
    Result := -1;
    Exit;
  end else begin
    man := CreatePoolManager;
    man.Wrapper := TSocketWrapper.Create;
    man.Handle := sk;
  end;
  
  Result := GetIndexFromManager(man);
    
end;

function DebugWSAEvents(Event: Integer): string; overload;
var
  i : integer;
const
  EVENTSTR : array [0..7] of string = ('FD_READ', 'FD_WRITE', 'FD_OOB',
    'FD_ACCEPT', 'FD_CONNECT', 'FD_CLOSE', 'FD_QOS', 'FD_GROUP_QOS');
begin
  Result := '';
  for i := 0 to 7 do
    if (Event and (1 shl i)) > 0 then
      Result := Result + EventStr[i] + ' ';
end;

function DebugWSAEvents(events: TAsyncProcSet): string; overload;
var
  ev   : TAsyncProc;
const
  EVENTSTR : array [TAsyncProc] of string = ( 'FD_CONNECT', 'FD_CLOSE', 'FD_WRITE', 'FD_READ', 'FD_ACCEPT');
begin
  Result := '';
  for ev := low(TAsyncProc) to high(TAsyncProc) do
    if ev in events then
      Result := Result + EVENTSTR[ev] + ' ';
end;



function nixwWSAASyncSelect(s: TSocket; wnd: HWND; wMsg, Event: Integer): Integer;
var
  man : TSocketManager;
  a   : TAsyncWrapper;
begin
  if (wMsg = 0) then begin
    Result := SOCKET_ERROR;
    Exit;
  end;
  //writeln('+++++ WSAASyncSelect for wMsg = ', wMsg, ' ', DebugWSAEvents(Event));
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then begin
    //writeln('+++++ Unfound socket. Exiting...');
    Exit;
  end;
  
  if (Event = 0) then begin
    if (man.Wrapper is TAsyncWrapper) then begin
      GetAsyncManager.RemoveManager(man);
      man.Wrapper.Free;
      man.Wrapper := TSocketWrapper.Create;
      man.Wrapper.Handle := man.Handle;
    end;
    Result := 0;
    

  end else begin
    if not (man.Wrapper is TAsyncWrapper) then begin
      man.Wrapper.Free;
      man.Wrapper := TAsyncWrapper.Create;
      man.Wrapper.Handle := man.Handle;
    end;

    SetSocketNonBlocking(man.Handle);

    a := TAsyncWrapper(man.Wrapper);
    //writeln('+++++ TAsyncWrapper = ', Integer(a));
    a.WinHwnd := wnd;
    a.wMsg := wMsg;
    a.Event := Event;
    a.Notifier := GetNotifier;// TLCLFormMessageNotifier.Create;
    a.TaskEvents := EventsToAProcSet(Event);

    if man.isListen and (ap_Accept in a.TaskEvents) then
      a.isAccepting := true;
    {if man.isListen then begin
      a.TaskEvents := a.TaskEvents + [ap_Accept];
      a.isAccepting := true;
    end;}

    if a.isConnected and (ap_Write in a.TaskEvents) then
      a.Notifier.Notify(man, ap_Write, 0);
    GetAsyncManager.AddManager(man);
  end;

  Result := 0;
end;

function nixwconnect(s: TSocket; var name: TWinSockAddr; namelen: Integer): Integer;
var
  man : TSocketManager;
  sck : SockAddr;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  FillChar(sck, sizeof(sck), 0);
  sck.sin_family := name.sin_family;
  sck.sin_addr.s_addr := NToHl(name.sin_addr.S_addr);
//  sck.sin_addr.s_addr := name.sin_addr.S_addr;
  sck.sin_port := name.sin_port;
  {$ifdef NixSockDebug} DebugLine('[nixwConnect] ip = %s, Port = %d', [NetAddrToStr(in_addr(name.sin_addr.S_addr)), NToHs(name.sin_port)], DbgLVL);{$endif}
  //writeln('Connect port = ', NToHs(name.sin_port));
  Result := man.Wrapper.connect(sck, sizeof(sck));
  WSAUpdateSocketError;
  if (Result < 0) and (WSASocketError = ESysEINPROGRESS) then
    WSASocketError := EsockEWOULDBLOCK;
  {$ifdef NixSockDebug} DebugLine('[nixwConnect] Result = %d, %d', [Result, WSASocketError], DbgLVL);{$endif}
end;

function nixwclosesocket(s: TSocket): Integer; stdcall;
var
  man : TSocketManager;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  Result := man.Wrapper.Close;
  if man.Wrapper is TASyncWrapper then
    GetAsyncManager.RemoveManager(man);

  man.Wrapper.Free;
  man.Wrapper := nil;
  ReleasePoolManager(man);
end;

procedure nixwWSASetLastError(iError: Integer);
begin
  WSASocketError := iError;
end;

function nixwWSAGetLastError: Integer;
begin
  Result := WSASocketError;
end;

function nixwWSACancelAsyncRequest(hAsyncTaskHandle: THandle): Integer;
begin
  Result := 0;
end;
{
function nixwWSAAsyncGetHostByName(HWindow: HWND; wMsg: LongWord; name,
  buf: PChar; buflen: Integer): THandle;
begin
  Result := 0;
end;

function nixwWSAAsyncGetHostByAddr(HWindow: HWND; wMsg: LongWord; addr: PChar;
  len, Struct: Integer; buf: PChar; buflen: Integer): THandle;
begin
  Result := 0;
end;
}
function nixwGetServByName(name, proto: PChar): PServEnt;
var
  n : string;
  p : string;
begin
  n := name;
  p := proto;
  Result := nil;
  if not GetServiceByName(n, p, lServEnt.fpEnt) then Exit;

  with lServEnt do begin
    if fpEnt.Name = '' then ent.s_name := nil
    else ent.s_name := @fpEnt.Name[1];

    if fpEnt.Aliases = '' then ent.s_aliases := nil
    else ent.s_aliases := @fpEnt.Aliases[1];

    if fpEnt.Protocol = '' then ent.s_proto := nil
    else ent.s_proto := @fpEnt.Protocol[1];
    ent.s_port := fpEnt.Port;
  end;

  Result := @lServEnt.ent;
end;

function nixwGetProtoByName(name: PChar): PProtoEnt;
var
  s : string;
begin
  s := name;
  Result := nil;
  if not GetProtocolByName(s, lProtoEnt.fpEnt) then Exit;

  with lProtoEnt do begin
    ent.p_proto := fpEnt.Number;
    if fpEnt.Name = '' then ent.p_name := nil
    else ent.p_name := @fpEnt.Name[1];

    if fpEnt.Aliases = '' then ent.p_aliases := nil
    else ent.p_aliases := @fpEnt.Aliases[1];

    Result := @ent;
  end;
end;

function nixwGetHostByAddr(addr: Pointer; len, Struct: Integer): PHostEnt;
var
  nixaddr   : THostAddr;
  entry     : THostEntry;
begin
  Result := nil;
  case Struct of
    AF_INET: begin
      nixaddr.s_addr := PWinInAddr(addr)^.s_addr;
      if GetHostByAddr(nixaddr, entry)or ResolveHostByAddr(nixaddr, entry) then begin
        EntryToHostEntRecord(entry, lHostEnt);
        Result := @lHostEnt.ent;
      end;
      WSAUpdateSocketError;
    end;
    AF_INET6: begin
      Result := nil;
      {
      if len > sizeof(nixaddr6) then len := sizeof(nixaddr6);
      move(addr^, nixaddr6, len);
      if ResolveHostByAddr6(nixaddr6, entry6) then begin
        // todo:
      end;}
      //WSAUpdateSocketError;

    end;
  end;
end;

function nixwGetHostByName(name: PChar): PHostEnt;
var
  entry : THostEntry;
begin
  FillChar(entry, sizeof(entry), 0);
  if GetHostByName(name, entry) or ResolveHostByName(name, entry) then begin
    EntryToHostEntRecord(entry, lHostEnt);
    Result := @lHostEnt.ent;
  end else
    Result := nil;
  WSAUpdateSocketError;
end;

function nixwGetHostName(name: PChar; len: Integer): Integer;
begin
  Result := 0;
end;

function nixwShutdown(s: TSocket; how: Integer): Integer;
var
  man : TSocketManager;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  //writeln('*** shuting down socket ', man.Wrapper.Handle);
  Result := man.Wrapper.shutdown(how);
  WSAUpdateSocketError;
end;

function nixwSetSockOpt(s: TSocket; level, optname: Integer; optval: PChar;
  optlen: Integer): Integer;
var
  man : TSocketManager;
  
  lng : PWinLinger;
  l  : TLinger;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;
  
  case optname of
  
    SO_LINGER:begin
      lng := PWinLinger(optval);
      l.l_linger := lng.l_linger;
      l.l_onoff := lng.l_onoff;
      Result :=  man.Wrapper.SetSockOpt(level, optname, @l, sizeof(l));
    end;

  else
    Result := man.Wrapper.SetSockOpt(level, optname, optval, optlen);
  end;

  WSAUpdateSocketError;
end;

function nixwGetSockOpt(s: TSocket; level, optname: Integer; optval: PChar;
  var optlen: Integer): Integer;
var
  man : TSocketManager;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  Result := man.Wrapper.GetSockOpt(level, optname, optval, optlen);
  WSAUpdateSocketError;
end;

function nixwSendTo(s: TSocket; var Buf; len, flags: Integer;
  var addrto: TWinSockAddr; tolen: Integer): Integer;
var
  man : TSocketManager;
  nixaddr : sockaddr;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  WinToNixAddr(addrto, nixaddr);
  Result := man.Wrapper.SendTo(Buf, len, flags, nixaddr, sizeof(nixaddr));

  WSAUpdateSocketError;
end;

function nixwSend(s: TSocket; var Buf; len, flags: Integer): Integer;
var
  man : TSocketManager;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  Result := man.Wrapper.Send(Buf, len, flags);
  WSAUpdateSocketError;
end;

function nixwRecv(s: TSocket; var Buf; len, flags: Integer): Integer;
var
  man : TSocketManager;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  Result := man.Wrapper.Recv(Buf, len, flags);
  WSAUpdateSocketError;
end;

function nixwRecvFrom(s: TSocket; var Buf; len, flags: Integer;
  var from: TWinSockAddr; var fromlen: Integer): Integer;
var
  man : TSocketManager;
  nixaddr : sockaddr;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  Result := man.Wrapper.RecvFrom(Buf, len, flags, nixaddr, fromlen);

  NixToWinAddr(nixaddr, from);
  WSAUpdateSocketError;
end;

function nixwntohs(netshort: Word): Word;
begin
  Result := NToHs(netshort);
end;

function nixwntohl(netlong: LongWord): LongWord;
begin
  Result := NToHl(netlong)
end;

function nixwListen(s: TSocket; backlog: Integer): Integer;
var
  man : TSocketManager;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  {$ifdef NixSockDebug} DebugLine('[nixwListen] listening to socket %d', [man.Handle], DbgLVL); {$endif}
  if not Assigned(man) then Exit;

  man.isListen := true;
  Result := man.Wrapper.Listen(backlog);

  WSAUpdateSocketError;
end;

function nixwIoctlSocket(s: TSocket; cmd: DWORD; var arg: LongWord): Integer;
var
  man : TSocketManager;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  Result := man.Wrapper.IoctlSocket(cmd, arg);
  WSAUpdateSocketError;
end;

function nixwInet_ntoa(inaddr: TWinInAddr): PChar;
var
  nixadd : TInAddr;
begin
  nixadd.s_addr := inaddr.S_addr;
  TempString:=NetAddrToStr(nixadd);
  if TempString = '' then Result := nil
  else Result := @TempString[1];
end;

function nixwInet_addr(cp: PChar): LongWord;
var
  s : AnsiString;
begin
  if cp = nil then
    Result := 0
  else begin
    s := cp;
    Result := LongWord(StrToHostAddr(s));
  end;
end;


function nixwGetSockName(s: TSocket; var name: TWinSockAddr; var namelen: Integer): Integer;
var
  man : TSocketManager;
  addr  : sockaddr;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  namelen := sizeof(addr);
  Result := fpgetsockname(man.Handle, @addr, @namelen);

  if Result >= 0 then NixToWinAddr(addr, name);
end;

function nixwGetPeerName(s: TSocket; var name: TWinSockAddr; var namelen: Integer): Integer;
var
  man : TSocketManager;
  addr  : sockaddr;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  namelen := sizeof(addr);
  Result := fpgetpeername(man.Handle, @addr, @namelen);

  if Result >= 0 then NixToWinAddr(addr, name);
end;

function nixwBind(s: TSocket; var addr: TWinSockAddr; namelen: Integer): Integer;
var
  man : TSocketManager;
  nixaddr : SockAddr;
begin
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then Exit;

  FillChar(nixaddr, sizeof(nixaddr), 0);

  WinToNixAddr(addr, nixaddr);
  Result := man.Wrapper.Bind(nixaddr, sizeof(nixaddr));

  WSAUpdateSocketError;
end;


function nixwAccept(s: TSocket; addr: PWinSockAddr; addrlen: PInteger): TSocket;
var
  man     : TSocketManager;
  wrp     : TSocketWrapper;
  next    : TSocketManager;
  nixaddr : sockaddr;
begin
  {$ifdef NixSockDebug} DebugLine('[nixwAccept] nixWAccept ',DbgLVL); {$endif}
  Result := SOCKET_ERROR;
  man := GetManagerFromIndex(s);
  if not Assigned(man) then begin
    {$ifdef NixSockDebug} DebugLine('[nixwAccept] cannot find wrapper',DbgLVL); {$endif}
    Exit;
  end;

  {$ifdef NixSockDebug} DebugLine('[nixwAccept] accepting from socket %d', [man.Handle], DbgLVL); {$endif}
  Result := man.Wrapper.Accept(@nixaddr, addrlen);
  if Result >= 0 then begin
    NixToWinAddr(nixaddr, addr^);

    next := CreatePoolManager;
    if man.Wrapper is TASyncWrapper then begin
      // this important for Linux systems, since, accepted socket
      // DOES NOT inherites NON_BLOCKING option of the listening socket!
      SetSocketNonBlocking(Result);
      wrp := TAsyncWrapper.Create;

      TAsyncWrapper(wrp).WinHwnd := TAsyncWrapper(man.Wrapper).WinHwnd;
      TAsyncWrapper(wrp).wMsg := TAsyncWrapper(man.Wrapper).wMsg;
      TAsyncWrapper(wrp).Event := TAsyncWrapper(man.Wrapper).Event;

      TAsyncWrapper(wrp).isConnected := true;
      TAsyncWrapper(wrp).Notifier := GetNotifier;
      TAsyncWrapper(wrp).TaskEvents := TAsyncWrapper(man.Wrapper).TaskEvents;

      //writeln(Format('*** Inherit WSA events %d from parent %d ' + DebugWSAEvents(TAsyncWrapper(wrp).TaskEvents),
      //  [Result, TAsyncWrapper(man.Wrapper).Handle]));

      //TAsyncWrapper(wrp).WantedEvents := TAsyncWrapper(man.Wrapper).WantedEvents;
    end else if man.Wrapper is TSocketWrapper then
      wrp := TSocketWrapper.Create;

    if Assigned(wrp) then begin
      wrp.Handle := Result;
    end;
    next.Wrapper := wrp;
    next.Handle := Result;
    GetAsyncManager.AddManager(next);
    Result := GetIndexFromManager(next);

    if next.Wrapper is TAsyncWrapper then begin
      if ap_Write in TAsyncWrapper(next.Wrapper).TaskEvents then TAsyncWrapper(wrp).Notifier.Notify(next, ap_Write, 0);
    end;

  end;

  WSAUpdateSocketError;
end;

function nixWSACleanup: Integer;
begin
  Result := 0;
end;

initialization
{$ifdef NixSockDebug}
  DbgLVL := RegisterDebugLVL;
  Exclude(DebugLvl, DbgLvl);
{$endif}

end.

