unit rtcWafCli; 

{$include rtcDefs.inc}

interface

uses
  {$ifdef Windows}Windows, Messages,{$endif}
  SysUtils, StdCtrls, Variants, Classes, Graphics, Controls,

  rtcFunction, rtcDataCli, rtcCliModule,
  rtcInfo, rtcConn, rtcHttpCli, rtcSyncObjs,

  rtcCache, rtcLog, rtcThrPool, rtcTimer,

  rtcLink, rtcFormsCli;

type
  TRtcWafClientModule = class;

  TRtcWafClientEvent = procedure(Sender:TRtcWafClientModule) of object;
  TRtcWafClientErrorEvent = procedure(Sender:TRtcWafClientModule; const ErrorMessage:string) of object;

  TRtcWafClientModule = class(TRtcComponent)
  private
    FCliGet: TRtcDataClient;
    FCliPut: TRtcDataClient;

    ModGet: TRtcClientModule;
    ModPut: TRtcClientModule;

    resRun: TRtcResult;
    resGet: TRtcResult;
    resShow: TRtcResult;
    resPut: TRtcResult;
    resStop: TRtcResult;
    QuitTimer: TRtcTimer;

    CS:TRtcCritSec;
    ToSend:TRtcArray;
    isSending,
    isShowing:boolean;
    AppID:AnsiString;
    App:TrtcApplicationC;

    FOnStart: TRtcWafClientEvent;
    FOnShow: TRtcWafClientEvent;
    FOnError: TRtcWafClientErrorEvent;
    FOnStop: TRtcWafClientEvent;
    FOnHide: TRtcWafClientEvent;

    procedure Set_CliGet(const Value: TRtcDataClient);
    procedure Set_CliPut(const Value: TRtcDataClient);

    function Get_AutoEncrypt: integer;
    function Get_AutoLogin: boolean;
    function Get_AutoRepost: integer;
    function Get_DataFormat: TRtcDataFormat;
    function Get_ForceEncrypt: boolean;
    function Get_ModuleFileName: AnsiString;
    function Get_ModuleHost: AnsiString;
    function Get_OnLogin: TRtcFunctionPrepareEvent;
    function Get_OnLoginAborted: TRtcResultEvent;
    function Get_OnLoginResult: TRtcResultEvent;
    function Get_OnSessionClose: TRtcNotifyEvent;
    function Get_OnSessionOpen: TRtcNotifyEvent;
    function Get_SecureKey: AnsiString;

    procedure Set_AutoEncrypt(const Value: integer);
    procedure Set_AutoLogin(const Value: boolean);
    procedure Set_AutoRepost(const Value: integer);

    procedure Set_DataFormat(const Value: TRtcDataFormat);
    procedure Set_ForceEncrypt(const Value: boolean);
    procedure Set_ModuleFileName(const Value: AnsiString);
    procedure Set_ModuleHost(const Value: AnsiString);
    procedure Set_OnLogin(const Value: TRtcFunctionPrepareEvent);
    procedure Set_OnLoginAborted(const Value: TRtcResultEvent);
    procedure Set_OnLoginResult(const Value: TRtcResultEvent);
    procedure Set_OnSessionClose(const Value: TRtcNotifyEvent);
    procedure Set_OnSessionOpen(const Value: TRtcNotifyEvent);
    procedure Set_SecureKey(const Value: AnsiString);

    {$ifdef COMPRESS}
    function Get_Compress: TRtcCompressLevel;
    procedure Set_Compress(const Value: TRtcCompressLevel);
    {$endif}

  protected
    procedure resRunReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
    procedure resGetReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
    procedure resShowReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
    procedure resPutReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
    procedure resStopReturn(Sender: TRtcConnection; Data, Result: TRtcValue);

    procedure resGetRequestAborted(Sender: TRtcConnection; Data, Result: TRtcValue);
    procedure resPutRequestAborted(Sender: TRtcConnection; Data, Result: TRtcValue);

    procedure QuitTimerTimer;

    procedure DoQuit(Sender:TObject);
    procedure DoError(const s:string);
    procedure ToExecute(const Data:TRtcValueObject);
    procedure DoTerminate;
    procedure CallShowing(Sender:TRtcConnection);

  private
    { Private declarations }
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

    procedure Run;
    procedure Quit;

  published
    { Client connection component which will be used for all GET operations. }
    property ClientGet:TRtcDataClient read FCliGet write Set_CliGet;
    { Client connection component which will be used for all PUT operations. }
    property ClientPut:TRtcDataClient read FCliPut write Set_CliPut;

    { Event fired when the Application is starting up (initial handshake OK). }
    property OnAppStart:TRtcWafClientEvent read FOnStart write FOnStart;
    { Event fired when the Application is stopping (closing in progress).
      This event will only be fired if OnAppStart event was fired before. }
    property OnAppStop:TRtcWafClientEvent read FOnStop write FOnStop;

    { Event fired when the Application has received its first data
      which will most likely display its main form (first real data arrived). }
    property OnAppShow:TRtcWafClientEvent read FOnShow write FOnShow;
    { Event fired after the main form was closed and the Application is no longer visible.
      This event will only be fired if the OnAppShow event was fired before. }
    property OnAppHide:TRtcWafClientEvent read FOnHide write FOnHide;

    { Event fired when unespected errors happen, usually caused by communication problems. }
    property OnAppError:TRtcWafClientErrorEvent read FOnError write FOnError;

  published
    // properties and events from RtcClientModule ...
    {$IFDEF COMPRESS}
    { Use this property to define what compression level you want to use when sending
      data from Client to Server. Default Compression value is "cNone" (no compression).
      You can use different compression levels between client and server (for example,
      fast or no compression from client and max from server). If your client has to
      work with servers which don't support compression, you have to use "cNone". }
    property Compression:TRtcCompressLevel read Get_Compress write Set_Compress default cNone;
    {$ENDIF}

    { Use this property to define what data format you want to use when sending data from
      Client to Server. If your client will only be talking to a Server written using RTC
      components, it is recommended to use "fmt_RTC", which upports all RTC data types, automatic
      compression, automatic encryption and automatic Sessions, as well as nested function calls
      and sending multiple function calls in a single request. On the other hand, if your client
      has to communicate with a Server which does NOT support the RTC format, you can use
      the "fmt_XMLRPC" format, which will work with Servers implementing the XML-RPC format. @html(<br><br>)
      NOTE: Since advanced functionality like automatic Compression, auto Encryption
      and automatic Sessions are native to the RTC format and other Servers do not implement it,
      those advanced properties will be ignored and assumed to be FALSE with all formats other
      then the "fmt_RTC" format. If you need those advanced options, use the "fmt_RTC" format. }
    property DataFormat:TRtcDataFormat read Get_DataFormat write Set_DataFormat default fmt_RTC;

    { Set this property to a value other than 0 if you want to use automatic Encryption
      with a random generated key of "EncryptKey" bytes. One byte stands for
      encryption strength of 8 bits. For strong 256-bit encryption, use 32 bytes. @html(<br><br>)

      The final encryption key is combined from a client-side key and a key
      received from the server, where server decides about its encryption strength.
      If server doesn't support Encryption, data will not be encrypted,
      regardless of the value you use for AutoEncrypt. @html(<br><br>)

      EncryptionKey uses sessions to keep the encryption keys. When you set EncryptionKey
      to a value other than 0 (turn it ON), AutoSessions will be set to TRUE.
      Also, setting AutoSessions to FALSE will set EncryptionKey to 0 (OFF). }
    property EncryptionKey:integer read Get_AutoEncrypt write Set_AutoEncrypt default 0;

    { If you need a 100% secure connection, define a Secure Key string
      (combination of letters, numbers and special characters) for each
      ServerModule/ClientModule pair, additionally to the EncryptionKey value.
      ClientModule will be able to communicate with the ServerModule ONLY if
      they both use the same SecureKey. Default value for the SecureKey is
      an empty string (means: no secure key). @html(<br><br>)

      SecureKey will be used in the encryption initialisation handshake,
      to encrypt the first key combination sent by the ClientModule.
      Since all other data packages are already sent using some kind of encryption,
      by defining a SecureKey, you encrypt the only key part which would have
      been sent out without special encryption. }
    property SecureKey:AnsiString read Get_SecureKey write Set_SecureKey;

    { Setting this property to TRUE will tell the ClientModule to work with the
      Server ONLY if Server supports encryption. If AutoEncryptKey is > 0 and
      server doesn't support encryption, function calls will not be passed to
      the server and any response coming from the server will be rejected, until
      server enables encryption. }
    property ForceEncryption:boolean read Get_ForceEncrypt write Set_ForceEncrypt default False;

    { Set this property to a value other than 0 (zero) if you want the ClientModule to
      auto-repost any request up to "AutoRepost" times, in case the connection gets lost
      while sending data to server or receiving data from server.
      If value is lower than zero, requests will be reposted unlimited number of times. }
    property AutoRepost:integer read Get_AutoRepost write Set_AutoRepost default 0;

    { Set this property to TRUE if you want to enable the use of the
      OnLogin, OnLoginResult and OnLoginAborted events to implement automatic login. }
    property AutoLogin:boolean read Get_AutoLogin write Set_AutoLogin default false;

    { "Request.Host" will be assigned this property before sending the request out. @html(<br>)
      It is not necessary to set this property if your server's ServerModule component
      left its ModuleHost property blank and there's no remote Functions used by that
      ServerModule which would explicitly use the "Request.Host" header. On the other hand,
      for servers which serve multiple hosts, mostly where ServerModule has assigned its
      ModuleHost property, it is very important to set this ClientModule's ModuleHost
      property to the appropriate host name. }
    property ModuleHost:AnsiString read Get_ModuleHost write Set_ModuleHost;
    { To be able to call remote functions, this ClientModule's ModuleFileName
      property has to be identical to the "ModuleFileName" property of the ServerModule
      which you want to use. "Request.FileName" will be assigned this property before
      sending any request out, so you won't be preparing the Request headers manualy. @html(<br>)
      All data (parameters and function calls) will be passed to the server module through
      request's Content body, so that ServerModule won't need to check the request headers
      for anything else than it's FileName to know if the request is directed to it. }
    property ModuleFileName:AnsiString read Get_ModuleFileName write Set_ModuleFileName;

    { This event will be called after a new Session has been opened. }
    property OnSessionOpen:TRtcNotifyEvent read Get_OnSessionOpen write Set_OnSessionOpen;
    { This event will be called before an existing Session is going to be closed. }
    property OnSessionClose:TRtcNotifyEvent read Get_OnSessionClose write Set_OnSessionClose;

    { Use this event to implement automatic user login. Set AutoLogin to TRUE and
      this event will be fired immediately after the initial connection handshake.
      To prepare the remote function, use the "Data" object passed as a parameter.
      After the function call has returned, the OnLoginResult event will be triggered.
      If there was an error and the request was aborted, OnLoginAborted will be called. }
    property OnLogin:TRtcFunctionPrepareEvent read Get_OnLogin write Set_OnLogin;

    { Use this event to implement automatic user login. See "OnLogin" for more info. }
    property OnLoginResult:TRtcResultEvent read Get_OnLoginResult write Set_OnLoginResult;

    { Use this event to implement automatic user login. See "OnLogin" for more info. }
    property OnLoginAborted:TRtcResultEvent read Get_OnLoginAborted write Set_OnLoginAborted;
  end;

implementation

constructor TRtcWafClientModule.Create(AOwner:TComponent);
  begin
  inherited;

  CS:=TRtcCritSec.Create;
  ToSend:=nil;
  isSending:=False;
  isShowing:=False;

  ModGet:=TRtcClientModule.Create(nil);
  ModGet.AutoSessions:=True;
  ModGet.HyperThreading:=True;

  ModPut:=TRtcClientModule.Create(nil);
  ModPut.HyperThreading:=True;
  ModPut.AutoSessions:=True;

  resRun:=TRtcResult.Create(nil);
  resRun.OnReturn:=resRunReturn;
  resRun.RequestAborted:=resGetRequestAborted;

  resGet:=TRtcResult.Create(nil);
  resGet.OnReturn:=resGetReturn;
  resGet.RequestAborted:=resGetRequestAborted;

  resShow:=TRtcResult.Create(nil);
  resShow.OnReturn:=resShowReturn;
  resShow.RequestAborted:=resPutRequestAborted;

  resPut:=TRtcResult.Create(nil);
  resPut.OnReturn:=resPutReturn;
  resPut.RequestAborted:=resPutRequestAborted;

  resStop:=TRtcResult.Create(nil);
  resStop.OnReturn:=resStopReturn;
  resStop.RequestAborted:=resPutRequestAborted;

  QuitTimer:=TRtcTimer.Create(False);
  end;

destructor TRtcWafClientModule.Destroy;
  begin
  ModGet.Free;
  ModPut.Free;

  resRun.Free;
  resGet.Free;
  resShow.Free;
  resPut.Free;
  resStop.Free;

  TRtcTimer.Stop(QuitTimer);
  CS.Free;
  inherited;
  end;

procedure TRtcWafClientModule.Run;
  begin
  if not assigned(FCliGet) or not assigned(FCliPut) then
    raise Exception.Create('ClientGet or ClientPut not assigned.');

  FCliGet.AutoConnect:=True;
  FCliPut.AutoConnect:=True;

  with ModGet do
    begin
    Data.NewFunction('run');
    Call(resRun);
    end;
  end;

procedure TRtcWafClientModule.Quit;
  begin
  with ModPut do
    begin
    with Data.NewFunction('stop') do
      asString['app']:=AppID;
    Call(resStop);
    end;
  end;

procedure TRtcWafClientModule.resRunReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
  begin
  // We will either get the AppID as String, or an Exception
  if not Sender.inMainThread then
    Sender.Sync(resRunReturn,Data,Result)
  else if Result.isType=rtc_String then
    begin
    AppID:=Result.asString;

    App:=TrtcApplicationC.Create(nil);
    App.OnQuit:=DoQuit;
    App.OnSend:=ToExecute;

    if assigned(FOnStart) then FOnStart(self);

    Log('RUN', String(AppID));
    // Start the "get loop" ...
    with ModGet do
      begin
      Data.NewFunction('get');
      Call(resGet,true,Sender);
      end;

    // Load the Main Form ...
    with ModPut do
      begin
      with Data.NewFunction('show') do
        asString['app']:=AppID;
      Call(resShow);
      end;
    end
  else
    begin
    if Result.isType=rtc_Exception then
      DoError(Result.asException);
    DoTerminate;
    end;
  end;

procedure TRtcWafClientModule.resGetReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
  var
    a:integer;
    o:TRtcValueObject;
    Res:TRtcArray;
  begin
  if Result.isType=rtc_Exception then
    begin
    // Server error or Application Quit
    if not Sender.inMainThread then
      Sender.Sync(resGetReturn,Data,Result)
    else
      begin
      if not (Result.asException='QUIT') then
        DoError(Result.asException);
      DoTerminate;
      end;
    end
  else if assigned(App) then
    begin
    if Result.isType=rtc_Array then
      begin
      if not isShowing then
        begin
        isShowing:=True;
        if assigned(FOnShow) then
          if Sender.inMainThread then
            FOnShow(self)
          else
            Sender.Sync(CallShowing);
        end;
      Res:=Result.asArray;

      App.EnterExecute;
      try
        for a:=0 to Res.Count-1 do
          begin
          o:=Res.asObject[a];
          Res.asObject[a]:=nil;
          App.ToExecute(o);
          end;
      finally
        App.LeaveExecute;
        end;
      end;
    with ModGet do
      begin
      Data.NewFunction('get');
      Call(resGet,True,Sender);
      end;
    end
  else
    begin
    // Server error or Application Quit
    if not Sender.inMainThread then
      Sender.Sync(resGetReturn,Data,Result)
    else
      begin
      DoError('Get request terminated');
      DoTerminate;
      end;
    end;
  end;

procedure TRtcWafClientModule.resShowReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
  begin
  if (Result.isType<>rtc_Boolean) or (Result.asBoolean=False) then
    begin
    if not Sender.inMainThread then
      Sender.Sync(resShowReturn,Data,Result)
    else
      begin
      if Result.isType=rtc_Exception then
        DoError(Result.asException)
      else
        DoError('Show request terminated');
      DoTerminate;
      end;
    end;
  end;

procedure TRtcWafClientModule.ToExecute(const Data: TRtcValueObject);
  var
    goSend:boolean;
    SendNow:TRtcArray;
  begin
  // Called when data was prepared by the Client for sending
  goSend:=False;
  SendNow:=nil;
  CS.Enter;
  try
    if assigned(Data) then
      begin
      if not assigned(ToSend) then
        ToSend:=TRtcArray.Create;
      ToSend.asObject[ToSend.Count]:=Data;
      end;
    if assigned(ToSend) and not isSending then
      begin
      isSending:=True;
      goSend:=True;
      SendNow:=ToSend;
      ToSend:=nil;
      end;
  finally
    CS.Leave;
    end;
  if goSend then
    begin
    with ModPut do
      begin
      with Data.NewFunction('put') do
        begin
        asString['app']:=AppID;
        asObject['data']:=SendNow;
        end;
      Call(resPut);
      end;
    end;
  end;

procedure TRtcWafClientModule.resPutReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
  var
    goSend:boolean;
    SendNow:TRtcArray;
  begin
  if (Result.isType=rtc_Boolean) and (Result.asBoolean=True) then
    begin
    // Called when data was prepared by the Client for sending
    goSend:=False;
    SendNow:=nil;
    CS.Enter;
    try
      isSending:=False;
      if assigned(ToSend) then
        begin
        isSending:=True;
        goSend:=True;
        SendNow:=ToSend;
        ToSend:=nil;
        end;
    finally
      CS.Leave;
      end;
    if goSend then
      begin
      with ModPut do
        begin
        with Data.NewFunction('put') do
          begin
          asString['app']:=AppID;
          asObject['data']:=SendNow;
          end;
        Call(resPut,True,Sender);
        end;
      end;
    end
  else if not Sender.inMainThread then
    Sender.Sync(resPutReturn,Data,Result)
  else
    begin
    CS.Enter;
    try
      isSending:=False;
    finally
      CS.Leave;
      end;
    if Result.isType=rtc_Exception then
      DoError(Result.asException)
    else
      DoError('Put request terminated');
    DoTerminate;
    end;
  end;

procedure TRtcWafClientModule.resStopReturn(Sender: TRtcConnection; Data, Result: TRtcValue);
  begin
  if not ((Result.isType=rtc_Boolean) and (Result.asBoolean=True)) then
    begin
    if not Sender.inMainThread then
      Sender.Sync(resShowReturn,Data,Result)
    else
      DoTerminate;
    end;
  end;

procedure TRtcWafClientModule.DoQuit(Sender: TObject);
  begin
  Log('QUIT',String(AppID));
  // Application object is now being destroyed
  AppID:='';
  if assigned(App) then
    begin
    App.OnQuit:=nil;
    App.OnSend:=nil;
    App:=nil;
    end;
  { btnRun.Enabled:=True;
    btnQuit.Enabled:=False; }
  end;

procedure TRtcWafClientModule.resGetRequestAborted(Sender: TRtcConnection; Data, Result: TRtcValue);
  begin
  if not Sender.inMainThread then
    Sender.Sync(resGetRequestAborted,Data,Result)
  else
    begin
    DoError('Error connecting to Server.');
    DoTerminate;
    end;
  end;

procedure TRtcWafClientModule.resPutRequestAborted(Sender: TRtcConnection; Data, Result: TRtcValue);
  begin
  if not Sender.inMainThread then
    Sender.Sync(resPutRequestAborted,Data,Result)
  else
    begin
    DoError('Error connecting to Server.');
    DoTerminate;
    end;
  end;

procedure TRtcWafClientModule.DoTerminate;
  var
    a:TrtcApplicationC;
  begin
  if assigned(App) then
    begin
    Log('Terminated',String(AppID));
    AppID:='';
    a:=App; App:=nil;
    a.OnQuit:=nil;
    a.OnSend:=nil;
    a.Quit;
    end;

  TRtcTimer.Enable(QuitTimer,100,QuitTimerTimer,True,False);
  end;

procedure TRtcWafClientModule.QuitTimerTimer;
  begin
  FCliGet.AutoConnect:=False;
  FCliGet.SkipRequests;
  FCliGet.Disconnect;
  FCliGet.Session.Close;

  FCliPut.AutoConnect:=False;
  FCliPut.SkipRequests;
  FCliPut.Disconnect;
  FCliPut.Session.Close;

  if assigned(FOnStop) then
    FOnStop(self);

  if isShowing then
    begin
    isShowing:=False;
    if assigned(FOnHide) then
      FOnHide(self);
    end;
  end;

procedure TRtcWafClientModule.Set_CliGet(const Value: TRtcDataClient);
  begin
  if Value<>FCliGet then
    begin
    FCliGet := Value;
    ModGet.Client:=FCliGet;
    end;
  end;

procedure TRtcWafClientModule.Set_CliPut(const Value: TRtcDataClient);
  begin
  if Value<>FCliPut then
    begin
    FCliPut := Value;
    ModPut.Client:=FCliPut;
    end;
  end;

procedure TRtcWafClientModule.CallShowing(Sender: TRtcConnection);
  begin
  if assigned(FOnShow) then FOnShow(self);
  end;

procedure TRtcWafClientModule.DoError(const s: string);
  begin
  if assigned(FOnError) then FOnError(self,s);
  end;

function TRtcWafClientModule.Get_AutoEncrypt: integer;
  begin
  Result:=ModGet.EncryptionKey;
  end;

function TRtcWafClientModule.Get_AutoLogin: boolean;
  begin
  Result:=ModGet.AutoLogin;
  end;

function TRtcWafClientModule.Get_AutoRepost: integer;
  begin
  Result:=ModGet.AutoRepost;
  end;

function TRtcWafClientModule.Get_DataFormat: TRtcDataFormat;
  begin
  Result:=ModGet.DataFormat;
  end;

function TRtcWafClientModule.Get_ForceEncrypt: boolean;
  begin
  Result:=ModGet.ForceEncryption;
  end;

function TRtcWafClientModule.Get_ModuleFileName: AnsiString;
  begin
  Result:=ModGet.ModuleFileName;
  end;

function TRtcWafClientModule.Get_ModuleHost: AnsiString;
  begin
  Result:=ModGet.ModuleHost;
  end;

function TRtcWafClientModule.Get_OnLogin: TRtcFunctionPrepareEvent;
  begin
  Result:=ModGet.OnLogin;
  end;

function TRtcWafClientModule.Get_OnLoginAborted: TRtcResultEvent;
  begin
  Result:=ModGet.OnLoginAborted;
  end;

function TRtcWafClientModule.Get_OnLoginResult: TRtcResultEvent;
  begin
  Result:=ModGet.OnLoginResult;
  end;

function TRtcWafClientModule.Get_OnSessionClose: TRtcNotifyEvent;
  begin
  Result:=ModGet.OnSessionClose;
  end;

function TRtcWafClientModule.Get_OnSessionOpen: TRtcNotifyEvent;
  begin
  Result:=ModGet.OnSessionOpen;
  end;

function TRtcWafClientModule.Get_SecureKey: AnsiString;
  begin
  Result:=ModGet.SecureKey;
  end;

procedure TRtcWafClientModule.Set_AutoEncrypt(const Value: integer);
  begin
  ModGet.EncryptionKey:=Value;
  ModPut.EncryptionKey:=Value;
  end;

procedure TRtcWafClientModule.Set_AutoLogin(const Value: boolean);
  begin
  ModGet.AutoLogin:=Value;
  ModPut.AutoLogin:=Value;
  end;

procedure TRtcWafClientModule.Set_AutoRepost(const Value: integer);
  begin
  ModGet.AutoRepost:=Value;
  ModPut.AutoRepost:=Value;
  end;

procedure TRtcWafClientModule.Set_DataFormat(const Value: TRtcDataFormat);
  begin
  ModGet.DataFormat:=Value;
  ModPut.DataFormat:=Value;
  end;

procedure TRtcWafClientModule.Set_ForceEncrypt(const Value: boolean);
  begin
  ModGet.ForceEncryption:=Value;
  ModPut.ForceEncryption:=Value;
  end;

procedure TRtcWafClientModule.Set_ModuleFileName(const Value: AnsiString);
  begin
  ModGet.ModuleFileName:=Value;
  ModPut.ModuleFileName:=Value;
  end;

procedure TRtcWafClientModule.Set_ModuleHost(const Value: AnsiString);
  begin
  ModGet.ModuleHost:=Value;
  ModPut.ModuleHost:=Value;
  end;

procedure TRtcWafClientModule.Set_SecureKey(const Value: AnsiString);
  begin
  ModGet.SecureKey:=Value;
  ModPut.SecureKey:=Value;
  end;

procedure TRtcWafClientModule.Set_OnLogin(const Value: TRtcFunctionPrepareEvent);
  begin
  ModGet.OnLogin:=Value;
  ModPut.OnLogin:=Value;
  end;

procedure TRtcWafClientModule.Set_OnLoginAborted(const Value: TRtcResultEvent);
  begin
  ModGet.OnLoginAborted:=Value;
  ModPut.OnLoginAborted:=Value;
  end;

procedure TRtcWafClientModule.Set_OnLoginResult(const Value: TRtcResultEvent);
  begin
  ModGet.OnLoginResult:=Value;
  ModPut.OnLoginResult:=Value;
  end;

procedure TRtcWafClientModule.Set_OnSessionClose(const Value: TRtcNotifyEvent);
  begin
  ModGet.OnSessionClose:=Value;
  ModPut.OnSessionClose:=Value;
  end;

procedure TRtcWafClientModule.Set_OnSessionOpen(const Value: TRtcNotifyEvent);
  begin
  ModGet.OnSessionOpen:=Value;
  ModPut.OnSessionOpen:=Value;
  end;

{$ifdef COMPRESS}
function TRtcWafClientModule.Get_Compress: TRtcCompressLevel;
  begin
  Result:=ModGet.Compression;
  end;

procedure TRtcWafClientModule.Set_Compress(const Value: TRtcCompressLevel);
  begin
  ModGet.Compression:=Value;
  ModPut.Compression:=Value;
  end;
{$endif}

end.
