{ 
  RealThinClient W/A Framework: Client-side Form implementation
  Copyright (c) Danijel Tkalcec

  @exclude
}
unit rtc_FormCli;

{$include rtcDefs.inc}

interface

uses
  {$ifdef Windows}Windows, ShellAPI,{$endif}
  Messages, Classes, SysUtils, Graphics, Controls, Types, Forms,
  rtcInfo, rtcLog,

  rtcLink,
  rtcBaseCli,
  rtcControlsCli,
  rtc_FormX;

{$ifndef FPC}
  {.$DEFINE RTCWAF_MINIMIZEWITHMAINFORM}
  {$DEFINE RTCWAF_FORMS_TOTRAY}
{$endif}

type
  TrtcFormC=class(TRtcBaseFormC)
  protected
    EV:TRtcWinControlEvents;
    FOnDropFilesID:integer;

  {$IFDEF RTCWAF_MINIMIZEWITHMAINFORM}
    FOldRestore:TNotifyEvent;
    procedure WMSIZE(Var Msg: TWMSIZE); Message WM_SIZE;
    procedure WinRestore (Sender: TObject);
  {$ENDIF}
  {$IFDEF RTCWAF_FORMS_TOTRAY}
    procedure CreateParams(Var params: TCreateParams); override;
  {$ENDIF}

    function _Process(const xName:string; xData:TRtcValue):boolean; override;
    function _SetEvent(const xName:string; xID:longint):boolean; override;

    procedure _SetCom(Com:TRtcComC); override;

    {$ifndef FPC}
    procedure AcceptFiles( var msg : TMessage ); message WM_DROPFILES;
    {$else}
    procedure AcceptFiles(Sender: TObject; const FileNames: array of String);
    {$endif}
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    end;

implementation

{$ifndef FPC}
  {$R *.dfm}
{$endif}

constructor TrtcFormC.Create(AOwner: TComponent);
  begin
  inherited;
  Left:=0;
  Top:=0;
  EV:=nil;
  FOnDropFilesID:=0;
  {$ifdef FPC}
  AllowDropFiles:=False;
  {$endif}
  end;

destructor TrtcFormC.Destroy;
  var
    e:TObject;
  begin
  if (FOnDropFilesID>0) then
    {$ifndef FPC}
    DragAcceptFiles(Handle,False);
    {$else}
    AllowDropFiles:=False;
    {$endif}
  e:=EV;
  inherited;
  if assigned(e) then FreeAndNil(e);
  end;

{$ifndef FPC}
procedure TrtcFormC.AcceptFiles( var msg : TMessage );
  const
    cnMaxFileNameLen = 1024;
  var
    i,
    nCount     : integer;
    acFileName : array [0..cnMaxFileNameLen] of char;
    rec: TRtcArray;
  begin
  if (FOnDropFilesID=0) then
    begin
    msg.Result:=1;
    Exit;
    end;

  // find out how many files we're accepting
  nCount := DragQueryFile( msg.WParam,
                           $FFFFFFFF,
                           acFileName,
                           cnMaxFileNameLen );

  rec:=TRtcArray.Create;
  try
    // query Windows one at a time for the file name
    for i := 0 to nCount-1 do
      begin
      DragQueryFile( msg.WParam, i, acFileName, cnMaxFileNameLen );
      rec.asText[rec.Count]:=acFileName;
      end;
  finally
    // let Windows know that you're done
    DragFinish( msg.WParam );
    end;
  _Com.Event(FOnDropFilesID,rec);
  msg.Result:=0;
  end;
{$else}
procedure TrtcFormC.AcceptFiles(Sender: TObject; const FileNames: array of String);
  var
    rec:TRtcArray;
    i:integer;
  begin
  if (FOnDropFilesID=0) then Exit;

  rec:=TRtcArray.Create;
  for i := 0 to Length(FileNames)-1 do
    rec.asText[rec.Count]:=FileNames[i];
  _Com.Event(FOnDropFilesID,rec);
  end;
{$endif}

{$IFDEF RTCWAF_FORMS_TOTRAY}
procedure TrtcFormC.CreateParams(Var params: TCreateParams);
  begin
  inherited CreateParams( params );
  params.ExStyle := params.ExStyle or WS_EX_APPWINDOW;
  params.WndParent := GetDeskTopWindow;
  end;
{$ENDIF}

procedure TrtcFormC._SetCom(Com: TRtcComC);
  begin
  inherited;
  EV:=TRtcWinControlEvents.Create(Com);
  OnMouseDown:=EV.OnMouseDown(0);
  OnMouseUp:=EV.OnMouseUp(0);
  OnResize:=EV.OnResize(0);
  end;

function TrtcFormC._Process(const xName: string; xData: TRtcValue):boolean;
  begin
  Result:=True;
  if not inherited _Process(xName,xData) then
    begin
    Result:=False;
    Log('Unknown Form property: '+xName+' = '+String(xData.toCode));
    end;
  end;

function TrtcFormC._SetEvent(const xName: string; xID: Integer):boolean;
  begin
  Result:=True;
  if xName='onclick' then
    OnClick:=EV.OnClick(xID)
  else if xName='ondblclick' then
    OnDblClick:=EV.OnDblClick(xID)
  else if xName='onactivate' then
    OnActivate:=EV.OnActivate(xID)
  else if xName='ondeactivate' then
    OnDeactivate:=EV.OnDeactivate(xID)
  else if xName='onresize' then
    OnResize:=EV.OnResize(xID)
  else if xName='onkeydown' then
    OnKeyDown:=EV.OnKeyDown(xID)
  else if xName='onkeyup' then
    OnKeyUp:=EV.OnKeyUp(xID)
  else if xName='onkeypress' then
    OnKeyPress:=EV.OnKeyPress(xID)
  else if xName='onmousedown' then
    OnMouseDown:=EV.OnMouseDown(xID)
  else if xName='onmouseup' then
    OnMouseUp:=EV.OnMouseUp(xID)
  else if xName='onmousewheel' then
    OnMouseWheel:=EV.OnMouseWheel(xID)
  else if xName='onmousewheeldown' then
    OnMouseWheelDown:=EV.OnMouseWheelDown(xID)
  else if xName='onmousewheelup' then
    OnMouseWheelUp:=EV.OnMouseWheelUp(xID)
  else if xName='onmousemove' then
    OnMouseMove:=EV.OnMouseMove(xID)
  else if xName='onmousedownmove' then
    OnMouseMove:=EV.OnMouseDownMove(xID)
  else if xName='onmouseupmove' then
    OnMouseMove:=EV.OnMouseUpMove(xID)
  else if xName='ondragdrop' then
    begin
    OnDragDrop:=EV.OnDragDrop(xID);
    OnDragOver:=EV.OnDragOver(xID);
    end
  else if xName='ondropfiles' then
    begin
    FOnDropFilesID:=xID;
    {$ifndef FPC}
    DragAcceptFiles(Handle,xID<>0);
    {$else}
    AllowDropFiles:= xID<>0;
    if AllowDropFiles then
      OnDropFiles:=AcceptFiles
    else
      OnDropFiles:=nil;
    {$endif}
    end
  else if not inherited _SetEvent(xName,xID) then
    begin
    Result:=False;
    Log('Unknown Form event: '+xName+' = '+IntToStr(xID));
    end;
  end;

{$IFDEF RTCWAF_MINIMIZEWITHMAINFORM}
procedure TrtcFormC.WMSIZE(var Msg: TWMSIZE);
  begin
  if msg.SizeType = integer (wsMinimized) then
    begin
    if isMain then
      begin
      WindowState:=wsMinimized;
      FOldRestore:=Application.OnRestore;
      Application.OnRestore:=WinRestore;
      Application.Minimize;
      end;
    end;
  end;

procedure TRtcFormC.WinRestore (Sender: TObject);
  begin
  WindowState:=wsNormal;
  Application.Restore;
  Application.OnRestore:= FOldRestore;
  IF Assigned (FOldRestore) then FOldRestore(sender);
  end;
{$ENDIF}

initialization
RegisterClass(TrtcFormC);
finalization
UnRegisterClass(TrtcFormC);
end.
