{  
  RealThinClient W/A Framework: Client-side Form base class
  Copyright (c) Danijel Tkalcec

  @exclude
}
unit rtc_FormX;

{$include rtcDefs.inc}

interface

{ Some properties and methods are not available in Lazarus and should be avoided when writing
  applications which need to be executed on multiple platforms. Here is the list of things which
  will only work when the Player is compiled using Delphi for Windows:

  * TForm.ObjectMenuItem
  * TForm.WindowMenu
  * TForm.ScreenSnap
  * TForm.SnapBuffer
  * TForm.OldCreateOrder
  * TForm.DefaultMonitor
  * TForm.PrintScale
  * TForm.Scaled
  * TForm.TransparentColor
  * TForm.TransparentColorValue
  * TForm.AlphaBlend
  * TForm.AlphaBlendValue
  }

uses
  rtcTrashcan,
  {$ifdef WINDOWS} Windows, {$endif}
  Classes, SysUtils, Controls,
  Graphics, Forms, Menus,

  rtcInfo,

  rtcLink, rtcGraphicsCli, rtcControlsCli, ExtCtrls;

type
  TrtcFormX=class(TForm, IrtcfComponentX, IRtcFormX)
  protected
    isLoading:boolean;
    isMain:boolean;
    MyMenu:TMainMenu;

    function _Link(const xName:string; xData:TComponent):boolean; virtual;
    function _Process(const xName:string; xData:TRtcValue):boolean; virtual;
    function _Self:TComponent;
    procedure _Paint(Can:TCanvas);

    procedure _BeginLoad; virtual;
    procedure _EndLoad; virtual;
    function _Loading:boolean;

    function GetFormLeft:integer;
    function GetFormTop:integer;
    end;

implementation

{$ifndef FPC}
  {$R *.dfm}
{$endif}

{ TRtcFormX }

function TrtcFormX._Self: TComponent;
  begin
  Result:=self;
  end;

procedure TrtcFormX._Paint(Can: TCanvas);
  begin
  Canvas.Lock;
  try
    Canvas.Handle:=Can.Handle;
    try
      Paint;
    finally
      Canvas.Handle:=0;
      end;
  finally
    Canvas.Unlock;
    end;
  end;

function TrtcFormX._Link(const xName: string; xData: TComponent): boolean;
  var
    h:integer;
    vb,hb:boolean;
  begin
  Result:=True;
  if (xName='activecontrol') then
    begin
    if assigned(xData) and (xData is TWinControl) then
      ActiveControl:=TWinControl(xData)
    else
      ActiveControl:=nil;
    end
  else if (xName='menu') then
    begin
    if assigned(xData) and (xData is TMainMenu) then
      begin
      MyMenu:=TMainMenu(xData);
      if Visible then
        begin
        h:=ClientHeight;
        vb:=VertScrollBar.Visible;
        hb:=HorzScrollBar.Visible;
        VertScrollBar.Visible:=False;
        HorzScrollBar.Visible:=False;
        Menu:=MyMenu;
        ClientHeight:=h;
        VertScrollBar.Visible:=vb;
        HorzScrollBar.Visible:=hb;
        end;
      end
    else
      begin
      MyMenu:=nil;
      if Visible then
        begin
        h:=ClientHeight;
        Menu:=nil;
        ClientHeight:=h;
        end;
      end;
    end
  else if (xName='popupmenu') then
    begin
    if assigned(xData) and (xData is TPopupMenu) then
      PopupMenu:=TPopupMenu(xData)
    else
      PopupMenu:=nil;
    end
  {$ifndef FPC}
  else if (xName='objectmenuitem') then
    begin
    if assigned(xData) and (xData is TMenuItem) then
      ObjectMenuItem:=TMenuItem(xData)
    else
      ObjectMenuItem:=nil;
    end
  else if (xName='windowmenu') then
    begin
    if assigned(xData) and (xData is TMenuItem) then
      WindowMenu:=TMenuItem(xData)
    else
      WindowMenu:=nil;
    end
  {$endif}
  else
    Result:=Rtcf_Link(self,xName,xData);
  end;

function TrtcFormX._Process(const xName: string; xData: TRtcValue):boolean;
  procedure SetBorderIcons;
    var
      rec:TRtcRecord;
      bi:TBorderIcons;
    begin
    if xData.isType=rtc_Record then
      begin
      rec:=xData.asRecord;
      bi:=[];
      if rec.asBoolean['sy'] then Include(bi,biSystemMenu);
      if rec.asBoolean['mi'] then Include(bi,biMinimize);
      if rec.asBoolean['ma'] then Include(bi,biMaximize);
      if rec.asBoolean['he'] then Include(bi,biHelp);
      BorderIcons:=bi;
      end;
    end;
  procedure Do_Show;
    var
      h:integer;
      vb,hb:boolean;
    begin
    if not Visible then
      begin
      if assigned(MyMenu) then
        begin
        h:=ClientHeight;
        vb:=VertScrollBar.Visible;
        hb:=HorzScrollBar.Visible;
        VertScrollBar.Visible:=False;
        HorzScrollBar.Visible:=False;
        Show;
        Menu:=MyMenu;
        ClientHeight:=h;
        VertScrollBar.Visible:=vb;
        HorzScrollBar.Visible:=hb;
        end
      else
        Show;
      BringToFront;
      {$ifndef FPC}
      BringWindowToTop(Handle);
      {$endif}
      end;
    end;
  procedure Do_Hide;
    var
      h:integer;
    begin
    if assigned(MyMenu) then
      begin
      h:=ClientHeight;
      Menu:=nil;
      Hide;
      ClientHeight:=h;
      end
    else
      Hide;
    end;
  begin
  Result:=True;
  if xName='ismain' then
    isMain:=xData.asBoolean
  else if xName='show' then
    Do_Show
  else if xName='hide' then
    Do_Hide
  else if xName='tofront' then
    BringToFront
  else if xName='toback' then
    SendToBack
  else if xName='setfocus' then
    SetFocus
  else if xName='width' then
    ClientWidth:=xData.asInteger
  else if xName='height' then
    ClientHeight:=xData.asInteger
  else if xName='pwidth' then
    Width:=xData.asInteger
  else if xName='pheight' then
    Height:=xData.asInteger
  else if xName='autosize' then
    AutoSize:=xData.asBoolean
  else if xName='autoscroll' then
    AutoScroll:=xData.asBoolean
  else if xName='borderstyle' then
    BorderStyle:=TFormBorderStyle(xData.asInteger)
  else if xName='bordericons' then
    SetBorderIcons
  else if xName='borderwidth' then
    BorderWidth:=TBorderWidth(xData.asInteger)
  else if xName='ctl3d' then
    Ctl3D:=xData.asBoolean
  else if xName='dragkind' then
    DragKind:=TDragKind(xData.asInteger)
  else if xName='dragmode' then
    DragMode:=TDragMode(xData.asInteger)
  else if xName='docksite' then
    DockSite:=xData.asBoolean
  else if xName='formstyle' then
    FormStyle:=TFormStyle(xData.asInteger)
  else if xName='keypreview' then
    KeyPreview:=xData.asBoolean
  else if xName='pixelsperinch' then
    PixelsPerInch:=xData.asInteger
  else if xName='usedockmanager' then
    UseDockManager:=xData.asBoolean
  else if xName='windowstate' then
    WindowState:=TWindowState(xData.asInteger)
  else if xName='position' then
    Position:=TPosition(xData.asInteger)
  else if xName='caption' then
    Caption:=xData.asText
  else if xName='color' then
    Color:=TColor(xData.asInteger)
{$ifndef FPC}
  {$IFDEF IDE_7up}
  else if xName='screensnap' then
    ScreenSnap:=xData.asBoolean
  else if xName='snapbuffer' then
    SnapBuffer:=xData.asInteger
  {$ENDIF}
  else if xName='oldcreateorder' then
    OldCreateOrder:=xData.asBoolean
  else if xName='defaultmonitor' then
    DefaultMonitor:=TDefaultMonitor(xData.asInteger)
  else if xName='printscale' then
    PrintScale:=TPrintScale(xData.asInteger)
  else if xName='scaled' then
    Scaled:=xData.asBoolean
  else if xName='transparentcolor' then
    TransparentColor:=xData.asBoolean
  else if xName='transparentcolorvalue' then
    TransparentColorValue:=TColor(xData.asInteger)
  else if xName='alphablend' then
    AlphaBlend:=xData.asBoolean
  else if xName='alphablendvalue' then
    AlphaBlendValue:=xData.asInteger
{$endif}
  else if Copy(xName,1,5)='font.' then
    Rtcf_ProcessFont(Font,xName,xData)
  else if Copy(xName,1,8)='hscroll.' then
    Rtcf_ProcessControlScrollBar(HorzScrollBar,xName,xData)
  else if Copy(xName,1,8)='vscroll.' then
    Rtcf_ProcessControlScrollBar(VertScrollBar,xName,xData)
  else if not Rtcf_ProcessComponent(self,xName,xData) then
    Result:=False;
  end;

procedure TrtcFormX._BeginLoad;
  var
    a:integer;
  begin
  isLoading:=True;
  {$ifdef fpc}
  PixelsPerInch:=96;
  Font.Name:='Arial';
  Font.Height:=-11;
  Color:=clBtnFace;
  {$endif}
  for a:=0 to ComponentCount-1 do
    if Supports(Components[a],IrtcfComponentX) then
      (Components[a] as IrtcfComponentX)._BeginLoad;

  for a:=0 to ControlCount-1 do
    if Supports(Controls[a],IrtcfComponentX) then
      (Controls[a] as IrtcfComponentX)._BeginLoad;
  end;

procedure TrtcFormX._EndLoad;
  var
    a:integer;
  begin
  isLoading:=False;
  for a:=0 to ComponentCount-1 do
    if Supports(Components[a],IrtcfComponentX) then
      (Components[a] as IrtcfComponentX)._EndLoad;

  for a:=0 to ControlCount-1 do
    if Supports(Controls[a],IrtcfComponentX) then
      (Controls[a] as IrtcfComponentX)._EndLoad;

  FixupTabList;
  end;

function TrtcFormX._Loading: boolean;
  begin
  Result:=isLoading;
  end;

function TrtcFormX.GetFormLeft: integer;
  begin
  Result:=-HorzScrollBar.Position;
  end;

function TrtcFormX.GetFormTop: integer;
  begin
  Result:=-VertScrollBar.Position;
  end;

initialization
RegisterClass(TrtcFormX);
finalization
UnRegisterClass(TrtcFormX);
end.
